@@ -7,7 +7,7 @@
  * Copyright 2005-2006, Anders Broman <anders.broman@ericsson.com>
  * Built from the gsm-map dissector Copyright 2004, Anders Broman <anders.broman@ericsson.com>
  *
- * $Id: packet-camel-template.c 18595 2006-06-28 14:19:08Z kukosa $
+ * $Id: packet-camel-template.c 19427 2006-10-04 16:02:59Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -54,6 +54,8 @@
 #include "packet-gsm_map.h"
 #include "packet-gsm_a.h"
 #include "packet-tcap.h"
+#include "epan/camel-persistentdata.h"
+#include "epan/tcap-persistentdata.h"
 
 #define PNAME  "Camel"
 #define PSNAME "CAMEL"
@@ -62,6 +64,8 @@
 /* Initialize the protocol and registered fields */
 int proto_camel = -1;
 int date_format = 1; /*assume european date format */
+int camel_tap = -1;
+
 static int hf_digit = -1; 
 static int hf_camel_invokeCmd = -1;             /* Opcode */
 static int hf_camel_invokeid = -1;              /* INTEGER */
@@ -87,8 +91,30 @@ static int hf_camel_PDPTypeNumber_ietf = -1;
 static int hf_camel_PDPAddress_IPv4 = -1;
 static int hf_camel_PDPAddress_IPv6 = -1;
 static int hf_camel_cellGlobalIdOrServiceAreaIdFixedLength = -1;
+static int hf_camel_RP_Cause = -1;
+
+/* Used by camel-persistentdata.c */
+int hf_camelsrt_SessionId=-1;
+int hf_camelsrt_RequestNumber=-1;
+int hf_camelsrt_Duplicate=-1;
+int hf_camelsrt_RequestFrame=-1;
+int hf_camelsrt_ResponseFrame=-1;
+int hf_camelsrt_DeltaTime=-1;
+int hf_camelsrt_SessionTime=-1;
+int hf_camelsrt_DeltaTime31=-1;
+int hf_camelsrt_DeltaTime75=-1;
+int hf_camelsrt_DeltaTime65=-1;
+int hf_camelsrt_DeltaTime22=-1;
+int hf_camelsrt_DeltaTime35=-1;
+int hf_camelsrt_DeltaTime80=-1;
+
+static struct camelsrt_info_t * gp_camelsrt_info;
+
 #include "packet-camel-hf.c"
-static guint global_tcap_itu_ssn = 0;
+
+gboolean gcamel_HandleSRT=FALSE;
+extern gboolean gcamel_PersistentSRT;
+extern gboolean gcamel_DisplaySRT;
 
 /* Initialize the subtree pointers */
 static gint ett_camel = -1;
@@ -104,6 +130,7 @@ static gint ett_camel_MSRadioAccessCapability = -1;
 static gint ett_camel_MSNetworkCapability = -1;
 static gint ett_camel_AccessPointName = -1;
 static gint ett_camel_pdptypenumber = -1;
+static gint ett_camel_stat = -1;
 
 #include "packet-camel-ett.c"
 
@@ -122,6 +149,8 @@ static guint8 PDPTypeNumber;
 
 
 static int  dissect_invokeCmd(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int offset);
+static guint8 dissect_RP_cause_ie(tvbuff_t *tvb, guint32 offset, guint len,
+				  proto_tree *tree, int hf_cause_value, guint8 *cause_value);
 
 static const true_false_string camel_extension_value = {
   "No Extension",
@@ -183,6 +212,34 @@ static const value_string camel_number_plan_values[] = {
 
 /* End includes from old" packet-camel.c */
 
+static const value_string camel_RP_Cause_values[] = {
+  { 1, "Unassigned (unallocated) number" },
+  { 8, "Operator determined barring" },
+  { 10, "Call barred" },
+  { 11, "Reserved" },
+  { 21, "Short message transfer rejected" },
+  { 27, "Destination out of order" },
+  { 28, "Unidentified subscriber" },
+  { 29, "Facility Rejected" },
+  { 30, "Unknown subscriber" },
+  { 38, "Network out of order" },
+  { 41, "Temporary failure" },
+  { 42, "Congestion" },
+  { 47, "Resources unavailable, unspecified" },
+  { 50, "Requested facility not subscribed" },
+  { 69, "Requested facility not implemented" },
+  { 81, "Invalid short message transfer reference value" },
+  { 95, "Semantically incorrect message" },
+  { 96, "Invalid mandatory information" },
+  { 97, " Message Type non-existent or not implemented" },
+  { 98, "Message not compatible with short message protocol state" },
+  { 99, "Information element non existent or not implemented" },
+  { 111, "Protocol error, unspecified" },
+  { 127, "Interworking, unspecified" },
+  { 22,"Memory capacity exceeded" },
+  { 0, NULL }
+};
+
 #include "packet-camel-fn.c"
 
 const value_string camel_opr_code_strings[] = {
@@ -252,11 +309,47 @@ char camel_number_to_char(int number)
    return (char) (number + 55 );
 }
 
+/*
+ * 24.011 8.2.5.4
+ */   
+static guint8
+dissect_RP_cause_ie(tvbuff_t *tvb, guint32 offset, guint len,
+		    proto_tree *tree, int hf_cause_value, guint8 *cause_value)
+{
+  guint8	oct;
+  guint32	curr_offset;
+  static char a_bigbuf[1024];
+  
+  curr_offset = offset;
+  oct = tvb_get_guint8(tvb, curr_offset);
+
+  *cause_value = oct & 0x7f; 
+  
+  other_decode_bitfield_value(a_bigbuf, oct, 0x7f, 8);
+  proto_tree_add_uint_format(tree, hf_cause_value,
+			     tvb, curr_offset, 1, *cause_value,
+			     "%s : %s",
+			     a_bigbuf,
+			     val_to_str(*cause_value, camel_RP_Cause_values, 
+					"Unknown Cause (%u), treated as (41) \"Temporary failure\" for MO-SMS or (111) \"Protocol error,unspecified\" for MT-SMS"));
+  curr_offset++;
+  
+  if ((oct & 0x80)) {
+    oct = tvb_get_guint8(tvb, curr_offset);
+    proto_tree_add_uint_format(tree, hf_cause_value,
+			       tvb, curr_offset, 1, oct,
+			       "Diagnostic : %u", oct);
+    curr_offset++;
+  }
+  return(curr_offset - offset);
+}
+
 static guint32 opcode=0;
 
 static int
 dissect_camel_Opcode(gboolean implicit_tag _U_, tvbuff_t *tvb, int offset, packet_info *pinfo _U_, proto_tree *tree, int hf_index) {
   offset = dissect_ber_integer(FALSE, pinfo, tree, tvb, offset, hf_index, &opcode);
+  gp_camelsrt_info->opcode=opcode;
 
   if (check_col(pinfo->cinfo, COL_INFO)){
     /* Add Camel Opcode to INFO column */
@@ -360,7 +453,7 @@ static int dissect_invokeData(packet_info *pinfo, proto_tree *tree, tvbuff_t *tv
     /* ContinueSMS: no arguments - do nothing */
     break;
   case 66: /*ReleaseSMS*/
-    offset=dissect_camel_ReleaseSMSArg(FALSE, tvb, offset, pinfo, tree, -1);
+    offset=dissect_camel_ReleaseSMSArg(FALSE, tvb, offset, pinfo, tree, hf_camel_RP_Cause);
     break;
   case 67: /*ResetTimerSMS*/
     offset=dissect_camel_ResetTimerSMSArg(FALSE, tvb, offset, pinfo, tree, -1);
@@ -564,18 +657,50 @@ static const ber_choice_t camelPDU_choice[] = {
 static guint8 camel_pdu_type = 0;
 static guint8 camel_pdu_size = 0;
 
+static void dissect_camelext_CAPGPRSReferenceNumber(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree) {
+  proto_item    *item=NULL;
+  proto_tree    *tree=NULL;
+  proto_item    *camel_item=NULL;
+  proto_tree    *camel_tree=NULL;
+
+
+  if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "Camel");
+  }
+
+  /* create display subtree for the protocol */
+  if(parent_tree){
+    camel_item = proto_tree_add_item(parent_tree, proto_camel, tvb, 0, -1, FALSE);
+    camel_tree = proto_item_add_subtree(camel_item, ett_camel);
+  }
+  /* create display subtree for the protocol */
+  if(camel_tree){
+    item = proto_tree_add_text(camel_tree, tvb, 0, -1, "GPRS Reference Number");
+    tree = proto_item_add_subtree(item, ett_camel_CAPGPRSReferenceNumber);
+  }
+  dissect_camel_CAPGPRSReferenceNumber(FALSE, tvb, 0, pinfo, tree, -1);
+}
+
 static int
 dissect_camel_camelPDU(gboolean implicit_tag _U_, tvbuff_t *tvb, int offset, packet_info *pinfo _U_, proto_tree *tree, int hf_index) {
 
   char *version_ptr;
+  struct tcap_private_t * p_private_tcap;
 
   opcode = 0;
   application_context_version = 0;
   if (pinfo->private_data != NULL){
-    version_ptr = strrchr(pinfo->private_data,'.');
-    if (version_ptr) {
-      application_context_version = atoi(version_ptr+1);
+    p_private_tcap=pinfo->private_data; 
+    
+    if (p_private_tcap->acv==TRUE ){
+      version_ptr = strrchr(p_private_tcap->oid,'.');
+      if (version_ptr)
+	application_context_version = atoi(version_ptr+1);
     }
+    gp_camelsrt_info->tcap_context=p_private_tcap->context; 
+    if (p_private_tcap->context)
+      gp_camelsrt_info->tcap_session_id
+	= ( (struct tcaphash_context_t *) (p_private_tcap->context))->session_id;
   }
 
   camel_pdu_type = tvb_get_guint8(tvb, offset)&0x0f;
@@ -598,6 +723,8 @@ dissect_camel(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 {
   proto_item    *item=NULL;
   proto_tree    *tree=NULL;
+  proto_item  *stat_item=NULL;
+  proto_tree  *stat_tree=NULL;
 
   if (check_col(pinfo->cinfo, COL_PROTOCOL)) {
     col_set_str(pinfo->cinfo, COL_PROTOCOL, "Camel");
@@ -608,9 +735,21 @@ dissect_camel(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
      item = proto_tree_add_item(parent_tree, proto_camel, tvb, 0, -1, FALSE);
      tree = proto_item_add_subtree(item, ett_camel);
   }
-
+  /* camelsrt reset counter, and initialise global pointer
+     to store service response time related data */
+  gp_camelsrt_info=camelsrt_razinfo();
   dissect_camel_camelPDU(FALSE, tvb, 0, pinfo, tree, -1);
-
+  
+  /* If a Tcap context is associated to this transaction */
+  if (gcamel_HandleSRT &&
+      gp_camelsrt_info->tcap_context ) {
+    if (gcamel_DisplaySRT && tree) {
+      stat_item = proto_tree_add_text(tree, tvb, 0, 0, "Stat");
+      stat_tree = proto_item_add_subtree(stat_item, ett_camel_stat);
+    }
+    camelsrt_call_matching(tvb, pinfo, stat_tree, gp_camelsrt_info);
+    tap_queue_packet(camel_tap, pinfo, gp_camelsrt_info);
+  }
 }
 
 /*--- proto_reg_handoff_camel ---------------------------------------*/
@@ -631,10 +770,18 @@ static void range_add_callback(guint32 ssn)
 void proto_reg_handoff_camel(void) {
 
   static int camel_prefs_initialized = FALSE;
-  
   if (!camel_prefs_initialized) {
     camel_prefs_initialized = TRUE;
     camel_handle = create_dissector_handle(dissect_camel, proto_camel);
+
+    register_ber_oid_dissector_handle("0.4.0.0.1.0.50.0",camel_handle, proto_camel, "itu-t(0) identified-organization(4) etsi(0) mobileDomain(0) gsm-Network|umts-Network(1) applicationContext(0) cap-gsmssf-to-gsmscf(50) version1(0)" );
+    register_ber_oid_dissector_handle("0.4.0.0.1.0.50.1",camel_handle, proto_camel, "itu-t(0) identified-organization(4) etsi(0) mobileDomain(0) gsm-Network|umts-Network(1) applicationContext(0) cap-gsmssf-to-gsmscf(50) version2(1)" );
+    register_ber_oid_dissector_handle("0.4.0.0.1.0.51.1",camel_handle, proto_camel, "itu-t(0) identified-organization(4) etsi(0) mobileDomain(0) gsm-Network|umts-Network(1) applicationContext(0) cap-assist-handoff-gsmssf-to-gsmscf(51) version2(1)" );
+    register_ber_oid_dissector_handle("0.4.0.0.1.0.52.1",camel_handle, proto_camel, "itu-t(0) identified-organization(4) etsi(0) mobileDomain(0) gsm-Network|umts-Network(1) applicationContext(0) cap-gsmSRF-to-gsmscf(52) version2(1)" );
+    register_ber_oid_dissector_handle("0.4.0.0.1.21.3.50",camel_handle, proto_camel, "itu-t(0) identified-organization(4) etsi(0) mobileDomain(0) gsm-Network(1) cAP3OE(21) ac(3) id-ac-CAP-gprsSSF-gsmSCF-AC(50)" );
+    register_ber_oid_dissector_handle("0.4.0.0.1.21.3.61",camel_handle, proto_camel, "itu-t(0) identified-organization(4) etsi(0) mobileDomain(0) gsm-Network(1) cAP3OE(21) ac(3) id-ac-cap3-sms-AC(61)" );
+
+    register_ber_oid_dissector("0.4.0.0.1.1.5.2", dissect_camelext_CAPGPRSReferenceNumber, proto_camel, "itu-t(0) identified-organization(4) etsi(0) mobileDomain(0) gsm-Network(1) abstractSyntax(1) cap-GPRS-ReferenceNumber(5) version3(2)");
   } else {
     range_foreach(ssn_range, range_delete_callback);
   }
@@ -722,10 +869,94 @@ void proto_register_camel(void) {
       { "PDPAddress IPv6",  "camel.PDPAddress_IPv6",
 	  FT_IPv6, BASE_NONE, NULL, 0,
 	  "IPAddress IPv6", HFILL }},
-    { &hf_camel_cellGlobalIdOrServiceAreaIdFixedLength,
+   { &hf_camel_cellGlobalIdOrServiceAreaIdFixedLength,
       { "CellGlobalIdOrServiceAreaIdFixedLength", "camel.CellGlobalIdOrServiceAreaIdFixedLength",
         FT_BYTES, BASE_HEX, NULL, 0,
         "LocationInformationGPRS/CellGlobalIdOrServiceAreaIdOrLAI", HFILL }},
+  { &hf_camel_RP_Cause,
+      { "RP Cause",  "camel.RP_Cause",
+      FT_UINT8, BASE_DEC, NULL, 0,
+	"RP Cause Value", HFILL }},
+
+  /* Camel Service Response Time */
+    { &hf_camelsrt_SessionId,
+      { "Session Id",
+        "camel.srt.session_id",
+        FT_UINT32, BASE_DEC, NULL, 0x0,
+        "", HFILL }
+    },
+    { &hf_camelsrt_RequestNumber,
+      { "Request Number",
+        "camel.srt.request_number",
+        FT_UINT64, BASE_DEC, NULL, 0x0,
+        "", HFILL }
+    },
+    { &hf_camelsrt_Duplicate,
+      { "Request Duplicate",
+        "camel.srt.duplicate",
+        FT_UINT32, BASE_DEC, NULL, 0x0,
+        "", HFILL }
+    },
+    { &hf_camelsrt_RequestFrame,
+      { "Requested Frame",
+        "camel.srt.reqframe",
+        FT_FRAMENUM, BASE_NONE, NULL, 0x0,
+        "SRT Request Frame", HFILL }
+    },
+    { &hf_camelsrt_ResponseFrame,
+      { "Response Frame",
+        "camel.srt.rspframe",
+        FT_FRAMENUM, BASE_NONE, NULL, 0x0,
+        "SRT Response Frame", HFILL }
+    },
+    { &hf_camelsrt_DeltaTime,
+      { "Service Response Time",
+        "camel.srt.deltatime",
+        FT_RELATIVE_TIME, BASE_NONE, NULL, 0x0,
+        "DeltaTime between Request and Response", HFILL }
+    },
+    { &hf_camelsrt_SessionTime,
+      { "Session duration",
+        "camel.srt.sessiontime",
+        FT_RELATIVE_TIME, BASE_NONE, NULL, 0x0,
+        "Duration of the TCAP session", HFILL }
+    },
+    { &hf_camelsrt_DeltaTime31,
+      { "Service Response Time",
+        "camel.srt.deltatime31",
+        FT_RELATIVE_TIME, BASE_NONE, NULL, 0x0,
+        "DeltaTime between InitialDP and Continue", HFILL }
+    },
+    { &hf_camelsrt_DeltaTime65,
+      { "Service Response Time",
+        "camel.srt.deltatime65",
+        FT_RELATIVE_TIME, BASE_NONE, NULL, 0x0,
+        "DeltaTime between InitialDPSMS and ContinueSMS", HFILL }
+    },
+    { &hf_camelsrt_DeltaTime75,
+      { "Service Response Time",
+        "camel.srt.deltatime75",
+        FT_RELATIVE_TIME, BASE_NONE, NULL, 0x0,
+        "DeltaTime between InitialDPGPRS and ContinueGPRS", HFILL }
+    },
+    { &hf_camelsrt_DeltaTime35,
+      { "Service Response Time",
+        "camel.srt.deltatime35",
+        FT_RELATIVE_TIME, BASE_NONE, NULL, 0x0,
+        "DeltaTime between ApplyCharginReport and ApplyCharging", HFILL }
+    },
+    { &hf_camelsrt_DeltaTime22,
+      { "Service Response Time",
+        "camel.srt.deltatime22",
+        FT_RELATIVE_TIME, BASE_NONE, NULL, 0x0,
+        "DeltaTime between EventReport(Disconnect) and Release Call", HFILL }
+    },
+  { &hf_camelsrt_DeltaTime80,
+      { "Service Response Time",
+        "camel.srt.deltatime80",
+        FT_RELATIVE_TIME, BASE_NONE, NULL, 0x0,
+        "DeltaTime between EventReportGPRS and ContinueGPRS", HFILL }
+    },
 
 #ifdef REMOVED
 #endif
@@ -747,24 +978,16 @@ void proto_register_camel(void) {
 	&ett_camel_MSNetworkCapability,
 	&ett_camel_AccessPointName,
 	&ett_camel_pdptypenumber,
+    &ett_camel_stat,
+
 #include "packet-camel-ettarr.c"
   };
-
   /* Register protocol */
   proto_camel = proto_register_protocol(PNAME, PSNAME, PFNAME);
 
   proto_register_field_array(proto_camel, hf, array_length(hf));
   proto_register_subtree_array(ett, array_length(ett));
 
-  register_ber_oid_dissector_handle("0.4.0.0.1.0.50.1",camel_handle, proto_camel, "itu-t(0) identified-organization(4) etsi(0) mobileDomain(0) gsm-Network|umts-Network(1) applicationContext(0) cap-gsmssf-to-gsmscf(50) version2(1)" );
-  register_ber_oid_dissector_handle("0.4.0.0.1.0.51.1",camel_handle, proto_camel, "itu-t(0) identified-organization(4) etsi(0) mobileDomain(0) gsm-Network|umts-Network(1) applicationContext(0) cap-assist-handoff-gsmssf-to-gsmscf(51) version2(1)" );
-  register_ber_oid_dissector_handle("0.4.0.0.1.0.52.1",camel_handle, proto_camel, "itu-t(0) identified-organization(4) etsi(0) mobileDomain(0) gsm-Network|umts-Network(1) applicationContext(0) cap-gsmSRF-to-gsmscf(52) version2(1)" );
-  register_ber_oid_dissector_handle("0.4.0.0.1.21.3.50",camel_handle, proto_camel, "itu-t(0) identified-organization(4) etsi(0) mobileDomain(0) gsm-Network(1) cAP3OE(21) ac(3) id-ac-CAP-gprsSSF-gsmSCF-AC(50)" );
-
-  add_oid_str_name("0.4.0.0.1.1.5.2","iitu-t(0) identified-organization(4) etsi(0) mobileDomain(0) gsm-Network(1) abstractSyntax(1) cap-GPRS-ReferenceNumber(5) version3(2)");
-
-    
-
   /* Register our configuration options, particularly our ssn:s */
   /* Set default SSNs */
   range_convert_str(&global_ssn_range, "6-9", MAX_SSN);
@@ -781,5 +1004,19 @@ void proto_register_camel(void) {
     "TCAP SSNs",
     "TCAP Subsystem numbers used for Camel",
     &global_ssn_range, MAX_SSN);
+
+  prefs_register_bool_preference(camel_module, "srt",
+				 "Service Response Time Analyse",
+				 "Activate the analyse for Response Time",
+				 &gcamel_HandleSRT);
+
+  prefs_register_bool_preference(camel_module, "persistentsrt",
+				 "Persistent stats for SRT",
+				 "Statistics for Response Time",
+				 &gcamel_PersistentSRT);
+  
+  /* Routine for statistic */ 
+  register_init_routine(&camelsrt_init_routine);
+  camel_tap=register_tap(PSNAME);
 }
 
