@@ -1,6 +1,6 @@
 /* dumpcap.c
  *
- * $Id: dumpcap.c 18235 2006-05-28 20:28:20Z etxrab $
+ * $Id: dumpcap.c 19624 2006-10-20 06:24:56Z jake $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -193,13 +193,17 @@ cmdarg_err_cont(const char *fmt, ...)
 BOOL WINAPI ConsoleCtrlHandlerRoutine(DWORD dwCtrlType)
 {
     g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_INFO,
-        "Console: Ctrl+C");
+        "Console: Control signal");
     g_log(LOG_DOMAIN_CAPTURE_CHILD, G_LOG_LEVEL_DEBUG,
-        "Console: Ctrl+C CtrlType: %u", dwCtrlType);
+        "Console: Control signal, CtrlType: %u", dwCtrlType);
 
-    capture_loop_stop();
-
-    return TRUE;
+    /* Keep capture running if we're a service and a user logs off */
+    if (capture_child || (dwCtrlType != CTRL_LOGOFF_EVENT)) {
+        capture_loop_stop();
+        return TRUE;
+    } else {
+        return FALSE;
+    }
 }
 #endif
 
@@ -270,12 +274,11 @@ main(int argc, char *argv[])
 
   /* Assemble the compile-time version information string */
   comp_info_str = g_string_new("Compiled ");
-  g_string_append(comp_info_str, "with ");
-  get_compiled_version_info(comp_info_str);
+  get_compiled_version_info(comp_info_str, NULL);
 
   /* Assemble the run-time version information string */
   runtime_info_str = g_string_new("Running ");
-  get_runtime_version_info(runtime_info_str);
+  get_runtime_version_info(runtime_info_str, NULL);
 
   /* the default_log_handler will use stdout, which makes trouble in */
   /* capture child mode, as it uses stdout for it's sync_pipe */
@@ -379,6 +382,7 @@ main(int argc, char *argv[])
   if (argc != 0) {
     /*
      * Extra command line arguments were specified; complain.
+     * XXX - interpret as capture filter, as tcpdump and tshark do?
      */
     cmdarg_err("Invalid argument: %s", argv[0]);
     arg_error = TRUE;
@@ -624,3 +628,4 @@ signal_pipe_check_running(void)
 
 
 const char *netsnmp_get_version(void) { return ""; }
+
