@@ -1,7 +1,7 @@
 /* Edit capture files.  We can delete packets, adjust timestamps, or
  * simply convert from one format to another format.
  *
- * $Id: editcap.c 18800 2006-07-27 17:53:29Z gerald $
+ * $Id: editcap.c 19676 2006-10-24 22:21:04Z gerald $
  *
  * Originally written by Richard Sharpe.
  * Improved by Guy Harris.
@@ -476,7 +476,7 @@ int main(int argc, char *argv[])
 		struct tm starttm;
 
 		memset(&starttm,0,sizeof(struct tm));
-
+		
 		if(!strptime(optarg,"%F %T",&starttm)) {
 			fprintf(stderr, "editcap: \"%s\" isn't a valid time format\n\n",
 					optarg);
@@ -484,6 +484,8 @@ int main(int argc, char *argv[])
 		}
 
 		check_startstop = TRUE;
+		starttm.tm_isdst = -1;
+		
 		starttime = mktime(&starttm);
 		break;
 	}
@@ -499,6 +501,7 @@ int main(int argc, char *argv[])
 			exit(1);
 		}
 		check_startstop = TRUE;
+		stoptm.tm_isdst = -1;
 		stoptime = mktime(&stoptm);
 		break;
 	}
