@@ -1,7 +1,7 @@
 /* addr_resolv.c
  * Routines for network object lookup
  *
- * $Id: addr_resolv.c 18234 2006-05-28 19:49:07Z etxrab $
+ * $Id: addr_resolv.c 19075 2006-08-29 16:54:39Z jake $
  *
  * Laurent Deniel <laurent.deniel@free.fr>
  *
@@ -585,7 +585,7 @@ static int fgetline(char **buf, int *size, FILE *fp)
     return -1;
 
   len = 0;
-  while ((c = getc(fp)) != EOF && c != '\n') {
+  while ((c = getc(fp)) != EOF && c != '\r' && c != '\n') {
     if (len+1 >= *size) {
       if ((*buf = g_realloc(*buf, *size += BUFSIZ)) == NULL)
 	return -1;
