@@ -1,7 +1,7 @@
 /* column-utils.c
  * Routines for column utilities.
  *
- * $Id: column-utils.c 18578 2006-06-26 20:46:15Z ulfl $
+ * $Id: column-utils.c 19539 2006-10-15 06:10:59Z sahlberg $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -972,6 +972,17 @@ col_set_port(packet_info *pinfo, int col, gboolean is_res, gboolean is_src)
     pinfo->cinfo->col_expr_val[col][COL_MAX_LEN - 1] = '\0';
     break;
 
+  case PT_USB:
+    /* XXX - resolve USB endpoint numbers */
+    g_snprintf(pinfo->cinfo->col_buf[col], COL_MAX_LEN, "0x%08x", port);
+    if (is_src)
+      strcpy(pinfo->cinfo->col_expr[col], "usb.src.endpoint");
+    else
+      strcpy(pinfo->cinfo->col_expr[col], "usb.dst.endpoint");
+    g_snprintf(pinfo->cinfo->col_expr_val[col], COL_MAX_LEN, "0x%08x", port);
+    pinfo->cinfo->col_expr_val[col][COL_MAX_LEN - 1] = '\0';
+    break;
+
   default:
     break;
   }
