@@ -1,7 +1,7 @@
 /* packet-atm.c
  * Routines for ATM packet disassembly
  *
- * $Id: packet-atm.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-atm.c 19189 2006-09-11 12:03:29Z martinm $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -42,6 +42,7 @@ static int proto_atm = -1;
 static int hf_atm_aal = -1;
 static int hf_atm_vpi = -1;
 static int hf_atm_vci = -1;
+static int hf_atm_cid = -1;
 static int proto_atm_lane = -1;
 static int proto_ilmi = -1;
 static int proto_aal1 = -1;
@@ -66,6 +67,7 @@ static dissector_handle_t llc_handle;
 static dissector_handle_t sscop_handle;
 static dissector_handle_t lane_handle;
 static dissector_handle_t ilmi_handle;
+static dissector_handle_t fp_handle;
 static dissector_handle_t data_handle;
 
 /*
@@ -915,6 +917,7 @@ update_crc(guint32 crc_accum, const guint8 *data_blk_ptr, int data_blk_size)
 
 static void
 dissect_reassembled_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
+    proto_item *atm_ti,
     proto_tree *atm_tree, gboolean truncated)
 {
   guint        length, reported_length;
@@ -1014,7 +1017,8 @@ dissect_reassembled_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
       proto_tree_add_text(atm_tree, tvb, 0, 0, "Cells: %u",
 		reported_length/48);
     }
-    if (length >= reported_length) {
+    if (pinfo->pseudo_header->atm.aal == AAL_5 &&
+        length >= reported_length) {
       /*
        * XXX - what if the packet is truncated?  Can that happen?
        * What if you capture with Windows Sniffer on an ATM link
@@ -1051,23 +1055,23 @@ dissect_reassembled_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
           if (tree) {
             if (pad_length > 0) {
               proto_tree_add_text(atm_tree, tvb, aal5_length, pad_length,
-  		"Padding");
+                                  "Padding");
             }
             proto_tree_add_text(atm_tree, tvb, length - 8, 1, "AAL5 UU: 0x%02x",
-		tvb_get_guint8(tvb, length - 8));
+                                tvb_get_guint8(tvb, length - 8));
             proto_tree_add_text(atm_tree, tvb, length - 7, 1, "AAL5 CPI: 0x%02x",
-		tvb_get_guint8(tvb, length - 7));
+                                tvb_get_guint8(tvb, length - 7));
             proto_tree_add_text(atm_tree, tvb, length - 6, 2, "AAL5 len: %u",
-		aal5_length);
+                                aal5_length);
             crc = tvb_get_ntohl(tvb, length - 4);
             calc_crc = update_crc(0xFFFFFFFF, tvb_get_ptr(tvb, 0, length),
-		length);
+                                  length);
             proto_tree_add_text(atm_tree, tvb, length - 4, 4,
-		"AAL5 CRC: 0x%08X (%s)", crc,
-		(calc_crc == 0xC704DD7B) ? "correct" : "incorrect");
-	  }
+                                "AAL5 CRC: 0x%08X (%s)", crc,
+                                (calc_crc == 0xC704DD7B) ? "correct" : "incorrect");
+          }
           next_tvb = tvb_new_subset(tvb, 0, aal5_length, aal5_length);
-	}
+        }
       }
     }
   }
@@ -1106,6 +1110,36 @@ dissect_reassembled_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
     }
     break;
 
+  case AAL_2:
+    switch (pinfo->pseudo_header->atm.type) {
+      case TRAF_UMTS_FP:
+        /* Skip first 4 bytes of message
+        - side
+        - length
+        - UUI
+        Ignoring for now... */
+        proto_tree_add_uint(atm_tree, hf_atm_cid, tvb, 0, 0,
+                            pinfo->pseudo_header->atm.aal2_cid);
+        proto_item_append_text(atm_ti, " (vpi=%u vci=%u cid=%u)",
+                               pinfo->pseudo_header->atm.vpi,
+                               pinfo->pseudo_header->atm.vci,
+                               pinfo->pseudo_header->atm.aal2_cid);
+
+        next_tvb = tvb_new_subset(tvb, 4,
+                                  tvb_length_remaining(tvb, 4),
+                                  tvb_length_remaining(tvb, 4));
+        call_dissector(fp_handle, next_tvb, pinfo, tree);
+        break;
+
+      default:
+        if (tree) {
+          /* Dump it as raw data. */
+          call_dissector(data_handle, next_tvb, pinfo, tree);
+          break;
+        }
+    }
+    break;
+
   default:
     if (tree) {
       /* Dump it as raw data. */
@@ -1523,7 +1557,7 @@ dissect_atm_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
     gboolean truncated)
 {
   proto_tree   *atm_tree = NULL;
-  proto_item   *ti = NULL;
+  proto_item   *atm_ti = NULL;
 
   if (check_col(pinfo->cinfo, COL_PROTOCOL))
     col_set_str(pinfo->cinfo, COL_PROTOCOL, "ATM");
@@ -1560,8 +1594,8 @@ dissect_atm_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   }
 
   if (tree) {
-    ti = proto_tree_add_protocol_format(tree, proto_atm, tvb, 0, 0, "ATM");
-    atm_tree = proto_item_add_subtree(ti, ett_atm);
+    atm_ti = proto_tree_add_protocol_format(tree, proto_atm, tvb, 0, 0, "ATM");
+    atm_tree = proto_item_add_subtree(atm_ti, ett_atm);
 
     switch (pinfo->pseudo_header->atm.channel) {
 
@@ -1590,11 +1624,11 @@ dissect_atm_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   }
   if (pinfo->pseudo_header->atm.flags & ATM_RAW_CELL) {
     /* This is a single cell, with the cell header at the beginning. */
-    proto_item_set_len(ti, 5);
+    proto_item_set_len(atm_ti, 5);
     dissect_atm_cell(tvb, pinfo, tree, atm_tree);
   } else {
     /* This is a reassembled PDU. */
-    dissect_reassembled_pdu(tvb, pinfo, tree, atm_tree, truncated);
+    dissect_reassembled_pdu(tvb, pinfo, tree, atm_tree, atm_ti, truncated);
   }
 }
 
@@ -1625,6 +1659,11 @@ proto_register_atm(void)
 		{ &hf_atm_vci,
 		{ "VCI",		"atm.vci", FT_UINT16, BASE_DEC, NULL, 0x0,
 			"", HFILL }},
+
+		{ &hf_atm_cid,
+		{ "CID",		"atm.cid", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"", HFILL }},
+
 	};
 	static gint *ett[] = {
 		&ett_atm,
@@ -1673,6 +1712,7 @@ proto_reg_handoff_atm(void)
 	lane_handle = find_dissector("lane");
 	ilmi_handle = find_dissector("ilmi");
 	data_handle = find_dissector("data");
+	fp_handle = find_dissector("fp");
 
 	atm_handle = create_dissector_handle(dissect_atm, proto_atm);
 	dissector_add("wtap_encap", WTAP_ENCAP_ATM_PDUS, atm_handle);
