@@ -2,7 +2,7 @@
  * Routines for BACnet (APDU) dissection
  * Copyright 2004, Herbert Lischka <lischka@kieback-peter.de>, Berlin
  *
- * $Id: packet-bacapp.h 18758 2006-07-18 15:54:03Z gerald $
+ * $Id: packet-bacapp.h 19410 2006-10-03 18:11:55Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -1483,7 +1483,6 @@ static guint
 fDeviceObjectPropertyReference (tvbuff_t *tvb, proto_tree *tree, guint offset);
 #endif
 
-#if 0
 /**
  * BACnetDeviceObjectReference ::= SEQUENCE {
  *  deviceIdentifier    [0] BACnetObjectIdentifier OPTIONAL,
@@ -1496,7 +1495,6 @@ fDeviceObjectPropertyReference (tvbuff_t *tvb, proto_tree *tree, guint offset);
  */
 static guint
 fDeviceObjectReference (tvbuff_t *tvb, proto_tree *tree, guint offset);
-#endif
 
 #if 0
 /**
@@ -2011,19 +2009,6 @@ fSelectionCriteria (tvbuff_t *tvb, proto_tree *tree, guint offset);
 static guint
 fObjectSelectionCriteria (tvbuff_t *tvb, proto_tree *subtree, guint offset);
 
-/**
- * ObjectSpecifier ::= CHOICE {
- * 	objectType	[0] BACnetObjectType,
- *  objectIdentifier	[1] BACnetObjectIdentifier
- * }
- * @param tvb
- * @param tree
- * @param offset
- * @return modified offset
- */
-static guint
-fObjectSpecifier (tvbuff_t *tvb, proto_tree *tree, guint offset);
-
 /**
  * BACnet-Error ::= SEQUENCE {
  *    error-class ENUMERATED {},
