@@ -1,7 +1,7 @@
 /* packet-bpdu.c
  * Routines for BPDU (Spanning Tree Protocol) disassembly
  *
- * $Id: packet-bpdu.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-bpdu.c 19211 2006-09-12 12:58:16Z jake $
  *
  * Copyright 1999 Christophe Tronche <ch.tronche@computer.org>
  *
@@ -35,6 +35,7 @@
 #include <epan/llcsaps.h>
 #include <epan/ppptypes.h>
 #include <epan/chdlctypes.h>
+#include <epan/etypes.h>
 #include <epan/addr_resolv.h>
 
 /* Offsets of fields within a BPDU */
@@ -802,4 +803,5 @@ proto_reg_handoff_bpdu(void)
   dissector_add("llc.dsap", SAP_BPDU, bpdu_handle);
   dissector_add("chdlctype", CHDLCTYPE_BPDU, bpdu_handle);
   dissector_add("llc.cisco_pid", 0x010b, bpdu_handle);
+  dissector_add("ethertype", ETHERTYPE_STP, bpdu_handle);
 }
