@@ -2,7 +2,7 @@
  * Routines for dcerpc endpoint mapper dissection
  * Copyright 2001, Todd Sabin <tas@webspan.net>
  *
- * $Id: packet-dcerpc-epm.c 18939 2006-08-17 19:09:41Z ulfl $
+ * $Id: packet-dcerpc-epm.c 19175 2006-09-07 20:33:02Z ulfl $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -266,7 +266,9 @@ epm_dissect_uuid (tvbuff_t *tvb, int offset,
 #define PROTO_ID_IP		0x09
 #define PROTO_ID_RPC_CL		0x0a
 #define PROTO_ID_RPC_CO		0x0b
+#define PROTO_ID_SPX		0x0c    /* from DCOM spec (is this correct?) */
 #define PROTO_ID_UUID		0x0d
+#define PROTO_ID_IPX		0x0e    /* from DCOM spec (is this correct?) */
 #define PROTO_ID_NAMED_PIPES	0x0f
 #define PROTO_ID_NAMED_PIPES_2	0x10
 #define PROTO_ID_NETBIOS	0x11
@@ -297,7 +299,9 @@ static const value_string proto_id_vals[] = {
 	{ PROTO_ID_IP,			"DOD IP"},
 	{ PROTO_ID_RPC_CL,		"RPC connectionless protocol"},
 	{ PROTO_ID_RPC_CO,		"RPC connection-oriented protocol"},
+	{ PROTO_ID_SPX,		        "SPX?"},
 	{ PROTO_ID_UUID,		"UUID"},
+	{ PROTO_ID_IPX,		        "IPX?"},
 	{ PROTO_ID_NAMED_PIPES,		"Named Pipes"}, 
 	{ PROTO_ID_NAMED_PIPES_2,	"Named Pipes"},
 	{ PROTO_ID_NETBIOS,		"NetBIOS"},
@@ -409,6 +413,10 @@ epm_dissect_tower_data (tvbuff_t *tvb, int offset,
 
         switch(proto_id){
 
+        case PROTO_ID_UUID:
+            /* XXX - is this big or little endian? */
+            proto_tree_add_item(tr, hf_epm_ver_min, tvb, offset, 2, FALSE);
+            break;
         case PROTO_ID_TCP: /* this one is always big endian */
             proto_tree_add_item(tr, hf_epm_proto_tcp_port, tvb, offset, 2, FALSE);
             proto_item_append_text(tr, "TCP Port:%d", tvb_get_ntohs(tvb, offset));
@@ -430,7 +438,8 @@ epm_dissect_tower_data (tvbuff_t *tvb, int offset,
 
 	case PROTO_ID_RPC_CL:
 	    proto_item_append_text(tr, "RPC connectionless protocol");
-	    /* XXX - two (zero) bytes still undecoded, don't know what it is */
+            /* XXX - is this big or little endian? */
+            proto_tree_add_item(tr, hf_epm_ver_min, tvb, offset, 2, FALSE);
 	    break;
 
         case PROTO_ID_NAMED_PIPES: /* \\PIPE\xxx   named pipe */
