@@ -5,7 +5,7 @@
  * This information is based off the released idl files from opengroup.
  * ftp://ftp.opengroup.org/pub/dce122/dce/src/file.tar.gz file/flserver/fldb_proc.idl
  *
- * $Id: packet-dcerpc-fldb.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-dcerpc-fldb.c 18931 2006-08-16 20:55:21Z ulfl $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -1967,10 +1967,10 @@ proto_register_fldb (void)
      {"Principal Name", "vldbentry.siteprincipal", FT_STRING, BASE_NONE, NULL,
       0x0, "", HFILL}},
     {&hf_fldb_vldbentry_siteowner,
-     {"Site Owner", "vldbentry.siteowner", FT_STRING, BASE_NONE, NULL, 0x0,
+     {"Site Owner", "vldbentry.siteowner", FT_GUID, BASE_NONE, NULL, 0x0,
       "UUID", HFILL}},
     {&hf_fldb_vldbentry_siteobjid,
-     {"Site Object ID", "vldbentry.siteobjid", FT_STRING, BASE_NONE, NULL,
+     {"Site Object ID", "vldbentry.siteobjid", FT_GUID, BASE_NONE, NULL,
       0x0, "UUID", HFILL}},
     {&hf_fldb_vldbentry_volids_high,
      {"VolIDs high", "vldbentry.volidshigh", FT_UINT32, BASE_DEC, NULL, 0x0,
@@ -2103,10 +2103,10 @@ proto_register_fldb (void)
     {&hf_fldb_spare5,
      {"spare5", "fldb.spare5", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL}},
     {&hf_fldb_uuid_objid,
-     {"objid", "fldb.uuid_objid", FT_STRING, BASE_NONE, NULL, 0x0, "UUID",
+     {"objid", "fldb.uuid_objid", FT_GUID, BASE_NONE, NULL, 0x0, "UUID",
       HFILL}},
     {&hf_fldb_uuid_owner,
-     {"owner", "fldb.uuid_owner", FT_STRING, BASE_NONE, NULL, 0x0, "UUID",
+     {"owner", "fldb.uuid_owner", FT_GUID, BASE_NONE, NULL, 0x0, "UUID",
       HFILL}},
     {&hf_fldb_volid_high,
      {"volid high", "fldb.volid_high", FT_UINT32, BASE_DEC, NULL, 0x0, "",
