@@ -1,7 +1,7 @@
 /* packet-dcom.c
  * Routines for DCOM generics
  *
- * $Id: packet-dcom.c 18939 2006-08-17 19:09:41Z ulfl $
+ * $Id: packet-dcom.c 19597 2006-10-18 19:42:10Z jake $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -87,6 +87,7 @@
 #include <epan/emem.h>
 #include <epan/addr_resolv.h>
 #include <epan/inet_aton.h>
+#include <isprint.h>
 #include "packet-dcerpc.h"
 #include "packet-dcom.h"
 #include "prefs.h"
@@ -118,7 +119,7 @@ static int hf_dcom_extent_id = -1;
 
 static int hf_dcom_hresult = -1;
 static int hf_dcom_tobedone = -1;
-static int hf_dcom_tobedone_len = -1;
+static int hf_dcom_nospec = -1;
 static int hf_dcom_array_size = -1;
 static int hf_dcom_pointer_val = -1;
 
@@ -230,6 +231,7 @@ GList *dcom_interfaces;
 
 static const value_string dcom_thisthat_flag_vals[] = {
     { 0, "INFO_NULL" },
+    { 1, "INFO_LOCAL" },
     { 0,  NULL }
 };
 
@@ -250,7 +252,7 @@ void dcom_interface_dump(void) {
 
         for(objects = machine->objects; objects != NULL; objects = g_list_next(objects)) {
             object = objects->data;
-            g_warning(" Object(#%4u): OID:0x%x%x private:0x%x", object->first_packet, object->oid, object->private_data);
+            g_warning(" Object(#%4u): OID:0x%" PRIx64 " private:%p", object->first_packet, object->oid, object->private_data);
 
             for(interfaces = object->interfaces; interfaces != NULL; interfaces = g_list_next(interfaces)) {
                 interf = interfaces->data;
@@ -532,6 +534,8 @@ const value_string dcom_hresult_vals[] = {
 	{ 0x80020009, "DISP_E_EXCEPTION" },
 	{ 0x8002000A, "DISP_E_OVERFLOW" },
 
+	{ 0x8002801D, "TYPE_E_LIBNOTREGISTERED" },
+
 	{ 0x80040154, "REGDB_E_CLASSNOTREG" },
 	{ 0x80040201, "CO_E_FAILEDTOGETSECCTX" },
 
@@ -745,7 +749,7 @@ dissect_dcom_extent(tvbuff_t *tvb, int offset,
 
 		    u32VariableOffset = dissect_dcom_dcerpc_array_size(tvb, u32VariableOffset, pinfo, sub_tree, drep, 
 							    &u32ArraySize2);
-		    u32VariableOffset = dissect_dcom_tobedone_data(tvb, u32VariableOffset, pinfo, sub_tree, drep, u32ArraySize2);
+		    u32VariableOffset = dissect_dcom_nospec_data(tvb, u32VariableOffset, pinfo, sub_tree, drep, u32ArraySize2);
 
 		    /* update subtree header */
             if(uuid_name != NULL) {
@@ -939,11 +943,31 @@ dissect_dcom_tobedone_data(tvbuff_t *tvb, int offset,
 {
 	proto_item *item;
 
-    proto_tree_add_uint(tree, hf_dcom_tobedone_len, tvb, offset, length, length);
 
 	item = proto_tree_add_bytes(tree, hf_dcom_tobedone, tvb, offset, length, 
 		tvb_get_ptr(tvb, offset, length));
-	expert_add_info_format(pinfo, item, PI_UNDECODED, PI_WARN, "%u bytes still undecoded", length);
+        PROTO_ITEM_SET_GENERATED(item);
+	expert_add_info_format(pinfo, item, PI_UNDECODED, PI_WARN, "Dissection incomplete");
+
+	offset += length;
+
+	return offset;
+}
+
+
+/* mark data as "No Specification Available" */
+/* XXX: handout data to generic "unkown data" dissector? */
+extern int
+dissect_dcom_nospec_data(tvbuff_t *tvb, int offset,
+	packet_info *pinfo _U_, proto_tree *tree, guint8 *drep _U_, int length)
+{
+	proto_item *item;
+
+
+	item = proto_tree_add_bytes(tree, hf_dcom_nospec, tvb, offset, length, 
+		tvb_get_ptr(tvb, offset, length));
+        PROTO_ITEM_SET_GENERATED(item);
+	expert_add_info_format(pinfo, item, PI_UNDECODED, PI_NOTE, "No specification available, dissection not possible");
 
 	offset += length;
 
@@ -1470,7 +1494,7 @@ dissect_dcom_append_UUID(tvbuff_t *tvb, int offset,
     if (field_index != -1) {
         proto_item_append_text(pi, "[%u]: ", field_index);
     } else {
-        proto_item_append_text(pi, ": ", field_index);
+        proto_item_append_text(pi, ": ");
     }
 
     if(uuid_name) {
@@ -1485,7 +1509,7 @@ dissect_dcom_append_UUID(tvbuff_t *tvb, int offset,
                           uuid->Data4[6], uuid->Data4[7]);
 
     if(uuid_name) {
-	    proto_item_append_text(pi, ")", uuid_name);
+	    proto_item_append_text(pi, ")");
     }
 
 	/* update column info now */
@@ -1910,7 +1934,7 @@ dissect_dcom_OBJREF(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 	        offset = dissect_dcom_DWORD(tvb, offset, pinfo, sub_tree, drep, 
                                 hf_dcom_objref_size, &u32Size);
             /* the following data depends on the CLSID, no docs available on this */
-			offset = dissect_dcom_tobedone_data(tvb, offset, pinfo, sub_tree, drep, u32Size);
+			offset = dissect_dcom_nospec_data(tvb, offset, pinfo, sub_tree, drep, u32Size);
 		break;
 	}
 
@@ -2049,9 +2073,9 @@ proto_register_dcom (void)
 		{ &hf_dcom_actual_count,
 		{ "ActualCount", "dcom.actual_count", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
 		{ &hf_dcom_tobedone,
-		{ "ToBeDone", "dcom.tobedone", FT_BYTES, BASE_HEX, NULL, 0x0, "", HFILL }},
-		{ &hf_dcom_tobedone_len,
-		{ "ToBeDoneLen", "dcom.tobedone_len", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
+		{ "To Be Done", "dcom.tobedone", FT_BYTES, BASE_HEX, NULL, 0x0, "", HFILL }},
+		{ &hf_dcom_nospec,
+		{ "No Specification Available", "dcom.nospec", FT_BYTES, BASE_HEX, NULL, 0x0, "", HFILL }},
 		{ &hf_dcom_variant,
 		{ "Variant", "dcom.variant", FT_NONE, BASE_HEX, NULL, 0x0, "", HFILL }},
 		{ &hf_dcom_variant_type,
