@@ -1,7 +1,7 @@
 /* packet-diameter.c
  * Routines for Diameter packet disassembly
  *
- * $Id: packet-diameter.c 18679 2006-07-07 04:51:12Z etxrab $
+ * $Id: packet-diameter.c 18809 2006-08-01 17:07:28Z martinm $
  *
  * Copyright (c) 2001 by David Frascone <dave@frascone.com>
  *
@@ -1797,7 +1797,6 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 			                                      avpNameString,avph.avp_code);
 			if (!AVP_code_found)
 			{
-				printf("adding expert info!\n");
 				expert_add_info_format(pinfo, ti,
 				                       PI_UNDECODED, PI_NOTE,
 				                       "AVP info not available (code %u)",
@@ -1902,10 +1901,14 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 								proto_tree_add_item(avpi_tree, hf_diameter_avp_data_v6addr,
 								                    tvb, offset, avpDataLength, FALSE);
 							} else {
-								proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
-								                            tvb, offset, avpDataLength,
-								                            tvb_get_ptr(tvb, offset, avpDataLength),
-								                            "Error! Bad Address Length (Address in RFC3588 format?)");
+								ti = proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
+								                                 tvb, offset, avpDataLength,
+								                                 tvb_get_ptr(tvb, offset, avpDataLength),
+								                                 "Error! Bad Address Length (Address in RFC3588 format?)");
+								expert_add_info_format(pinfo, ti,
+								                       PI_MALFORMED, PI_NOTE,
+								                       "Bad Address Length (%u)",
+								                       avpDataLength);
 							}
 							break;
 						case DIAMETER_RFC:
@@ -1935,10 +1938,14 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 						proto_tree_add_item(avpi_tree, hf_diameter_avp_data_int32,
 						                    tvb, offset, avpDataLength, FALSE);
 					} else {
-						proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
-						                            tvb, offset, avpDataLength,
-						                            tvb_get_ptr(tvb, offset, avpDataLength),
-						                            "Error!  Bad Integer32 Length");
+						ti = proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
+						                                 tvb, offset, avpDataLength,
+						                                 tvb_get_ptr(tvb, offset, avpDataLength),
+						                                "Error!  Bad Integer32 Length");
+						expert_add_info_format(pinfo, ti,
+						                       PI_MALFORMED, PI_NOTE,
+						                       "Bad Integer32 Length (%u)",
+						                       avpDataLength);
 					}
 					break;
 
@@ -1951,10 +1958,14 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 						                           tvb, offset, avpDataLength, data,
 						                           "Value: 0x%08x (%u)", data, data);
 					} else {
-						proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
-						                            tvb, offset, avpDataLength,
-						                            tvb_get_ptr(tvb, offset, avpDataLength),
-						                            "Error!  Bad Unsigned32 Length");
+						ti = proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
+						                                 tvb, offset, avpDataLength,
+						                                 tvb_get_ptr(tvb, offset, avpDataLength),
+						                                 "Error!  Bad Unsigned32 Length");
+						expert_add_info_format(pinfo, ti,
+						                       PI_MALFORMED, PI_NOTE,
+						                       "Bad Unsigned32 Length (%u)",
+						                       avpDataLength);
 					}
 					break;
 
@@ -1969,10 +1980,14 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 						                           "Value: 0x%08x (%u): %s", data,
 						                           data, valstr);
 					} else {
-						proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
-						                            tvb, offset, avpDataLength,
-						                            tvb_get_ptr(tvb, offset, avpDataLength),
-						                            "Error!  Bad Enumerated Length");
+						ti = proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
+						                                tvb, offset, avpDataLength,
+						                                tvb_get_ptr(tvb, offset, avpDataLength),
+						                                "Error!  Bad Enumerated Length");
+						expert_add_info_format(pinfo, ti,
+						                       PI_MALFORMED, PI_NOTE,
+						                       "Bad Enumerated Length (%u)",
+						                       avpDataLength);
 					}
 					break;
 
@@ -1981,10 +1996,14 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 						proto_tree_add_item(avpi_tree, hf_diameter_avp_data_int64,
 						                    tvb, offset, 8, FALSE);
 					} else {
-						proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
+						ti = proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
 						                            tvb, offset, avpDataLength,
 						                            tvb_get_ptr(tvb, offset, avpDataLength),
 						                            "Error!  Bad Integer64 Length");
+						expert_add_info_format(pinfo, ti,
+						                       PI_MALFORMED, PI_NOTE,
+						                       "Bad Integer64 Length (%u)",
+						                       avpDataLength);
 					}
 					break;
 
@@ -1993,10 +2012,14 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 						proto_tree_add_item(avpi_tree, hf_diameter_avp_data_uint64,
 						                    tvb, offset, 8, FALSE);
 					} else {
-						proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
-						                            tvb, offset, avpDataLength,
-						                            tvb_get_ptr(tvb, offset, avpDataLength),
-						                            "Error!  Bad Unsigned64 Length");
+						ti = proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
+						                                tvb, offset, avpDataLength,
+						                                tvb_get_ptr(tvb, offset, avpDataLength),
+						                               "Error!  Bad Unsigned64 Length");
+						expert_add_info_format(pinfo, ti,
+						                       PI_MALFORMED, PI_NOTE,
+						                       "Bad Unsigned64 Length (%u)",
+						                       avpDataLength);
 					}
 					break;
 
@@ -2025,10 +2048,14 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 							                            "Error!  Time before 00:00:00 UTC, January 1, 1970");
 						}
 					} else {
-						proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
-						                            tvb, offset, avpDataLength,
-						                            tvb_get_ptr(tvb, offset, avpDataLength),
-						                            "Error!  Bad Time Length");
+						ti = proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
+						                                 tvb, offset, avpDataLength,
+						                                 tvb_get_ptr(tvb, offset, avpDataLength),
+						                                 "Error!  Bad Time Length");
+						expert_add_info_format(pinfo, ti,
+						                       PI_MALFORMED, PI_NOTE,
+						                       "Bad Time Length (%u)",
+						                       avpDataLength);
 					}
 					break;
 
@@ -2043,10 +2070,14 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 						                           "Value: 0x%08x (%u): %s", data,
 						                           data, valstr);
 					} else {
-						proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
-						                            tvb, offset, avpDataLength,
-						                            tvb_get_ptr(tvb, offset, avpDataLength),
-						                            "Error!  Bad Enumerated Length");
+						ti = proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
+						                                 tvb, offset, avpDataLength,
+						                                 tvb_get_ptr(tvb, offset, avpDataLength),
+						                                 "Error!  Bad Enumerated Length");
+						expert_add_info_format(pinfo, ti,
+						                       PI_MALFORMED, PI_NOTE,
+						                       "Bad Enumerated Length (%u)",
+						                       avpDataLength);
 					}
 					break;
 
@@ -2054,10 +2085,14 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 					if (avpDataLength == 4) {
 						proto_tree_add_item(avpi_tree, hf_diameter_vendor_id, tvb, offset, avpDataLength, FALSE);
 					} else {
-						proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
+						ti = proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
 						                            tvb, offset, avpDataLength,
 						                            tvb_get_ptr(tvb, offset, avpDataLength),
 						                            "Error!  Bad Vendor ID Length");
+						expert_add_info_format(pinfo, ti,
+						                       PI_MALFORMED, PI_NOTE,
+						                       "Bad Vendor ID Length (%u)",
+						                       avpDataLength);
 					}
 					break;
 
@@ -2072,10 +2107,14 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 						                           "Application ID: %s %d (0x%08x)",
 						                           valstr, data, data);
 					} else {
-						proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
-						                            tvb, offset, avpDataLength,
-						                            tvb_get_ptr(tvb, offset, avpDataLength),
-						                            "Error!  Bad Application ID Length");
+						ti = proto_tree_add_bytes_format(avpi_tree, hf_diameter_avp_data_bytes,
+						                                 tvb, offset, avpDataLength,
+						                                 tvb_get_ptr(tvb, offset, avpDataLength),
+						                                 "Error!  Bad Application ID Length");
+						expert_add_info_format(pinfo, ti,
+						                       PI_MALFORMED, PI_NOTE,
+						                       "Bad Application ID Length (%u)",
+						                       avpDataLength);
 					}
 					break;
 
@@ -2104,10 +2143,9 @@ static void dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree
 				}
 					break;
 				case DIAMETER_PRIVATE_ID:
-				{
 					proto_tree_add_item(avpi_tree, hf_diameter_avp_private_id,
 					                    tvb, offset, avpDataLength, FALSE);
-				}
+					break;
 
 				default:
 				case DIAMETER_OCTET_STRING:
