@@ -1,7 +1,7 @@
 /* ethertype.c
  * Routines for calling the right protocol for the ethertype.
  *
- * $Id: packet-ethertype.c 18944 2006-08-18 08:46:31Z sahlberg $
+ * $Id: packet-ethertype.c 19447 2006-10-07 14:48:04Z jmayer $
  *
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -47,11 +47,12 @@ static dissector_handle_t data_handle;
 const value_string etype_vals[] = {
     {ETHERTYPE_IP,					"IP"				},
     {ETHERTYPE_IPv6,				"IPv6"				},
-    {ETHERTYPE_CISCOWL,				"Cisco Wireless"		},
+    {ETHERTYPE_WLCCP,				"Cisco Wireless Lan Context Control Protocol"		},
     {ETHERTYPE_CENTRINO_PROMISC,	"IEEE 802.11 (Centrino promiscuous)" },
     {ETHERTYPE_XNS_IDP,				"XNS Internet Datagram Protocol" },
     {ETHERTYPE_X25L3,				"X.25 Layer 3"			},
     {ETHERTYPE_ARP,					"ARP"				},
+    {ETHERTYPE_EPL_V1,					"EPL_V1"				},
     {ETHERTYPE_REVARP,				"RARP"				},
     {ETHERTYPE_DEC_LB,				"DEC LanBridge"			},
     {ETHERTYPE_ATALK,				"Appletalk"			},
@@ -63,7 +64,8 @@ const value_string etype_vals[] = {
     {ETHERTYPE_TRAIN,				"Netmon Train"			},
     {ETHERTYPE_LOOP,				"Loopback"			}, /* Ethernet Loopback */
     {ETHERTYPE_FOUNDRY,				"Foundry proprietary"		},
-    {ETHERTYPE_WCP,					"Wellfleet Compression Protocol" },
+    {ETHERTYPE_WCP,				"Wellfleet Compression Protocol" },
+    {ETHERTYPE_STP,				"Spanning Tree Protocol"	},
     {ETHERTYPE_ISMP,				"Cabletron Interswitch Message Protocol" },
     {ETHERTYPE_ISMP_TBFLOOD,		"Cabletron SFVLAN 1.8 Tag-Based Flood" },
     				/* for ISMP, see RFC 2641, RFC 2642, RFC 2643 */
@@ -96,6 +98,7 @@ const value_string etype_vals[] = {
     {ETHERTYPE_PROFINET,			"PROFINET"			},
     {ETHERTYPE_AOE,					"ATA over Ethernet"             },
     {ETHERTYPE_TELKONET,					"Telkonet powerline"    },
+    {ETHERTYPE_EPL_V2,				"ETHERNET Powerlink v2"			},
     {ETHERTYPE_CSM_ENCAPS,			"CSM_ENCAPS Protocol"             },
     {ETHERTYPE_IEEE802_OUI_EXTENDED, "IEEE 802a OUI Extended Ethertype" },
     {ETHERTYPE_IEC61850_GOOSE,		"IEC 61850/GOOSE" },
