@@ -4,7 +4,7 @@
  *   Copyright 2003  Ronnie Sahlberg, exchange first/last matching and 
  *                                    tap listener and misc updates
  *
- * $Id: packet-fc.c 18894 2006-08-14 08:29:29Z sahlberg $
+ * $Id: packet-fc.c 19510 2006-10-13 08:42:33Z sahlberg $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -49,6 +49,7 @@
 #include <epan/reassemble.h>
 #include <epan/conversation.h>
 #include <epan/etypes.h>
+#include "packet-scsi.h"
 #include "packet-fc.h"
 #include "packet-fclctl.h"
 #include "packet-fcbls.h"
@@ -107,6 +108,7 @@ static int hf_fc_param = -1;
 static int hf_fc_ftype = -1;    /* Derived field, non-existent in FC hdr */
 static int hf_fc_reassembled = -1;
 static int hf_fc_eisl = -1;
+static int hf_fc_relative_offset = -1;
 
 /* Network_Header fields */
 static int hf_fc_nh_da = -1;
@@ -585,6 +587,7 @@ dissect_fc_helper (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean
     guint32 frag_id;
     guint32 frag_size;
     guint8 df_ctl, seq_id;
+    guint32 f_ctl;
     
     guint32 param;
     guint16 real_seqcnt;
@@ -639,6 +642,7 @@ dissect_fc_helper (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean
     fchdr.seqcnt = tvb_get_ntohs (tvb, offset+14);
     fchdr.oxid=tvb_get_ntohs(tvb,offset+16);
     fchdr.rxid=tvb_get_ntohs(tvb,offset+18);
+    fchdr.relative_offset=0;
     param = tvb_get_ntohl (tvb, offset+20);
     seq_id = tvb_get_guint8 (tvb, offset+12);
 
@@ -676,9 +680,13 @@ dissect_fc_helper (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean
         fc_ex->last_exchange_frame=0;
         fc_ex->lun=0xffff;
         fc_ex->scsi_opcode=0xffff;
+	fc_ex->task_flags=0;
+	fc_ex->data_length=0;
+	fc_ex->bidir_data_length=0;
         fc_ex->fc_time=pinfo->fd->abs_ts;
         fc_ex->flags=0;
         fc_ex->alloc_len=0;
+	fc_ex->extra_data=NULL;
 	se_tree_insert32(fc_conv_data->exchanges, fchdr.oxid, fc_ex);
     }
     /* populate the exchange struct */
@@ -887,6 +895,7 @@ dissect_fc_helper (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean
 
 
     dissect_fc_fctl(pinfo, fc_tree, tvb, offset+9);
+    f_ctl = tvb_get_ntoh24(tvb, offset+9);
 
 
     proto_tree_add_item (fc_tree, hf_fc_seqid, tvb, offset+12, 1, FALSE);
@@ -924,6 +933,13 @@ dissect_fc_helper (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean
             proto_tree_add_item (fc_tree, hf_fc_param, tvb, offset+20,
                                  4, FALSE);
         }
+    } else if (ftype == FC_FTYPE_SCSI ) {
+        if (f_ctl&FC_FCTL_REL_OFFSET){
+            proto_tree_add_item (fc_tree, hf_fc_relative_offset, tvb, offset+20, 4, FALSE);
+            fchdr.relative_offset=tvb_get_ntohl(tvb, offset+20);
+        } else {
+            proto_tree_add_item (fc_tree, hf_fc_param, tvb, offset+20, 4, FALSE);
+        }
     } else {
         proto_tree_add_item (fc_tree, hf_fc_param, tvb, offset+20, 4, FALSE);
     }
@@ -1264,6 +1280,9 @@ proto_register_fc(void)
            0, "Time since the first frame of the Exchange", HFILL }},
         { &hf_fc_eisl,
           {"EISL Header", "fc.eisl", FT_BYTES, BASE_HEX, NULL, 0, "EISL Header", HFILL}},
+        { &hf_fc_relative_offset,
+          {"Relative Offset", "fc.relative_offset", FT_UINT32, BASE_DEC, NULL,
+           0, "Relative offset of data", HFILL}},
     };
 
     /* Setup protocol subtree array */
