@@ -2,7 +2,7 @@
  * Routines for Fibre Channel Protocol for SCSI (FCP)
  * Copyright 2001, Dinesh G Dutt <ddutt@cisco.com>
  *
- * $Id: packet-fcp.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-fcp.c 19511 2006-10-13 09:16:45Z sahlberg $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -46,9 +46,9 @@
 #include <epan/packet.h>
 #include <epan/conversation.h>
 #include <epan/etypes.h>
+#include "packet-scsi.h"
 #include "packet-fc.h"
 #include "packet-fcp.h"
-#include "packet-scsi.h"
 
 /* Initialize the protocol and registered fields */
 static int proto_fcp         = -1;
@@ -61,6 +61,7 @@ static int hf_fcp_addlcdblen = -1;
 static int hf_fcp_rddata     = -1;
 static int hf_fcp_wrdata     = -1;
 static int hf_fcp_dl         = -1;
+static int hf_fcp_bidir_dl   = -1;
 static int hf_fcp_data_ro    = -1;
 static int hf_fcp_burstlen   = -1;
 static int hf_fcp_rspflags   = -1;
@@ -97,7 +98,7 @@ static gint ett_fcp_taskmgmt = -1;
 static gint ett_fcp_rsp_flags = -1;
 
 typedef struct _fcp_conv_data_t {
-    se_tree_t *luns;
+    emem_tree_t *luns;
 } fcp_conv_data_t;
 
 static dissector_table_t fcp_dissector;
@@ -397,7 +398,7 @@ dissect_fcp_cmnd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, pro
     int offset = 0;
     int len,
         add_len = 0;
-    guint8 flags, lun0;
+    guint8 flags, rwflags, lun0;
         guint16 lun=0xffff;
     tvbuff_t *cdb_tvb;
     int tvb_len, tvb_rlen;
@@ -440,6 +441,7 @@ dissect_fcp_cmnd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, pro
     if(!itl){
         itl=se_alloc(sizeof(itl_nexus_t));
         itl->cmdset=0xff;
+	itl->conversation=conversation;
         se_tree_insert32(fcp_conv_data->luns, lun, itl);
     }
 
@@ -447,6 +449,15 @@ dissect_fcp_cmnd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, pro
     proto_tree_add_item(tree, hf_fcp_taskattr, tvb, offset+9, 1, 0);
     dissect_task_mgmt_flags(pinfo, tree, tvb, offset+10);
     proto_tree_add_item(tree, hf_fcp_addlcdblen, tvb, offset+11, 1, 0);
+    rwflags=tvb_get_guint8(tvb, offset+11);
+    if(fchdr->itlq){
+	if(rwflags&0x02){
+	    fchdr->itlq->task_flags|=SCSI_DATA_READ;
+	}
+	if(rwflags&0x01){
+	    fchdr->itlq->task_flags|=SCSI_DATA_WRITE;
+	}
+    }
     proto_tree_add_item(tree, hf_fcp_rddata, tvb, offset+11, 1, 0);
     proto_tree_add_item(tree, hf_fcp_wrdata, tvb, offset+11, 1, 0);
 
@@ -461,12 +472,26 @@ dissect_fcp_cmnd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, pro
 
     proto_tree_add_item(tree, hf_fcp_dl, tvb, offset+12+16+add_len,
 			 4, 0);
+    if(fchdr->itlq){
+	fchdr->itlq->data_length=tvb_get_ntohl(tvb, offset+12+16+add_len);
+    }
+
+    if( ((rwflags&0x03)==0x03)
+    &&  tvb_length_remaining(tvb, offset+12+16+add_len+4)>=4){
+	proto_tree_add_item(tree, hf_fcp_bidir_dl, tvb, offset+12+16+add_len+4,
+			 4, 0);
+	if(fchdr->itlq){
+	    fchdr->itlq->bidir_data_length=tvb_get_ntohl(tvb, offset+12+16+add_len+4);
+	}
+
+    }
+	
 }
 
 static void
 dissect_fcp_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree, conversation_t *conversation, fc_hdr *fchdr, itl_nexus_t *itl)
 {
-    dissect_scsi_payload(tvb, pinfo, parent_tree, FALSE, fchdr->itlq, itl);
+    dissect_scsi_payload(tvb, pinfo, parent_tree, FALSE, fchdr->itlq, itl, fchdr->relative_offset);
 }
 
 /* fcp-3  9.5 table 24 */
@@ -621,7 +646,7 @@ dissect_fcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     fcp_conv_data=conversation_get_proto_data(fchdr->conversation, proto_fcp);
     if(!fcp_conv_data){
         fcp_conv_data=se_alloc(sizeof(fcp_conv_data_t));
-        fcp_conv_data->luns=se_tree_create_non_persistent(SE_TREE_TYPE_RED_BLACK, "FCP Luns");
+        fcp_conv_data->luns=se_tree_create_non_persistent(EMEM_TREE_TYPE_RED_BLACK, "FCP Luns");
         conversation_add_proto_data(fchdr->conversation, proto_fcp, fcp_conv_data);
     }
 
@@ -713,6 +738,8 @@ proto_register_fcp (void)
           {"WRDATA", "fcp.wrdata", FT_BOOLEAN, 8, NULL, 0x01, "", HFILL}},
         { &hf_fcp_dl,
           {"FCP_DL", "fcp.dl", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL}},
+        { &hf_fcp_bidir_dl,
+          {"FCP_BIDIRECTIONAL_READ_DL", "fcp.bidir_dl", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL}},
         { &hf_fcp_data_ro,
           {"FCP_DATA_RO", "fcp.data_ro", FT_UINT32, BASE_DEC, NULL, 0x0, "",
            HFILL}},
