@@ -3,7 +3,7 @@
  * Copyright 2004, Anders Broman.
  * Copyright 2004, Olivier Biot.
  *
- * $Id: packet-multipart.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-multipart.c 19566 2006-10-16 23:13:21Z ulfl $
  *
  * Refer to the AUTHORS file or the AUTHORS section in the man page
  * for contacting the author(s) of this file.
@@ -95,16 +95,20 @@ static const multipart_header_t multipart_headers[] = {
 	{ "Unknown-header", NULL },		/* Pad so that the real headers start at index 1 */
 	{ "Content-Disposition", NULL },
 	{ "Content-Encoding", "e" },
+	{ "Content-Id", NULL },
 	{ "Content-Language", NULL },
 	{ "Content-Length", "l" },
+	{ "Content-Transfer-Encoding", NULL },
 	{ "Content-Type", "c" },
 };
 
-#define POS_CONTENT_DISPOSITION		1
-#define POS_CONTENT_ENCODING		2
-#define POS_CONTENT_LANGUAGE		3
-#define POS_CONTENT_LENGTH			4
-#define POS_CONTENT_TYPE			5
+#define POS_CONTENT_DISPOSITION			1
+#define POS_CONTENT_ENCODING			2
+#define POS_CONTENT_ID					3
+#define POS_CONTENT_LANGUAGE			4
+#define POS_CONTENT_LENGTH				5
+#define POS_CONTENT_TRANSFER_ENCODING	6
+#define POS_CONTENT_TYPE				7
 
 /* Initialize the header fields */
 static gint hf_multipart_type = -1;
@@ -112,8 +116,10 @@ static gint hf_header_array[] = {
 	-1, /* "Unknown-header" - Pad so that the real headers start at index 1 */
 	-1, /* "Content-Disposition" */
 	-1, /* "Content-Encoding" */
+	-1, /* "Content-Id" */
 	-1, /* "Content-Language" */
 	-1, /* "Content-Length" */
+	-1, /* "Content-Transfer-Encoding" */
 	-1, /* "Content-Type" */
 };
 
@@ -238,6 +244,10 @@ unfold_and_compact_mime_header(const char *lines, gint *first_colon_offset)
 					break;
 				}
 			}
+			/* if already zero terminated now, rewind one char to avoid an "off by one" */
+			if(c == 0) {
+				q--;
+			}
 		} else { /* Regular character */
 			if (sep_seen) {
 				sep_seen = 0;
@@ -730,13 +740,17 @@ static void dissect_multipart(tvbuff_t *tvb, packet_info *pinfo,
 
 	/* Add stuff to the protocol tree */
 	if (tree) {
+		proto_item *type_ti;
 		ti = proto_tree_add_item(tree, proto_multipart,
 				tvb, 0, -1, FALSE);
 		subtree = proto_item_add_subtree(ti, ett_multipart);
 		proto_item_append_text(ti, ", Type: %s, Boundary: \"%s\"",
 				m_info->type, m_info->boundary);
-		proto_tree_add_string(subtree, hf_multipart_type,
-				tvb, 0, 0, pinfo->match_string);
+
+		/* Show multi-part type as a generated field */
+		type_ti = proto_tree_add_string(subtree, hf_multipart_type,
+		                                tvb, 0, 0, pinfo->match_string);
+		PROTO_ITEM_SET_GENERATED(type_ti);
 	}
 
 	/*
@@ -820,42 +834,56 @@ proto_register_multipart(void)
 			{	"Type",
 				"mime_multipart.type",
 				FT_STRING, BASE_NONE, NULL, 0x00,
-				"RFC 3261: MIME multipart encapsulation type", HFILL
+				"MIME multipart encapsulation type", HFILL
 			}
 		},
 		{ &hf_header_array[POS_CONTENT_DISPOSITION],
 			{	"Content-Disposition",
 				"mime_multipart.header.content-disposition",
 				FT_STRING, BASE_NONE, NULL, 0x00,
-				"RFC 3261: Content-Disposition Header", HFILL
+				"RFC 2183: Content-Disposition Header", HFILL
 			}
 		},
 		{ &hf_header_array[POS_CONTENT_ENCODING],
 			{	"Content-Encoding",
 				"mime_multipart.header.content-encoding",
 				FT_STRING, BASE_NONE, NULL, 0x00,
-				"RFC 3261: Content-Encoding Header", HFILL
+				"Content-Encoding Header", HFILL
+			}
+		},
+		{ &hf_header_array[POS_CONTENT_ID],
+			{	"Content-Id",
+				"mime_multipart.header.content-id",
+				FT_STRING, BASE_NONE, NULL, 0x00,
+				"RFC 2045: Content-Id Header", HFILL
 			}
 		},
 		{ &hf_header_array[POS_CONTENT_LANGUAGE],
 			{	"Content-Language",
 				"mime_multipart.header.content-language",
 				FT_STRING, BASE_NONE, NULL, 0x00,
-				"RFC 3261: Content-Language Header", HFILL
+				"Content-Language Header", HFILL
 			}
 		},
 		{ &hf_header_array[POS_CONTENT_LENGTH],
 			{	"Content-Length",
 				"mime_multipart.header.content-length",
 				FT_STRING, BASE_NONE, NULL, 0x0,
-				"RFC 3261: Content-Length Header", HFILL
+				"Content-Length Header", HFILL
+			}
+		},
+		{ &hf_header_array[POS_CONTENT_TRANSFER_ENCODING],
+			{	"Content-Transfer-Encoding",
+				"mime_multipart.header.content-transfer-encoding",
+				FT_STRING, BASE_NONE, NULL, 0x00,
+				"RFC 2045: Content-Transfer-Encoding Header", HFILL
 			}
 		},
 		{ &hf_header_array[POS_CONTENT_TYPE],
 			{	"Content-Type",
 				"mime_multipart.header.content-type",
 				FT_STRING, BASE_NONE,NULL,0x0,
-				"RFC 3261: Content-Type Header", HFILL
+				"Content-Type Header", HFILL
 			}
 		},
 	};
