@@ -2,7 +2,7 @@
  * Routines for NTP packet dissection
  * Copyright 1999, Nathan Neulinger <nneul@umr.edu>
  *
- * $Id: packet-ntp.c 18678 2006-07-07 04:01:09Z gerald $
+ * $Id: packet-ntp.c 19007 2006-08-23 15:51:15Z jake $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -180,6 +180,8 @@ static const struct {
 	{ "IRIG",	"IRIG-B timecode" },
 	{ "1PPS",	"External 1 PPS input" },
 	{ "FREE",	"(Internal clock)" },
+	{ "INIT",	"(Initialization)" },
+	{ "\0\0\0\0",	"NULL" },
 	{ NULL,		NULL}
 };
 
@@ -548,9 +550,9 @@ dissect_ntp_std(tvbuff_t *tvb, proto_tree *ntp_tree, guint8 flags)
 	 * higher level server. My decision was to resolve this address.
 	 */
 	refid = tvb_get_ptr(tvb, 12, 4);
+	buff = ep_alloc(NTP_TS_SIZE);
 	if (stratum <= 1) {
-		buff=ep_alloc(NTP_TS_SIZE);
-		g_snprintf (buff, NTP_TS_SIZE, "Unindentified reference source '%.4s'",
+		g_snprintf (buff, NTP_TS_SIZE, "Unidentified reference source '%.4s'",
 			refid);
 		for (i = 0; primary_sources[i].id; i++) {
 			if (memcmp (refid, primary_sources[i].id, 4) == 0) {
@@ -561,7 +563,6 @@ dissect_ntp_std(tvbuff_t *tvb, proto_tree *ntp_tree, guint8 flags)
 		}
 	} else {
 		int buffpos;
-		buff = ep_alloc(NTP_TS_SIZE);
 		refid_addr = tvb_get_ipv4(tvb, 12);
 		buffpos = g_snprintf(buff, NTP_TS_SIZE, "%s", get_hostname (refid_addr));
 		if (buffpos >= NTP_TS_SIZE) {
