@@ -5,7 +5,7 @@
  * Changed 03/12/2003 Rui Carmo (http://the.taoofmac.com - added all 3GPP VSAs, some parsing)
  * Changed 07/2005 Luis Ontanon <luis.ontanon@gmail.com> - use FreeRADIUS' dictionary
  *
- * $Id: packet-radius.c 18580 2006-06-27 12:12:31Z sahlberg $
+ * $Id: packet-radius.c 19157 2006-09-05 16:06:41Z martinm $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -992,7 +992,7 @@ extern void radius_register_avp_dissector(guint32 vendor_id, guint32 attribute_i
 			vendor->ett = no_vendor.ett;
             
 			g_hash_table_insert(dict->vendors_by_id,GUINT_TO_POINTER(vendor->code),vendor);
-			g_hash_table_insert(dict->vendors_by_name,vendor->name,vendor);
+			g_hash_table_insert(dict->vendors_by_name,(gpointer)(vendor->name),vendor);
 		}
 		
 		dictionary_entry = g_hash_table_lookup(vendor->attrs_by_id,GUINT_TO_POINTER(attribute_id));
@@ -1149,6 +1149,7 @@ proto_register_radius(void)
 	radius_vendors = (value_string*) ri.vend_vs->data;
 	
 	proto_radius = proto_register_protocol("Radius Protocol", "RADIUS", "radius");
+	register_dissector("radius", dissect_radius, proto_radius);
 	
 	proto_register_field_array(proto_radius,(hf_register_info*)(ri.hf->data),ri.hf->len);
 	proto_register_subtree_array((gint**)(ri.ett->data), ri.ett->len);
