@@ -6,7 +6,7 @@
  * Copyright 2000, Philips Electronics N.V.
  * Written by Andreas Sikkema <h323@ramdyne.nl>
  *
- * $Id: packet-rtp.c 18581 2006-06-27 12:40:53Z kukosa $
+ * $Id: packet-rtp.c 19393 2006-10-02 10:49:57Z martinm $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -742,7 +742,7 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		offset += 2;
 		if ( hdr_extension > 0 ) {
 			if ( tree ) {
-				ti = proto_tree_add_text(rtp_tree, tvb, offset, csrc_count * 4, "Header extensions");
+				ti = proto_tree_add_text(rtp_tree, tvb, offset, hdr_extension * 4, "Header extensions");
 				/* I'm re-using the old tree variable here
 				   from the CSRC list!*/
 				rtp_csrc_tree = proto_item_add_subtree( ti,
