@@ -4,7 +4,7 @@
  * Jason Lango <jal@netapp.com>
  * Liberally copied from packet-http.c, by Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-sdp.c 18806 2006-07-28 21:26:39Z etxrab $
+ * $Id: packet-sdp.c 19139 2006-09-04 21:07:35Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -1221,10 +1221,9 @@ static void dissect_sdp_media_attribute(tvbuff_t *tvb, proto_item * ti, transpor
 
     proto_tree_add_item(sdp_media_attribute_tree, hf_media_encoding_name, tvb,
                         offset, tokenlen, FALSE);
-    transport_info->encoding_name = tvb_get_ephemeral_string(tvb, offset,
+	/* get_string is needed here as the string is "saved" in a hashtable */
+    transport_info->encoding_name = tvb_get_string(tvb, offset,
                                                              tokenlen);
-    proto_tree_add_text(sdp_media_attribute_tree, tvb, offset, tokenlen,
-                        "MIME type: %s", transport_info->encoding_name);
 
     key=g_malloc( sizeof(gint) );
     *key=atol(payload_type);
