@@ -1,6 +1,6 @@
 /* epan.h
  *
- * $Id: epan.c 18863 2006-08-10 06:06:21Z kukosa $
+ * $Id: epan.c 19327 2006-09-26 10:22:20Z lego $
  *
  * Wireshark Protocol Analyzer Library
  */
@@ -38,6 +38,10 @@
 #include "emem.h"
 #include "expert.h"
 
+#ifdef HAVE_LUA_5_1
+	int wslua_init(void*);
+#endif
+
 static void (*report_failure_func)(const char *, va_list);
 static void (*report_open_failure_func)(const char *, int, gboolean);
 static void (*report_read_failure_func)(const char *, int);
@@ -93,6 +97,10 @@ epan_init(const char *plugin_dir, void (*register_all_protocols)(void),
 	final_registration_all_protocols();
 	host_name_lookup_init();
 	expert_init();
+#ifdef HAVE_LUA_5_1
+	wslua_init(NULL);
+#endif
+
 }
 
 void
