@@ -1,7 +1,7 @@
 /* etypes.h
  * Defines ethernet packet types, similar to tcpdump's ethertype.h
  *
- * $Id: etypes.h 18944 2006-08-18 08:46:31Z sahlberg $
+ * $Id: etypes.h 19447 2006-10-07 14:48:04Z jmayer $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -88,6 +88,10 @@
 #define ETHERTYPE_IP		0x0800
 #endif
 
+#ifndef ETHERTYPE_EPL_V1
+#define ETHERTYPE_EPL_V1	0x3E3F
+#endif
+
 #ifndef ETHERTYPE_X25L3
 #define ETHERTYPE_X25L3		0x0805
 #endif
@@ -176,6 +180,10 @@
 #define ETHERTYPE_WCP		0x80ff	/* Wellfleet Compression Protocol */
 #endif
 
+#ifndef ETHERTYPE_STP
+#define ETHERTYPE_STP		0x8181	/* STP, HIPPI-ST */
+#endif
+
 #ifndef ETHERTYPE_ISMP
 #define ETHERTYPE_ISMP		0x81fd	/* Cabletron Interswitch Message Protocol */
 #endif
@@ -188,8 +196,8 @@
 #define ETHERTYPE_IPv6		0x86dd
 #endif
 
-#ifndef ETHERTYPE_CISCOWL
-#define ETHERTYPE_CISCOWL	0x872d	/* Cisco Wireless (Aironet) */
+#ifndef ETHERTYPE_WLCCP
+#define ETHERTYPE_WLCCP		0x872d	/* Cisco Wireless Lan Context Control Protocol */
 #endif
 
 #ifndef ETHERTYPE_MAC_CONTROL
@@ -260,6 +268,10 @@
 #define ETHERTYPE_AOE           0x88A2
 #endif
 
+#ifndef ETHERTYPE_EPL_V2
+#define ETHERTYPE_EPL_V2        0x88AB
+#endif
+
 #ifndef ETHERTYPE_BRDWALK
 #define ETHERTYPE_BRDWALK       0x88AE
 #endif
