@@ -1,7 +1,7 @@
 /* filesystem.c
  * Filesystem utility routines
  *
- * $Id: filesystem.c 18268 2006-05-31 17:38:42Z gerald $
+ * $Id: filesystem.c 19293 2006-09-22 22:34:54Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -44,7 +44,7 @@
 #ifdef _WIN32
 #include <windows.h>
 #include <tchar.h>
-#include "epan/strutil.h"
+#include "epan/unicode-utils.h"
 #else
 #include <pwd.h>
 #endif
@@ -503,32 +503,52 @@ const char *
 get_datafile_dir(void)
 {
 #ifdef _WIN32
-	/*
-	 * Do we have the pathname of the program?  If so, assume we're
-	 * running an installed version of the program.  If we fail,
-	 * we don't change "datafile_dir", and thus end up using the
-	 * default.
-	 *
-	 * XXX - does NSIS put the installation directory into
-	 * "\HKEY_LOCAL_MACHINE\SOFTWARE\Wireshark\InstallDir"?
-	 * If so, perhaps we should read that from the registry,
-	 * instead.
-	 */
-	if (progfile_dir != NULL)
-		return progfile_dir;
+	char *u3deviceexecpath;
+#endif
+	static char *datafile_dir = NULL;
 
-	/*
-	 * No, we don't.
-	 * Fall back on the default installation directory.
-	 */
-	return "C:\\Program Files\\Wireshark\\";
+	if (datafile_dir != NULL)
+		return datafile_dir;
+
+#ifdef _WIN32
+
+	u3deviceexecpath = getenv_utf8("U3_DEVICE_EXEC_PATH");
+
+	if (u3deviceexecpath != NULL) {
+		datafile_dir = u3deviceexecpath;
+	} else {
+		/*
+		 * Do we have the pathname of the program?  If so, assume we're
+		 * running an installed version of the program.  If we fail,
+		 * we don't change "datafile_dir", and thus end up using the
+		 * default.
+		 *
+		 * XXX - does NSIS put the installation directory into
+		 * "\HKEY_LOCAL_MACHINE\SOFTWARE\Wireshark\InstallDir"?
+		 * If so, perhaps we should read that from the registry,
+		 * instead.
+		 */
+		if (progfile_dir != NULL) {
+			/*
+			 * Yes, we do; use that.
+			 */
+			datafile_dir = progfile_dir;
+		} else {
+			/*
+			 * No, we don't.
+			 * Fall back on the default installation directory.
+			 */
+			datafile_dir = "C:\\Program Files\\Wireshark\\";
+		}
+	}
 #else
 	/*
 	 * Just use DATAFILE_DIR, as that's what the configure script
 	 * set it to be.
 	 */
-	return DATAFILE_DIR;
+	datafile_dir = DATAFILE_DIR;
 #endif
+	return datafile_dir;
 }
 
 /*
@@ -619,6 +639,7 @@ get_persconffile_dir(void)
 #ifdef _WIN32
 	char *appdatadir;
 	char *userprofiledir;
+	char *u3appdatapath;
 #else
 	const char *homedir;
 	struct passwd *pwd;
@@ -631,35 +652,46 @@ get_persconffile_dir(void)
 
 #ifdef _WIN32
 	/*
-	 * Use %APPDATA% or %USERPROFILE%, so that configuration files are
-	 * stored in the user profile, rather than in the home directory.
-	 * The Windows convention is to store configuration information
-	 * in the user profile, and doing so means you can use
-	 * Wireshark even if the home directory is an inaccessible
-	 * network drive.
+	 * See if we are running in a U3 environment.
 	 */
-	appdatadir = getenv_utf8("APPDATA");
-	if (appdatadir != NULL) {
+	u3appdatapath = getenv_utf8("U3_APP_DATA_PATH");
+	if (u3appdatapath != NULL) {
 		/*
-		 * Concatenate %APPDATA% with "\Wireshark".
+		 * We are; use the U3 application data path.
 		 */
-		pf_dir = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s",
-			appdatadir, PF_DIR);
+		pf_dir = u3appdatapath;
 	} else {
 		/*
-		 * OK, %APPDATA% wasn't set, so use
-		 * %USERPROFILE%\Application Data.
+		 * Use %APPDATA% or %USERPROFILE%, so that configuration
+		 * files are stored in the user profile, rather than in
+		 * the home directory.  The Windows convention is to store
+		 * configuration information in the user profile, and doing
+		 * so means you can use Wireshark even if the home directory
+		 * is an inaccessible network drive.
 		 */
-		userprofiledir = getenv_utf8("USERPROFILE");
-		if (userprofiledir != NULL) {
-			pf_dir = g_strdup_printf(
-			    "%s" G_DIR_SEPARATOR_S "Application Data" G_DIR_SEPARATOR_S "%s",
-			    userprofiledir, PF_DIR);
+		appdatadir = getenv_utf8("APPDATA");
+		if (appdatadir != NULL) {
+			/*
+			 * Concatenate %APPDATA% with "\Wireshark".
+			 */
+			pf_dir = g_strdup_printf("%s" G_DIR_SEPARATOR_S "%s",
+			    appdatadir, PF_DIR);
 		} else {
 			/*
-			 * Give up and use "C:".
+			 * OK, %APPDATA% wasn't set, so use
+			 * %USERPROFILE%\Application Data.
 			 */
-			pf_dir = g_strdup_printf("C:" G_DIR_SEPARATOR_S "%s", PF_DIR);
+			userprofiledir = getenv_utf8("USERPROFILE");
+			if (userprofiledir != NULL) {
+				pf_dir = g_strdup_printf(
+				    "%s" G_DIR_SEPARATOR_S "Application Data" G_DIR_SEPARATOR_S "%s",
+				    userprofiledir, PF_DIR);
+			} else {
+				/*
+				 * Give up and use "C:".
+				 */
+				pf_dir = g_strdup_printf("C:" G_DIR_SEPARATOR_S "%s", PF_DIR);
+			}
 		}
 	}
 #else
