@@ -1,7 +1,7 @@
 /* packet.c
  * Routines for packet disassembly
  *
- * $Id: packet.c 18869 2006-08-10 10:09:31Z jmayer $
+ * $Id: packet.c 19576 2006-10-17 14:13:49Z sahlberg $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -55,6 +55,7 @@
 
 #include <epan/reassemble.h>
 #include <epan/stream.h>
+#include <epan/expert.h>
 
 static gint proto_malformed = -1;
 static dissector_handle_t frame_handle = NULL;
@@ -148,8 +149,11 @@ init_dissection(void)
 	   data structures that "reassemble_init()" frees. */
 	reassemble_init();
 
-	/* Initialise the stream-handling tables */
+	/* Initialize the stream-handling tables */
 	stream_init();
+
+	/* Initialize the expert infos */
+	expert_init();
 }
 
 void
@@ -312,6 +316,7 @@ dissect_packet(epan_dissect_t *edt, union wtap_pseudo_header *pseudo_header,
 	edt->pi.link_number = 0;
 	edt->pi.annex_a_used = MTP2_ANNEX_A_USED_UNKNOWN;
 	edt->pi.profinet_type = 0;
+        edt->pi.usb_conv_info = NULL;
 
 	TRY {
 		edt->tvb = tvb_new_real_data(pd, fd->cap_len, fd->pkt_len);
