@@ -1,7 +1,7 @@
 /* packet_info.h
  * Definitions for packet info structures and routines
  *
- * $Id: packet_info.h 18615 2006-06-29 19:46:51Z ulfl $
+ * $Id: packet_info.h 19576 2006-10-17 14:13:49Z sahlberg $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -85,7 +85,17 @@ typedef struct _packet_info {
 				   finished setting things up, so the TCP
 				   desegmentor can desegment its payload). */
   int desegment_offset;		/* offset to stuff needing desegmentation */
-  guint32 desegment_len;	/* requested desegmentation additional length */
+#define DESEGMENT_ONE_MORE_SEGMENT 0x0fffffff
+#define DESEGMENT_UNTIL_FIN        0x0ffffffe
+  guint32 desegment_len;	/* requested desegmentation additional length
+				   or 
+				   DESEGMENT_ONE_MORE_SEGMENT:
+				     Desegment one more full segment 
+				     (not yet implemented)
+				   DESEGMENT_UNTIL_FIN:
+				     Desgment all data for this tcp session 
+				     until the FIN segment.
+				*/
   guint16 want_pdu_tracking;	/* >0 if the subdissector has specified
 				   a value in 'bytes_until_next_pdu'.
 				   When a dissector detects that the next PDU
@@ -164,6 +174,7 @@ typedef struct _packet_info {
   guint16 link_number;
   gchar   annex_a_used;
   guint16 profinet_type; 	/* the type of PROFINET packet (0: not a PROFINET packet) */
+  void *usb_conv_info;
 } packet_info;
 
 #endif /* __PACKET_INFO_H__ */
