@@ -1,7 +1,7 @@
 /* prefs.c
  * Routines for handling preferences
  *
- * $Id: prefs.c 18494 2006-06-17 11:11:34Z sahlberg $
+ * $Id: prefs.c 19487 2006-10-11 00:37:29Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -50,9 +50,8 @@
 #include <epan/prefs-int.h>
 
 /* Internal functions */
-static module_t *find_module(const char *name);
 static module_t *prefs_register_module_or_subtree(module_t *parent,
-    const char *name, const char *title, gboolean is_subtree,
+    const char *name, const char *title, const char *description, gboolean is_subtree,
     void (*apply_cb)(void));
 static struct preference *find_preference(module_t *, const char *);
 static int    set_pref(gchar*, gchar*);
@@ -139,10 +138,10 @@ module_compare_title(gconstpointer p1_arg, gconstpointer p2_arg)
  */
 module_t *
 prefs_register_module(module_t *parent, const char *name, const char *title,
-    void (*apply_cb)(void))
+    const char *description, void (*apply_cb)(void))
 {
-	return prefs_register_module_or_subtree(parent, name, title, FALSE,
-	    apply_cb);
+	return prefs_register_module_or_subtree(parent, name, title, description,
+	    FALSE, apply_cb);
 }
 
 /*
@@ -152,15 +151,15 @@ prefs_register_module(module_t *parent, const char *name, const char *title,
  * dialog box.
  */
 module_t *
-prefs_register_subtree(module_t *parent, const char *title)
+prefs_register_subtree(module_t *parent, const char *title, const char *description)
 {
-	return prefs_register_module_or_subtree(parent, NULL, title, TRUE,
+	return prefs_register_module_or_subtree(parent, NULL, title, description, TRUE,
 	    NULL);
 }
 
 static module_t *
 prefs_register_module_or_subtree(module_t *parent, const char *name,
-    const char *title, gboolean is_subtree, void (*apply_cb)(void))
+    const char *title, const char *description, gboolean is_subtree, void (*apply_cb)(void))
 {
 	module_t *module;
 	const char *p;
@@ -169,6 +168,7 @@ prefs_register_module_or_subtree(module_t *parent, const char *name,
 	module = g_malloc(sizeof (module_t));
 	module->name = name;
 	module->title = title;
+	module->description = description;
 	module->is_subtree = is_subtree;
 	module->apply_cb = apply_cb;
 	module->prefs = NULL;	/* no preferences, to start */
@@ -208,7 +208,7 @@ prefs_register_module_or_subtree(module_t *parent, const char *name,
 		 * protocol preferences to have a bogus "protocol.", or
 		 * something such as that, to be added to all their names).
 		 */
-		g_assert(find_module(name) == NULL);
+		g_assert(prefs_find_module(name) == NULL);
 
 		/*
 		 * Insert this module in the list of all modules.
@@ -261,12 +261,13 @@ prefs_register_protocol(int id, void (*apply_cb)(void))
 		/*
 		 * No.  Do so.
 		 */
-		protocols_module = prefs_register_subtree(NULL, "Protocols");
+		protocols_module = prefs_register_subtree(NULL, "Protocols", NULL);
 	}
 	protocol = find_protocol_by_id(id);
 	return prefs_register_module(protocols_module,
 	    proto_get_protocol_filter_name(id),
-	    proto_get_protocol_short_name(protocol), apply_cb);
+	    proto_get_protocol_short_name(protocol),
+	    proto_get_protocol_name(id), apply_cb);
 }
 
 /*
@@ -286,12 +287,13 @@ prefs_register_protocol_obsolete(int id)
 		/*
 		 * No.  Do so.
 		 */
-		protocols_module = prefs_register_subtree(NULL, "Protocols");
+		protocols_module = prefs_register_subtree(NULL, "Protocols", NULL);
 	}
 	protocol = find_protocol_by_id(id);
 	module = prefs_register_module(protocols_module,
 	    proto_get_protocol_filter_name(id),
-	    proto_get_protocol_short_name(protocol), NULL);
+	    proto_get_protocol_short_name(protocol),
+	    proto_get_protocol_name(id), NULL);
 	module->obsolete = TRUE;
 	return module;
 }
@@ -308,8 +310,8 @@ module_match(gconstpointer a, gconstpointer b)
 	return strcmp(name, module->name);
 }
 
-static module_t *
-find_module(const char *name)
+module_t *
+prefs_find_module(const char *name)
 {
 	GList *list_entry;
 
@@ -394,6 +396,19 @@ prefs_apply_all(void)
 	g_list_foreach(modules, call_apply_cb, NULL);
 }
 
+/*
+ * Call the "apply" callback function for a specific module if any of
+ * its preferences have changed, and then clear the flag saying its
+ * preferences have changed, as the module has been notified of that
+ * fact.
+ */
+void
+prefs_apply(module_t *module)
+{
+	if (module && module->prefs_changed)
+		call_apply_cb(module, NULL);
+}
+
 /*
  * Register a preference in a module's list of preferences.
  * If it has a title, give it an ordinal number; otherwise, it's a
@@ -491,7 +506,7 @@ find_preference(module_t *module, const char *name)
 gboolean
 prefs_is_registered_protocol(const char *name)
 {
-	module_t *m = find_module(name);
+	module_t *m = prefs_find_module(name);
 
 	return (m != NULL && !m->obsolete);
 }
@@ -502,7 +517,7 @@ prefs_is_registered_protocol(const char *name)
 const char *
 prefs_get_title_by_name(const char *name)
 {
-	module_t *m = find_module(name);
+	module_t *m = prefs_find_module(name);
 
 	return (m != NULL && !m->obsolete) ? m->title : NULL;
 }
@@ -1030,6 +1045,7 @@ init_prefs(void) {
   prefs.gui_fileopen_preview       = 3;
   prefs.gui_ask_unsaved            = TRUE;
   prefs.gui_find_wrap              = TRUE;
+  prefs.gui_use_pref_save          = FALSE;
   prefs.gui_webbrowser             = g_strdup(HTML_VIEWER " %s");
   prefs.gui_window_title           = g_strdup("");
   prefs.gui_layout_type            = layout_type_5;
@@ -1186,7 +1202,7 @@ read_prefs_file(const char *pf_path, FILE *pf, pref_set_pair_cb pref_set_pair_fc
   GString  *cur_var;
   gboolean  got_val = FALSE;
   gint      fline = 1, pline = 1;
-  gchar     hint[] = "(saving your preferences once should remove this warning)";
+  gchar     hint[] = "(applying your preferences once should remove this warning)";
 
   cur_val = g_string_new("");
   cur_var = g_string_new("");
@@ -1377,6 +1393,7 @@ prefs_set_pref(char *prefarg)
 #define PRS_GUI_FILEOPEN_PREVIEW         "gui.fileopen.preview"
 #define PRS_GUI_ASK_UNSAVED              "gui.ask_unsaved"
 #define PRS_GUI_FIND_WRAP                "gui.find_wrap"
+#define PRS_GUI_USE_PREF_SAVE            "gui.use_pref_save"
 #define PRS_GUI_GEOMETRY_SAVE_POSITION   "gui.geometry.save.position"
 #define PRS_GUI_GEOMETRY_SAVE_SIZE       "gui.geometry.save.size"
 #define PRS_GUI_GEOMETRY_SAVE_MAXIMIZED  "gui.geometry.save.maximized"
@@ -1721,6 +1738,13 @@ set_pref(gchar *pref_name, gchar *value)
     else {
 	    prefs.gui_find_wrap = FALSE;
     }
+  } else if (strcmp(pref_name, PRS_GUI_USE_PREF_SAVE) == 0) {
+    if (strcasecmp(value, "true") == 0) {
+	    prefs.gui_use_pref_save = TRUE;
+    }
+    else {
+	    prefs.gui_use_pref_save = FALSE;
+    }
   } else if (strcmp(pref_name, PRS_GUI_WEBBROWSER) == 0) {
     g_free(prefs.gui_webbrowser);
     prefs.gui_webbrowser = g_strdup(value);
@@ -1811,7 +1835,7 @@ set_pref(gchar *pref_name, gchar *value)
             had_a_dot = TRUE;
         }
         *dotp = '\0';		/* separate module and preference name */
-        module = find_module(pref_name);
+        module = prefs_find_module(pref_name);
 
         /*
          * XXX - "Diameter" rather than "diameter" was used in earlier
@@ -1830,14 +1854,14 @@ set_pref(gchar *pref_name, gchar *value)
          */
         if (module == NULL) {
           if (strcmp(pref_name, "Diameter") == 0)
-            module = find_module("diameter");
+            module = prefs_find_module("diameter");
           else if (strcmp(pref_name, "bxxp") == 0)
-            module = find_module("beep");
+            module = prefs_find_module("beep");
           else if (strcmp(pref_name, "gtpv0") == 0 ||
                    strcmp(pref_name, "gtpv1") == 0)
-            module = find_module("gtp");
+            module = prefs_find_module("gtp");
           else if (strcmp(pref_name, "smpp-gsm-sms") == 0)
-            module = find_module("gsm-sms-ud");
+            module = prefs_find_module("gsm-sms-ud");
         }
         *dotp = '.';		/* put the preference string back */
         dotp++;			/* skip past separator to preference name */
@@ -2004,7 +2028,7 @@ set_pref(gchar *pref_name, gchar *value)
           pref = find_preference(module, "desegment_body");
       } else if (strcmp(module->name, "smpp") == 0) {
         /* Handle preferences that moved from SMPP. */
-        module_t *new_module = find_module("gsm-sms-ud");
+        module_t *new_module = prefs_find_module("gsm-sms-ud");
         if(new_module){
           if (strcmp(dotp, "port_number_udh_means_wsp") == 0)
             pref = find_preference(new_module, "port_number_udh_means_wsp");
@@ -2352,6 +2376,11 @@ write_prefs(char **pf_path_return)
   fprintf(pf, PRS_GUI_FIND_WRAP ": %s\n",
 		  prefs.gui_find_wrap == TRUE ? "TRUE" : "FALSE");                  
 
+  fprintf(pf, "\n# Settings dialogs use a save button?\n");
+  fprintf(pf, "# TRUE or FALSE (case-insensitive).\n");
+  fprintf(pf, PRS_GUI_USE_PREF_SAVE ": %s\n",
+		  prefs.gui_use_pref_save == TRUE ? "TRUE" : "FALSE");                  
+
   fprintf(pf, "\n# The path to the webbrowser.\n");
   fprintf(pf, "# Ex: mozilla %%s\n");
   fprintf(pf, PRS_GUI_WEBBROWSER ": %s\n", prefs.gui_webbrowser);
@@ -2565,6 +2594,7 @@ copy_prefs(e_prefs *dest, e_prefs *src)
   dest->gui_fileopen_preview = src->gui_fileopen_preview;
   dest->gui_ask_unsaved = src->gui_ask_unsaved;
   dest->gui_find_wrap = src->gui_find_wrap;
+  dest->gui_use_pref_save = src->gui_use_pref_save;
   dest->gui_layout_type = src->gui_layout_type;
   dest->gui_layout_content_1 = src->gui_layout_content_1;
   dest->gui_layout_content_2 = src->gui_layout_content_2;
@@ -2653,3 +2683,4 @@ free_col_info(e_prefs *pr)
   pr->col_list = NULL;
 }
 
+
