@@ -1,7 +1,7 @@
 /* proto.h
  * Definitions for protocol display
  *
- * $Id: proto.h 18527 2006-06-21 07:26:39Z ulfl $
+ * $Id: proto.h 19383 2006-09-30 14:37:09Z gal $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -211,6 +211,8 @@ typedef struct field_info {
 /** The protocol field should be displayed as "generated by Wireshark",
  * used in field_info.flags. */
 #define FI_GENERATED		0x0002
+/* The protocol field is actually a URL */
+#define FI_URL                  0x0004
 
 
 /** convenience macro to get field_info.flags */
@@ -223,6 +225,7 @@ typedef struct field_info {
 typedef struct {
     GHashTable  *interesting_hfids;
     gboolean    visible;
+    gint        count;
 } tree_data_t;
 
 /** Each proto_tree, proto_item is one of these. */
@@ -289,6 +292,12 @@ typedef proto_node proto_item;
 /** mark this protocol field as generated by Wireshark (and not read from the packet data) */
 #define PROTO_ITEM_SET_GENERATED(proto_item)	\
 	((proto_item) ? FI_SET_FLAG((proto_item)->finfo, FI_GENERATED) : 0)
+/* is this protocol field actually a URL */
+#define PROTO_ITEM_IS_URL(proto_item)	\
+	((proto_item) ? FI_GET_FLAG((proto_item)->finfo, FI_URL) : 0)
+/* mark this protocol field as a URL */
+#define PROTO_ITEM_SET_URL(proto_item)	\
+	((proto_item) ? FI_SET_FLAG((proto_item)->finfo, FI_URL) : 0)
 
 typedef void (*proto_tree_foreach_func)(proto_node *, gpointer);
 
@@ -427,6 +436,11 @@ proto_tree_prime_hfid(proto_tree *tree, int hfid);
  @return parent item */
 extern proto_item* proto_tree_get_parent(proto_tree *tree);
 
+/** Get the root tree from any subtree.
+ @param tree the tree to get the root from
+ @return root tree */
+extern proto_tree* proto_tree_get_root(proto_tree *tree);
+
 /** Move an existing item behind another existing item.
  @param tree the tree to which both items belong
  @param fixed_item the item which keeps it's position
