@@ -1,7 +1,7 @@
 /* strutil.c
  * String utility routines
  *
- * $Id: strutil.c 18519 2006-06-19 15:53:03Z etxrab $
+ * $Id: strutil.c 19592 2006-10-18 17:55:29Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -262,7 +262,7 @@ format_text(const guchar *string, int len)
 
 /*
  * Given a string, generate a string from it that shows non-printable
- * characters as C-style escapes except a whitespace character 
+ * characters as C-style escapes except a whitespace character
  * (space, tab, carriage return, new line, vertical tab, or formfeed)
  * which will be replaved by a space, and return a pointer to it.
  */
@@ -431,6 +431,9 @@ hex_str_to_bytes(const char *hex_str, GByteArray *bytes, gboolean force_separato
 	char		one_digit[2];
 
 	g_byte_array_set_size(bytes, 0);
+	if (! hex_str) {
+		return FALSE;
+	}
 	p = (const guchar *)hex_str;
 	while (*p) {
 		q = p+1;
@@ -765,90 +768,3 @@ g_strlcat(gchar *dst, gchar *src, gsize size)
 	return strl+strs;
 }
 #endif
-
-#ifdef _WIN32
-
-/*
- * XXX - Should we use g_utf8_to_utf16() and g_utf16_to_utf8()
- * instead?  The goal of the functions below was to provide simple
- * wrappers for UTF-8 <-> UTF-16 conversion without making the
- * caller worry about freeing up memory afterward.
- */
-
-/* Convert from UTF-8 to UTF-16. */
-wchar_t * utf_8to16(const char *utf8str) {
-  static wchar_t *utf16buf[3];
-  static int utf16buf_len[3];
-  static int idx;
-
-  if (utf8str == NULL)
-    return NULL;
-
-  idx = (idx + 1) % 3;
-
-  /*
-   * Allocate the buffer if it's not already allocated.
-   */
-  if (utf16buf[idx] == NULL) {
-    utf16buf_len[idx] = INITIAL_FMTBUF_SIZE;
-    utf16buf[idx] = g_malloc(utf16buf_len[idx] * sizeof(wchar_t));
-  }
-
-  while (MultiByteToWideChar(CP_UTF8, 0, utf8str,
-      -1, NULL, 0) >= utf16buf_len[idx]) {
-    /*
-     * Double the buffer's size if it's not big enough.
-     * The size of the buffer starts at 128, so doubling its size
-     * adds at least another 128 bytes, which is more than enough
-     * for one more character plus a terminating '\0'.
-     */
-    utf16buf_len[idx] *= 2;
-    utf16buf[idx] = g_realloc(utf16buf[idx], utf16buf_len[idx] * sizeof(wchar_t));
-  }
-
-  if (MultiByteToWideChar(CP_UTF8, 0, utf8str,
-      -1, utf16buf[idx], utf16buf_len[idx]) == 0)
-    return NULL;
-
-  return utf16buf[idx];
-}
-
-/* Convert from UTF-16 to UTF-8. */
-gchar * utf_16to8(const wchar_t *utf16str) {
-  static gchar *utf8buf[3];
-  static int utf8buf_len[3];
-  static int idx;
-
-  if (utf16str == NULL)
-    return NULL;
-
-  idx = (idx + 1) % 3;
-
-  /*
-   * Allocate the buffer if it's not already allocated.
-   */
-  if (utf8buf[idx] == NULL) {
-    utf8buf_len[idx] = INITIAL_FMTBUF_SIZE;
-    utf8buf[idx] = g_malloc(utf8buf_len[idx]);
-  }
-
-  while (WideCharToMultiByte(CP_UTF8, 0, utf16str, -1,
-      NULL, 0, NULL, NULL) >= utf8buf_len[idx]) {
-    /*
-     * Double the buffer's size if it's not big enough.
-     * The size of the buffer starts at 128, so doubling its size
-     * adds at least another 128 bytes, which is more than enough
-     * for one more character plus a terminating '\0'.
-     */
-    utf8buf_len[idx] *= 2;
-    utf8buf[idx] = g_realloc(utf8buf[idx], utf8buf_len[idx]);
-  }
-
-  if (WideCharToMultiByte(CP_UTF8, 0, utf16str, -1,
-      utf8buf[idx], utf8buf_len[idx], NULL, NULL) == 0)
-    return NULL;
-
-  return utf8buf[idx];
-}
-
-#endif
