@@ -1,7 +1,7 @@
 /* to_str.c
  * Routines for utilities to convert various other types to strings.
  *
- * $Id: to_str.c 18197 2006-05-21 05:12:17Z sahlberg $
+ * $Id: to_str.c 19524 2006-10-14 05:02:40Z sahlberg $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -316,6 +316,17 @@ vines_addr_to_str_buf(const guint8 *addrp, gchar *buf, int buf_len)
   g_snprintf(buf, buf_len, "%08x.%04x", pntohl(&addrp[0]), pntohs(&addrp[4]));
 }
 
+
+void
+usb_addr_to_str_buf(const guint8 *addrp, gchar *buf, int buf_len)
+{
+  if(pletohl(&addrp[0])==0xffffffff){
+    g_snprintf(buf, buf_len, "host");
+  } else {
+    g_snprintf(buf, buf_len, "%d", pletohl(&addrp[0]));
+  }
+}
+
 #define	PLURALIZE(n)	(((n) > 1) ? "s" : "")
 #define	COMMA(do_it)	((do_it) ? ", " : "")
 
@@ -799,6 +810,9 @@ address_to_str_buf(const address *addr, gchar *buf, int buf_len)
   case AT_VINES:
     vines_addr_to_str_buf(addr->data, buf, buf_len);
     break;
+  case AT_USB:
+    usb_addr_to_str_buf(addr->data, buf, buf_len);
+    break;
   case AT_OSI:
     print_nsap_net_buf(addr->data, addr->len, buf, buf_len);
     break;
