@@ -1,7 +1,7 @@
 /* capture_dlg.c
  * Routines for packet capture windows
  *
- * $Id: capture_dlg.c 18928 2006-08-16 18:01:25Z gerald $
+ * $Id: capture_dlg.c 19401 2006-10-02 23:44:10Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -550,8 +550,7 @@ capture_prep_cb(GtkWidget *w _U_, gpointer d _U_)
                 *snap_hb, *snap_cb, *snap_sb, *snap_lb,
                 *promisc_cb,
                 *filter_hb, *filter_bt, *filter_te, *filter_cm,
-				*advanced_hb,
-
+				
                 *file_fr, *file_vb,
                 *file_hb, *file_bt, *file_lb, *file_te,
                 *multi_tb, *multi_files_on_cb,
@@ -571,8 +570,10 @@ capture_prep_cb(GtkWidget *w _U_, gpointer d _U_)
                 *resolv_fr, *resolv_vb,
                 *m_resolv_cb, *n_resolv_cb, *t_resolv_cb,
                 *bbox, *ok_bt, *cancel_bt,
-                *help_bt,
-				*advanced_bt;
+                *help_bt;
+#ifdef HAVE_AIRPCAP
+  GtkWidget     *advanced_hb, *advanced_bt;
+#endif
 #if GTK_MAJOR_VERSION < 2
   GtkAccelGroup *accel_group;
 #endif
@@ -592,9 +593,6 @@ capture_prep_cb(GtkWidget *w _U_, gpointer d _U_)
   gchar         *cap_title;
   gchar         *if_device;
 
-  gint *from_widget = NULL;
-
-
   if (cap_open_w != NULL) {
     /* There's already a "Capture Options" dialog box; reactivate it. */
     reactivate_window(cap_open_w);
@@ -848,20 +846,21 @@ capture_prep_cb(GtkWidget *w _U_, gpointer d _U_)
   OBJECT_SET_DATA(filter_bt, E_FILT_TE_PTR_KEY, filter_te);
 
   /* advanced row */
+#ifdef HAVE_AIRPCAP
   advanced_hb = gtk_hbox_new(FALSE,5);
   gtk_box_pack_start(GTK_BOX(capture_vb), advanced_hb, FALSE, FALSE, 0);
-  advanced_bt = gtk_button_new();
+
+  advanced_bt = gtk_button_new_with_label("Wireless Settings");
 
   /* set the text */
   #if GTK_MAJOR_VERSION >= 2
-  /* XXX - find a way to set the GtkButton label in GTK 1.x */
+  /* XXX - find a way to set the GtkButton label in GTK 2.x */
   gtk_button_set_label(GTK_BUTTON(advanced_bt), "Wireless Settings");
   #else
   /* Set the GtkButton label in GTK 1.x */
   gtk_label_set_text(GTK_LABEL(GTK_BIN(advanced_bt)->child), "Wireless Settings");
   #endif
 
-#ifdef HAVE_AIRPCAP
   /* Both the callback and the data are global */
   SIGNAL_CONNECT(advanced_bt,"clicked",options_airpcap_advanced_cb,airpcap_tb);
   OBJECT_SET_DATA(GTK_ENTRY(GTK_COMBO(if_cb)->entry),AIRPCAP_OPTIONS_ADVANCED_KEY,advanced_bt);
@@ -875,11 +874,11 @@ capture_prep_cb(GtkWidget *w _U_, gpointer d _U_)
 	{
 	gtk_widget_set_sensitive(advanced_bt,FALSE);
 	}
-#endif
 
   gtk_box_pack_start(GTK_BOX(linktype_hb),advanced_bt,FALSE,FALSE,0);
   gtk_widget_show(advanced_bt);
   gtk_widget_show(advanced_hb);
+#endif
 
   /* Capture file-related options frame */
   file_fr = gtk_frame_new("Capture File(s)");
