@@ -7,7 +7,7 @@
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com> and
  * Guy Harris <guy@alum.mit.edu>
  *
- * $Id: dfilter_expr_dlg.c 18205 2006-05-22 07:29:40Z guy $
+ * $Id: dfilter_expr_dlg.c 19676 2006-10-24 22:21:04Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 /* Todo -
@@ -696,6 +696,7 @@ value_list_sel_cb(GtkTreeSelection *sel, gpointer value_entry_arg)
 
             case FT_UINT8:
             case FT_UINT16:
+            case FT_UINT24:
             case FT_UINT32:
                 g_snprintf(value_string, sizeof value_string,
                          "%u", value->value);
@@ -703,6 +704,7 @@ value_list_sel_cb(GtkTreeSelection *sel, gpointer value_entry_arg)
 
             case FT_INT8:
             case FT_INT16:
+            case FT_INT24:
             case FT_INT32:
                 g_snprintf(value_string, sizeof value_string,
                          "%d", value->value);
