@@ -4,7 +4,7 @@
  * Helper routines common to all composite expert statistics
  * tap.
  *
- * $Id: expert_comp_table.c 18205 2006-05-22 07:29:40Z guy $
+ * $Id: expert_comp_table.c 19568 2006-10-17 01:29:55Z ulfl $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -96,20 +96,6 @@ gpointer userdata)
 }
 #endif
 
-/* XXX - move this to a common header file */
-static const value_string expert_group_vals[] = {
-	{ PI_CHECKSUM,		"Checksum" },
-	{ PI_SEQUENCE,		"Sequence" },
-	{ PI_RESPONSE_CODE, "Response" },
-    { PI_REQUEST_CODE,  "Request" },
-	{ PI_UNDECODED,		"Undecoded" },
-	{ PI_MALFORMED,		"Malformed" },
-	{ PI_REASSEMBLE,	"Reassemble" },
-/*	{ PI_SECURITY,		"Security" },*/
-	{ 0, NULL }
-};
-
-
 #if (GTK_MAJOR_VERSION < 2)
 
 typedef struct column_arrows {
@@ -491,7 +477,7 @@ static GtkItemFactoryEntry error_list_menu_items[] =
 		error_select_filter_cb, 6*256+0, NULL, NULL),
 #if (GTK_MAJOR_VERSION < 2)
    	/* Go to first packet matching this entry */
-	ITEM_FACTORY_ENTRY("/Goto First Occurance", NULL,
+	ITEM_FACTORY_ENTRY("/Goto First Occurrence", NULL,
 		error_select_filter_cb, 7*256+0, NULL, NULL),
 #endif
 };
