@@ -1,7 +1,7 @@
 /* filter_dlg.c
  * Dialog boxes for (display and capture) filter editing
  *
- * $Id: filter_dlg.c 18232 2006-05-28 17:19:08Z etxrab $
+ * $Id: filter_dlg.c 19418 2006-10-03 21:03:08Z ulfl $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -42,6 +42,7 @@
 #include "compat_macros.h"
 #include "gtkglobals.h"
 #include "help_dlg.h"
+#include <epan/prefs.h>
 
 #define E_FILT_DIALOG_PTR_KEY       "filter_dialog_ptr"
 #define E_FILT_BUTTON_PTR_KEY       "filter_button_ptr"
@@ -64,16 +65,22 @@ typedef struct _filter_cb_data {
 } filter_cb_data;
 
 static GtkWidget *filter_dialog_new(GtkWidget *button, GtkWidget *filter_te,
-                                    filter_list_type_t list,
+                                    filter_list_type_t list_type,
                                     construct_args_t *construct_args);
 static void filter_dlg_dclick(GtkWidget *dummy, gpointer main_w_arg,
 			      gpointer activate);
-static void filter_dlg_ok_cb(GtkWidget *ok_bt, gpointer dummy);
-static void filter_dlg_apply_cb(GtkWidget *apply_bt, gpointer dummy);
+static void filter_dlg_ok_cb(GtkWidget *ok_bt, gpointer data);
+static void filter_dlg_apply_cb(GtkWidget *apply_bt, gpointer data);
 static void filter_apply(GtkWidget *main_w, gboolean destroy);
+static void filter_dlg_save(filter_list_type_t list_type);
 static void filter_dlg_save_cb(GtkWidget *save_bt, gpointer parent_w);
 static void filter_dlg_destroy_cb(GtkWidget *win, gpointer data);
 
+static gboolean
+filter_dlg_delete_event_cb(GtkWidget *prefs_w, GdkEvent *event, gpointer data);
+static void
+filter_dlg_cancel_cb(GtkWidget *cancel_bt, gpointer data);
+
 static gint filter_sel_list_button_cb(GtkWidget *, GdkEventButton *,
                                       gpointer);
 #if GTK_MAJOR_VERSION < 2
@@ -249,15 +256,15 @@ remember_filter_dialog(GtkWidget *main_w, GList **filter_dialogs)
 
 /* Remove a filter dialog from the specified list of filter_dialogs. */
 static void
-forget_filter_dialog(GtkWidget *main_w, filter_list_type_t list)
+forget_filter_dialog(GtkWidget *main_w, filter_list_type_t list_type)
 {
-	switch (list) {
+	switch (list_type) {
 
-	case CFILTER_LIST:
+	case CFILTER_EDITED_LIST:
 		cfilter_dialogs = g_list_remove(cfilter_dialogs, main_w);
 		break;
 
-	case DFILTER_LIST:
+	case DFILTER_EDITED_LIST:
 		dfilter_dialogs = g_list_remove(dfilter_dialogs, main_w);
 		break;
 
@@ -269,14 +276,14 @@ forget_filter_dialog(GtkWidget *main_w, filter_list_type_t list)
 
 /* Get the dialog list corresponding to a particular filter list. */
 static GList *
-get_filter_dialog_list(filter_list_type_t list)
+get_filter_dialog_list(filter_list_type_t list_type)
 {
-	switch (list) {
+	switch (list_type) {
 
-	case CFILTER_LIST:
+	case CFILTER_EDITED_LIST:
 		return cfilter_dialogs;
 
-	case DFILTER_LIST:
+	case DFILTER_EDITED_LIST:
 		return dfilter_dialogs;
 
 	default:
@@ -285,9 +292,87 @@ get_filter_dialog_list(filter_list_type_t list)
 	}
 }
 
+
+static void
+fill_list(GtkWidget  *main_w, filter_list_type_t list_type)
+{
+    GList      *fl_entry;
+    filter_def *filt;
+    const gchar *filter_te_str = NULL;
+#if GTK_MAJOR_VERSION < 2
+    GtkWidget  *nl_item,
+               *nl_lb,
+               *l_select = NULL;
+    GtkWidget  *filter_l = OBJECT_GET_DATA(main_w, E_FILT_FILTER_L_KEY);
+#else
+    gboolean           l_select = FALSE;
+    GtkTreeView        *filter_l;
+    GtkListStore       *store;
+    GtkTreeIter        iter;
+    GtkTreeIter        sel_iter;
+
+    filter_l = GTK_TREE_VIEW(OBJECT_GET_DATA(main_w, E_FILT_FILTER_L_KEY));
+    store = GTK_LIST_STORE(gtk_tree_view_get_model(filter_l));
+#endif
+
+    /* fill in data */
+    fl_entry = get_filter_list_first(list_type);
+    while (fl_entry != NULL) {
+        filt    = (filter_def *) fl_entry->data;
+#if GTK_MAJOR_VERSION < 2
+        nl_lb   = gtk_label_new(filt->name);
+        nl_item = gtk_list_item_new();
+
+        SIGNAL_CONNECT(nl_item, "button_press_event", filter_sel_list_button_cb,
+                       filter_l);
+
+        gtk_misc_set_alignment (GTK_MISC (nl_lb), 0.0, 0.5);
+        gtk_container_add(GTK_CONTAINER(nl_item), nl_lb);
+        gtk_widget_show(nl_lb);
+        gtk_container_add(GTK_CONTAINER(filter_l), nl_item);
+        gtk_widget_show(nl_item);
+        OBJECT_SET_DATA(nl_item, E_FILT_LBL_KEY, nl_lb);
+        OBJECT_SET_DATA(nl_item, E_FILT_LIST_ITEM_MODEL_KEY, fl_entry);
+#else
+        gtk_list_store_append(store, &iter);
+        gtk_list_store_set(store, &iter, 0, filt->name,
+                           1, fl_entry, -1);
+#endif
+
+        if (filter_te_str && filt->strval) {
+            if (strcmp(filter_te_str, filt->strval) == 0) {
+#if GTK_MAJOR_VERSION < 2 
+                l_select = nl_item;
+#else
+                sel_iter = iter;
+                l_select = TRUE;
+#endif
+            }
+        }
+
+        fl_entry = fl_entry->next;
+    }
+}
+
+#if 0
+static void
+clear_list(GtkWidget *main_w) {
+    GtkWidget    *filter_l = OBJECT_GET_DATA(main_w, E_FILT_FILTER_L_KEY);
+#if GTK_MAJOR_VERSION >= 2
+    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(filter_l));
+#endif
+
+#if GTK_MAJOR_VERSION < 2
+    gtk_list_clear_items(GTK_LIST(filter_l), 0, -1);
+#else
+    gtk_list_store_clear(GTK_LIST_STORE(model));
+#endif
+}
+#endif /* 0 */
+
 static GtkWidget *
 filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
-                  filter_list_type_t list, construct_args_t *construct_args)
+                  filter_list_type_t list_type, construct_args_t *construct_args)
 {
     GtkWidget  *main_w,           /* main window */
                *main_vb,          /* main container */
@@ -295,7 +380,7 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
                *ok_bt,            /* "OK" button */
                *apply_bt,         /* "Apply" button */
                *save_bt,          /* "Save" button */
-               *close_bt,         /* "Cancel" button */
+               *cancel_bt,        /* "Cancel" button */
                *help_bt;          /* "Help" button */
     GtkWidget  *filter_vb,        /* filter settings box */
                *props_vb;
@@ -316,40 +401,37 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
                *edit_fr,
                *props_fr;
     GtkTooltips *tooltips;
-    GList      *fl_entry;
-    filter_def *filt;
-    static filter_list_type_t cfilter_list_type = CFILTER_LIST;
-    static filter_list_type_t dfilter_list_type = DFILTER_LIST;
+    static filter_list_type_t cfilter_list_type = CFILTER_EDITED_LIST;
+    static filter_list_type_t dfilter_list_type = DFILTER_EDITED_LIST;
     filter_list_type_t *filter_list_type_p;
     GList       **filter_dialogs;
     const gchar *filter_te_str = NULL;
 #if GTK_MAJOR_VERSION < 2
-    GtkWidget  *nl_item,
-               *nl_lb,
-               *l_select = NULL;
+    GtkWidget   *l_select = NULL;
 #else
     gboolean           l_select = FALSE;
     GtkListStore      *store;
     GtkCellRenderer   *renderer;
     GtkTreeViewColumn *column;
     GtkTreeSelection  *sel;
-    GtkTreeIter        iter;
     GtkTreeIter        sel_iter;
 #endif
 
     /* Get a pointer to a static variable holding the type of filter on
        which we're working, so we can pass that pointer to callback
        routines. */
-    switch (list) {
+    switch (list_type) {
 
     case CFILTER_LIST:
         filter_dialogs = &cfilter_dialogs;
         filter_list_type_p = &cfilter_list_type;
+        list_type = CFILTER_EDITED_LIST;
         break;
 
     case DFILTER_LIST:
         filter_dialogs = &dfilter_dialogs;
         filter_list_type_p = &dfilter_list_type;
+        list_type = DFILTER_EDITED_LIST;
         break;
 
     default:
@@ -472,42 +554,8 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
                     construct_args->activate_on_ok ? "" : NULL);
 
     /* fill in data */
-    fl_entry = get_filter_list_first(list);
-    while (fl_entry != NULL) {
-        filt    = (filter_def *) fl_entry->data;
-#if GTK_MAJOR_VERSION < 2
-        nl_lb   = gtk_label_new(filt->name);
-        nl_item = gtk_list_item_new();
+    fill_list(main_w, list_type);
 
-        SIGNAL_CONNECT(nl_item, "button_press_event", filter_sel_list_button_cb,
-                       filter_l);
-
-        gtk_misc_set_alignment (GTK_MISC (nl_lb), 0.0, 0.5);
-        gtk_container_add(GTK_CONTAINER(nl_item), nl_lb);
-        gtk_widget_show(nl_lb);
-        gtk_container_add(GTK_CONTAINER(filter_l), nl_item);
-        gtk_widget_show(nl_item);
-        OBJECT_SET_DATA(nl_item, E_FILT_LBL_KEY, nl_lb);
-        OBJECT_SET_DATA(nl_item, E_FILT_LIST_ITEM_MODEL_KEY, fl_entry);
-#else
-        gtk_list_store_append(store, &iter);
-        gtk_list_store_set(store, &iter, 0, filt->name,
-                           1, fl_entry, -1);
-#endif
-
-        if (filter_te_str && filt->strval) {
-            if (strcmp(filter_te_str, filt->strval) == 0) {
-#if GTK_MAJOR_VERSION < 2 
-                l_select = nl_item;
-#else
-                sel_iter = iter;
-                l_select = TRUE;
-#endif
-            }
-        }
-
-        fl_entry = fl_entry->next;
-    }
 #if GTK_MAJOR_VERSION >= 2 
     g_object_unref(G_OBJECT(store));
 #endif
@@ -554,7 +602,7 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
 
     OBJECT_SET_DATA(main_w, E_FILT_PARENT_FILTER_TE_KEY, parent_filter_te);
 
-    if (list == DFILTER_LIST) {
+    if (list_type == DFILTER_EDITED_LIST) {
         gtk_tooltips_set_tip(tooltips, filter_te, 
             "Enter a display filter. "
             "The background color of this field is changed by a continuous syntax check (green is valid, red is invalid).", 
@@ -570,78 +618,51 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
     }
 
 
-    /* button row */
-    if (parent_filter_te != NULL) {
-        if (construct_args->wants_apply_button) {
-            bbox = dlg_button_row_new(GTK_STOCK_OK, GTK_STOCK_APPLY, GTK_STOCK_SAVE, GTK_STOCK_CLOSE, GTK_STOCK_HELP, NULL);
-        } else {
-            bbox = dlg_button_row_new(GTK_STOCK_OK, GTK_STOCK_SAVE, GTK_STOCK_CLOSE, GTK_STOCK_HELP, NULL);
-        }
-    } else {
-        if (construct_args->wants_apply_button) {
-            bbox = dlg_button_row_new(GTK_STOCK_APPLY, GTK_STOCK_SAVE, GTK_STOCK_CLOSE, GTK_STOCK_HELP, NULL);
-        } else {
-            bbox = dlg_button_row_new(GTK_STOCK_SAVE, GTK_STOCK_CLOSE, GTK_STOCK_HELP, NULL);
-        }
-    }
+    /* button row (create all possible buttons and hide the unrequired later - it's a lot easier) */
+    bbox = dlg_button_row_new(GTK_STOCK_OK, GTK_STOCK_APPLY, GTK_STOCK_SAVE, GTK_STOCK_CANCEL, GTK_STOCK_HELP, NULL);
     gtk_box_pack_start(GTK_BOX(main_vb), bbox, FALSE, FALSE, 5);
     gtk_widget_show(bbox);
 
-    ok_bt = NULL;
-    if (parent_filter_te != NULL) {
-        /*
-         * We have a filter text entry that we can fill in if
-         * the "OK" button is clicked, so put in an "OK" button.
-         */
-        ok_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_OK);
-        SIGNAL_CONNECT(ok_bt, "clicked", filter_dlg_ok_cb, NULL);
-        gtk_tooltips_set_tip (tooltips, ok_bt, ("Apply the filters and close this dialog"), NULL);
+    ok_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_OK);
+    SIGNAL_CONNECT(ok_bt, "clicked", filter_dlg_ok_cb, filter_list_type_p);
+    gtk_tooltips_set_tip (tooltips, ok_bt, ("Apply the filters and close this dialog"), NULL);
 
-        /* Catch the "activate" signal on the filter name and filter
-           expression text entries, so that if the user types Return
-           there, we act as if the "OK" button had been selected, as
-           happens if Return is typed if some widget that *doesn't*
-           handle the Return key has the input focus. */
+    /* Catch the "activate" signal on the filter name and filter
+       expression text entries, so that if the user types Return
+       there, we act as if the "OK" button had been selected, as
+       happens if Return is typed if some widget that *doesn't*
+       handle the Return key has the input focus. */
+    if (parent_filter_te != NULL) {
         dlg_set_activate(name_te, ok_bt);
         dlg_set_activate(filter_te, ok_bt);
     }
 
-    if (construct_args->wants_apply_button) {
-        apply_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_APPLY);
-        SIGNAL_CONNECT(apply_bt, "clicked", filter_dlg_apply_cb, NULL);
-        gtk_tooltips_set_tip (tooltips, apply_bt, ("Apply the filters and keep this dialog open"), NULL);
-    }
+    apply_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_APPLY);
+    SIGNAL_CONNECT(apply_bt, "clicked", filter_dlg_apply_cb, filter_list_type_p);
+    gtk_tooltips_set_tip (tooltips, apply_bt, ("Apply the filters and keep this dialog open"), NULL);
 
     save_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_SAVE);
     SIGNAL_CONNECT(save_bt, "clicked", filter_dlg_save_cb, filter_list_type_p);
     gtk_tooltips_set_tip (tooltips, save_bt, ("Save the filters permanently and keep this dialog open"), NULL);
 
-    close_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
-    gtk_tooltips_set_tip (tooltips, close_bt, ("Close this dialog but don't apply the filter changes"), NULL);
-    if (parent_filter_te == NULL)
-        gtk_widget_grab_default(close_bt);
+    cancel_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
+    gtk_tooltips_set_tip (tooltips, cancel_bt, ("Cancel the changes"), NULL);
+    SIGNAL_CONNECT(cancel_bt, "clicked", filter_dlg_cancel_cb, filter_list_type_p);
 
     help_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_HELP);
-    if (list == CFILTER_LIST) {
+    if (list_type == CFILTER_EDITED_LIST) {
         SIGNAL_CONNECT(help_bt, "clicked", topic_cb, HELP_CAPTURE_FILTERS_DIALOG);
     } else {
         SIGNAL_CONNECT(help_bt, "clicked", topic_cb, HELP_DISPLAY_FILTERS_DIALOG);
     }
     gtk_tooltips_set_tip (tooltips, help_bt, ("Show topic specific help"), NULL);
 
-    window_set_cancel_button(main_w, close_bt, window_cancel_button_cb);
-
     if(ok_bt) {
         gtk_widget_grab_default(ok_bt);
     }
 
     remember_filter_dialog(main_w, filter_dialogs);
 
-    SIGNAL_CONNECT(main_w, "delete_event", window_delete_event_cb, NULL);
-    /* Call a handler when we're destroyed, so we can detach ourselves
-       from the button. */
-    SIGNAL_CONNECT(main_w, "destroy", filter_dlg_destroy_cb, filter_list_type_p);
-
     if (button != NULL) {
 	/* This dialog box was created by a "Filter" button.
 	   Set the E_FILT_BUTTON_PTR_KEY for the new dialog to point to
@@ -670,7 +691,26 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
     }
 #endif
 
+    SIGNAL_CONNECT(main_w, "delete_event", filter_dlg_delete_event_cb, filter_list_type_p);
+    SIGNAL_CONNECT(main_w, "destroy", filter_dlg_destroy_cb, filter_list_type_p);
+
     gtk_widget_show(main_w);
+
+    /* hide the Ok button, if we don't have to apply it and our caller wants a Save button */
+    if (parent_filter_te == NULL && prefs.gui_use_pref_save) {
+        gtk_widget_hide(ok_bt);
+    }
+
+    /* hide the Apply button, if our caller don't wants one */
+    if (!construct_args->wants_apply_button) {
+        gtk_widget_hide(apply_bt);
+    }
+
+    /* hide the Save button if the user uses implicit save */
+    if (!prefs.gui_use_pref_save) {
+        gtk_widget_hide(save_bt);
+    }
+
     window_present(main_w);
 
     return main_w;
@@ -742,21 +782,35 @@ filter_dlg_dclick(GtkWidget *filter_l, gpointer main_w_arg, gpointer activate)
 }
 
 static void
-filter_dlg_ok_cb(GtkWidget *ok_bt, gpointer data _U_)
+filter_dlg_ok_cb(GtkWidget *ok_bt, gpointer data)
 {
+	filter_list_type_t list_type = *(filter_list_type_t *)data;
+
 	/*
 	 * Destroy the dialog box and apply the filter.
 	 */
 	filter_apply(gtk_widget_get_toplevel(ok_bt), TRUE);
+
+	/* if we don't have a Save button, just save the settings now */
+	if (!prefs.gui_use_pref_save) {
+		filter_dlg_save(list_type);
+	}
 }
 
 static void
-filter_dlg_apply_cb(GtkWidget *apply_bt, gpointer dummy _U_)
+filter_dlg_apply_cb(GtkWidget *apply_bt, gpointer data)
 {
+	filter_list_type_t list_type = *(filter_list_type_t *)data;
+
 	/*
 	 * Apply the filter, but don't destroy the dialog box.
 	 */
 	filter_apply(gtk_widget_get_toplevel(apply_bt), FALSE);
+
+	/* if we don't have a Save button, just save the settings now */
+	if (!prefs.gui_use_pref_save) {
+		filter_dlg_save(list_type);
+	}
 }
 
 static void
@@ -815,14 +869,34 @@ filter_apply(GtkWidget *main_w, gboolean destroy)
 	}
 }
 
+
 static void
-filter_dlg_save_cb(GtkWidget *save_bt _U_, gpointer data)
+filter_dlg_save(filter_list_type_t list_type)
 {
-	filter_list_type_t list = *(filter_list_type_t *)data;
 	char *pf_dir_path;
 	char *f_path;
 	int f_save_errno;
-	const char *filter_type;
+        const char *filter_type;
+
+	switch (list_type) {
+
+	case CFILTER_EDITED_LIST:
+		filter_type = "capture";
+                list_type = CFILTER_LIST;
+                copy_filter_list(CFILTER_LIST, CFILTER_EDITED_LIST);
+		break;
+
+	case DFILTER_EDITED_LIST:
+		filter_type = "display";
+                list_type = DFILTER_LIST;
+                copy_filter_list(DFILTER_LIST, DFILTER_EDITED_LIST);
+		break;
+
+	default:
+		g_assert_not_reached();
+		filter_type = NULL;
+		break;
+	}
 
 	/* Create the directory that holds personal configuration files,
 	   if necessary.  */
@@ -834,24 +908,9 @@ filter_dlg_save_cb(GtkWidget *save_bt _U_, gpointer data)
 		return;
 	}
 
-	save_filter_list(list, &f_path, &f_save_errno);
+	save_filter_list(list_type, &f_path, &f_save_errno);
 	if (f_path != NULL) {
 		/* We had an error saving the filter. */
-		switch (list) {
-
-		case CFILTER_LIST:
-			filter_type = "capture";
-			break;
-
-		case DFILTER_LIST:
-			filter_type = "display";
-			break;
-
-		default:
-			g_assert_not_reached();
-			filter_type = NULL;
-			break;
-		}
 		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		    "Could not save to your %s filter file\n\"%s\": %s.",
 		    filter_type, f_path, strerror(f_save_errno));
@@ -859,10 +918,76 @@ filter_dlg_save_cb(GtkWidget *save_bt _U_, gpointer data)
 	}
 }
 
+
+static void
+filter_dlg_save_cb(GtkWidget *save_bt _U_, gpointer data)
+{
+	filter_list_type_t list_type = *(filter_list_type_t *)data;
+
+	filter_dlg_save(list_type);
+}
+
+#if 0
+/* update a remaining dialog if another one was cancelled */
+static void filter_dlg_update_list_cb(gpointer data, gpointer user_data)
+{
+    GtkWidget  *main_w = data;
+    filter_list_type_t list_type = *(filter_list_type_t *)user_data;
+
+    /* refill the list */
+    clear_list(main_w);
+    fill_list(main_w, list_type);
+}
+#endif
+
+/* cancel button pressed, revert changes and exit dialog */
+static void
+filter_dlg_cancel_cb(GtkWidget *cancel_bt, gpointer data)
+{
+    filter_list_type_t list_type = *(filter_list_type_t *)data;
+    GtkWidget  *main_w = gtk_widget_get_toplevel(cancel_bt);
+    static GList *filter_list;
+
+
+    window_destroy(GTK_WIDGET(main_w));
+
+    /* if this was the last open filter dialog, revert the changes made */
+    filter_list = get_filter_dialog_list(list_type);
+    if(g_list_length(filter_list) == 0) {
+        /* revert changes in the edited list */
+        switch (list_type) {
+        case CFILTER_EDITED_LIST:
+                copy_filter_list(CFILTER_EDITED_LIST, CFILTER_LIST);
+	        break;
+        case DFILTER_EDITED_LIST:
+                copy_filter_list(DFILTER_EDITED_LIST, DFILTER_LIST);
+	        break;
+        default:
+	        g_assert_not_reached();
+	        break;
+        }
+    }
+
+#if 0
+    /* update other open filter dialogs */
+    g_list_foreach(get_filter_dialog_list(list_type), filter_dlg_update_list_cb, &list_type);
+#endif
+}
+
+/* Treat this as a cancel, by calling "filter_dlg_cancel_cb()" */
+static gboolean
+filter_dlg_delete_event_cb(GtkWidget *main_w, GdkEvent *event _U_,
+                           gpointer data)
+{
+  filter_dlg_cancel_cb(main_w, data);
+  return FALSE;
+}
+
+
 static void
 filter_dlg_destroy_cb(GtkWidget *win, gpointer data)
 {
-	filter_list_type_t list = *(filter_list_type_t *)data;
+	filter_list_type_t list_type = *(filter_list_type_t *)data;
 	GtkWidget *button;
 
 	/* Get the button that requested that we be popped up, if any.
@@ -876,10 +1001,10 @@ filter_dlg_destroy_cb(GtkWidget *win, gpointer data)
 	} else {
 		/* This is an editing dialog popped up from, for example,
 		   a menu item; note that we no longer have one. */
-		switch (list) {
+		switch (list_type) {
 
 #ifdef HAVE_LIBPCAP
-		case CFILTER_LIST:
+		case CFILTER_EDITED_LIST:
 			g_assert(win == global_cfilter_w);
 			global_cfilter_w = NULL;
 			break;
@@ -891,7 +1016,7 @@ filter_dlg_destroy_cb(GtkWidget *win, gpointer data)
 	}
 
 	/* Remove this from the list of filter dialog windows. */
-	forget_filter_dialog(win, list);
+	forget_filter_dialog(win, list_type);
 }
 
 #if GTK_MAJOR_VERSION < 2
@@ -1074,7 +1199,7 @@ filter_new_bt_clicked_cb(GtkWidget *w, gpointer data)
   GtkWidget  *name_te = OBJECT_GET_DATA(main_w, E_FILT_NAME_TE_KEY);
   GtkWidget  *filter_te = OBJECT_GET_DATA(main_w, E_FILT_FILTER_TE_KEY);
   GtkWidget  *filter_l = OBJECT_GET_DATA(main_w, E_FILT_FILTER_L_KEY);
-  filter_list_type_t list = *(filter_list_type_t *)data;
+  filter_list_type_t list_type = *(filter_list_type_t *)data;
   GList      *fl_entry;
   const gchar *name, *strval;
   new_filter_cb_args_t args;
@@ -1093,13 +1218,13 @@ filter_new_bt_clicked_cb(GtkWidget *w, gpointer data)
   }
 
     /* Add a new entry to the filter list. */
-    fl_entry = add_to_filter_list(list, name, strval);
+    fl_entry = add_to_filter_list(list_type, name, strval);
 
     /* Update all the filter list widgets, not just the one in
        the dialog box in which we clicked on "Copy". */
     args.active_filter_l = filter_l;
     args.nflp = fl_entry;
-    g_list_foreach(get_filter_dialog_list(list), new_filter_cb, &args);
+    g_list_foreach(get_filter_dialog_list(list_type), new_filter_cb, &args);
 
 }
 
@@ -1165,7 +1290,7 @@ filter_name_te_changed_cb(GtkWidget *w, gpointer data)
     GtkWidget  *filter_l = OBJECT_GET_DATA(main_w, E_FILT_FILTER_L_KEY);
     filter_def *filt;
     GList      *fl_entry;
-    filter_list_type_t  list = *(filter_list_type_t *)data;
+    filter_list_type_t  list_type = *(filter_list_type_t *)data;
     const gchar         *name = "";
     const gchar         *strval = "";
 
@@ -1187,7 +1312,7 @@ filter_name_te_changed_cb(GtkWidget *w, gpointer data)
     name   = gtk_entry_get_text(GTK_ENTRY(name_te));
     strval = gtk_entry_get_text(GTK_ENTRY(filter_te));
 
-    if (DFILTER_LIST == list) {
+    if (DFILTER_EDITED_LIST == list_type) {
         /* colorize filter string entry */
         filter_te_syntax_check_cb(filter_te);
     }
@@ -1215,7 +1340,7 @@ filter_name_te_changed_cb(GtkWidget *w, gpointer data)
 
                 /* Update all the filter list widgets, not just the one in
                    the dialog box in which we clicked on "Copy". */
-                g_list_foreach(get_filter_dialog_list(list), chg_filter_cb,
+                g_list_foreach(get_filter_dialog_list(list_type), chg_filter_cb,
                                fl_entry);
             }
         }
@@ -1248,7 +1373,7 @@ filter_del_bt_clicked_cb(GtkWidget *w, gpointer data)
 {
     GtkWidget  *main_w = gtk_widget_get_toplevel(w);
     GtkWidget  *filter_l = OBJECT_GET_DATA(main_w, E_FILT_FILTER_L_KEY);
-    filter_list_type_t list = *(filter_list_type_t *)data;
+    filter_list_type_t list_type = *(filter_list_type_t *)data;
     GList      *fl_entry;
 #if GTK_MAJOR_VERSION < 2
     GList      *sl;
@@ -1280,15 +1405,15 @@ filter_del_bt_clicked_cb(GtkWidget *w, gpointer data)
 #endif
         if (fl_entry != NULL) {
             /* Remove the entry from the filter list. */
-            remove_from_filter_list(list, fl_entry);
+            remove_from_filter_list(list_type, fl_entry);
 
             /* Update all the filter list widgets, not just the one in
                the dialog box in which we clicked on "Delete". */
 #if GTK_MAJOR_VERSION < 2
-            g_list_foreach(get_filter_dialog_list(list), delete_filter_cb,
+            g_list_foreach(get_filter_dialog_list(list_type), delete_filter_cb,
                            &pos);
 #else
-            g_list_foreach(get_filter_dialog_list(list), delete_filter_cb, pos);
+            g_list_foreach(get_filter_dialog_list(list_type), delete_filter_cb, pos);
 #endif
         }
 #if GTK_MAJOR_VERSION >= 2
