@@ -1,7 +1,7 @@
 /* gui_utils.h
  * Declarations of GTK+-specific UI utility routines
  *
- * $Id: gui_utils.h 18928 2006-08-16 18:01:25Z gerald $
+ * $Id: gui_utils.h 19051 2006-08-27 10:41:30Z ulfl $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -291,14 +291,6 @@ extern GtkWidget *xpm_to_widget(const char ** xpm);
  */
 extern GtkWidget *xpm_to_widget_from_parent(GtkWidget *parent, const char ** xpm);
 
-/** Create a new hbox with an image packed into it
- * and return the box.
- *
- * @param xpm the character array containing the picture
- * @return a newly created GtkHBox containing the picture
- */
-GtkWidget *xpm_box( gchar **xpm );
-
 /** Copy a GString to the clipboard.
  *
  * @param str GString that is to be copied to the clipboard.
