@@ -1,6 +1,6 @@
 /* help_dlg.h
  *
- * $Id: help_dlg.h 18268 2006-05-31 17:38:42Z gerald $
+ * $Id: help_dlg.h 19089 2006-08-30 20:27:12Z ulfl $
  *
  * Laurent Deniel <laurent.deniel@free.fr>
  *
@@ -72,7 +72,18 @@ typedef enum {
     HELP_STATS_CONVERSATIONS_DIALOG,
     HELP_STATS_IO_GRAPH_DIALOG,
     HELP_CAPTURE_INTERFACES_DETAILS_DIALOG,
-    HELP_PREFERENCES_DIALOG
+    HELP_PREFERENCES_DIALOG,
+    HELP_CAPTURE_INFO_DIALOG,
+    HELP_EXPORT_FILE_DIALOG,
+    HELP_EXPORT_BYTES_DIALOG,
+    HELP_OPEN_DIALOG,
+    HELP_MERGE_DIALOG,
+    HELP_SAVE_DIALOG,
+    HELP_EXPORT_FILE_WIN32_DIALOG,
+    HELP_EXPORT_BYTES_WIN32_DIALOG,
+    HELP_OPEN_WIN32_DIALOG,
+    HELP_MERGE_WIN32_DIALOG,
+    HELP_SAVE_WIN32_DIALOG
 } topic_action_e;
 
 
