@@ -1,7 +1,7 @@
 /* main.h
  * Global defines, etc.
  *
- * $Id: main.h 18438 2006-06-12 19:35:26Z ulfl $
+ * $Id: main.h 18928 2006-08-16 18:01:25Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -62,6 +62,8 @@ extern GString *comp_info_str;
 /** Global runtime version string */
 extern GString *runtime_info_str;
 
+extern GtkWidget* airpcap_tb;
+
 /** Global capture options type. */
 typedef struct capture_options_tag * p_capture_options_t;
 /** Pointer to global capture options. */
@@ -70,6 +72,9 @@ extern p_capture_options_t capture_opts;
 extern void protect_thread_critical_region(void);
 extern void unprotect_thread_critical_region(void);
 
+void
+airpcap_toolbar_encryption_cb(GtkWidget *entry, gpointer user_data);
+
 /** User requested "Zoom In" by menu or toolbar.
  *
  * @param widget parent widget (unused)
