@@ -1,7 +1,7 @@
 /* menu.c
  * Menu routines
  *
- * $Id: menu.c 18977 2006-08-21 14:42:28Z gerald $
+ * $Id: menu.c 19578 2006-10-17 17:52:15Z lego $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -368,6 +368,10 @@ conversation_cb(GtkWidget * w, gpointer data _U_, int action)
 
 }
 
+#ifdef HAVE_LUA_5_1
+static gboolean have_items_in_tools_menu = FALSE;
+#endif
+
 
 /* main menu */
 static GtkItemFactoryEntry menu_items[] =
@@ -391,6 +395,10 @@ static GtkItemFactoryEntry menu_items[] =
     ITEM_FACTORY_STOCK_ENTRY("/File/File Set/Previous File", NULL, fileset_previous_cb, 0, WIRESHARK_STOCK_FILE_SET_PREVIOUS),
     ITEM_FACTORY_ENTRY("/File/<separator>", NULL, NULL, 0, "<Separator>", NULL),
     ITEM_FACTORY_ENTRY("/File/_Export", NULL, NULL, 0, "<Branch>", NULL),
+#if GTK_MAJOR_VERSION >= 2 && _WIN32
+    ITEM_FACTORY_ENTRY("/File/Export/File...", NULL, export_text_cmd_cb,
+                         0, NULL, NULL),
+#else
     ITEM_FACTORY_ENTRY("/File/Export/as \"Plain _Text\" file...", NULL, export_text_cmd_cb,
                              0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/File/Export/as \"_PostScript\" file...", NULL, export_ps_cmd_cb,
@@ -403,6 +411,7 @@ static GtkItemFactoryEntry menu_items[] =
     ITEM_FACTORY_ENTRY("/File/Export/as XML - \"P_DML\" (packet details) file...", NULL, export_pdml_cmd_cb,
                              0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/File/Export/<separator>", NULL, NULL, 0, "<Separator>", NULL),
+#endif
     ITEM_FACTORY_ENTRY("/File/Export/Selected Packet _Bytes...", "<control>H", savehex_cb,
                              0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/File/<separator>", NULL, NULL, 0, "<Separator>", NULL),
@@ -593,11 +602,14 @@ static GtkItemFactoryEntry menu_items[] =
                        init_conversation_notebook_cb, 0, WIRESHARK_STOCK_CONVERSATIONS),
     ITEM_FACTORY_STOCK_ENTRY("/Statistics/Endpoints", NULL,
                        init_hostlist_notebook_cb, 0, WIRESHARK_STOCK_ENDPOINTS),
+#ifdef HAVE_LUA_5_1
+    ITEM_FACTORY_ENTRY("/_Tools", NULL, NULL, 0, "<Branch>", NULL),
+#endif
     ITEM_FACTORY_ENTRY("/_Help", NULL, NULL, 0, "<Branch>", NULL),
     ITEM_FACTORY_STOCK_ENTRY("/Help/_Contents", "F1", topic_menu_cb, HELP_CONTENT, GTK_STOCK_HELP),
     ITEM_FACTORY_ENTRY("/Help/_Supported Protocols", NULL, supported_cb, 0, NULL, NULL),
 #if (GLIB_MAJOR_VERSION >= 2)
-#ifdef WIRESHARK_EUG_DIR
+#ifdef ENABLE_WSUG
     ITEM_FACTORY_ENTRY("/Help/User's Guide", NULL, topic_menu_cb, HELP_CONTENT, NULL, NULL),
 #endif
     ITEM_FACTORY_ENTRY("/Help/Manual Pages", NULL, NULL, 0, "<Branch>", NULL),
@@ -696,15 +708,15 @@ static GtkItemFactoryEntry packet_list_menu_items[] =
 static GtkItemFactoryEntry tree_view_menu_items[] =
 {
 
-    ITEM_FACTORY_ENTRY("/Copy", NULL, copy_selected_plist_cb, 0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/<separator>", NULL, NULL, 0, "<Separator>", NULL),
-
     ITEM_FACTORY_ENTRY("/Expand Subtrees", NULL, expand_tree_cb, 0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Expand All", NULL, expand_all_cb, 0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Collapse All", NULL, collapse_all_cb, 0, NULL, NULL),
 
     ITEM_FACTORY_ENTRY("/<separator>", NULL, NULL, 0, "<Separator>", NULL),
 
+    ITEM_FACTORY_ENTRY("/Copy", NULL, copy_selected_plist_cb, 0, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/<separator>", NULL, NULL, 0, "<Separator>", NULL),
+
     ITEM_FACTORY_ENTRY("/Apply as Filter", NULL, NULL, 0, "<Branch>", NULL),
     ITEM_FACTORY_ENTRY("/Apply as Filter/_Selected", NULL, match_selected_ptree_cb,
                        MATCH_SELECTED_REPLACE|MATCH_SELECTED_APPLY_NOW, NULL, NULL),
@@ -755,14 +767,15 @@ static GtkItemFactoryEntry tree_view_menu_items[] =
 static GtkItemFactoryEntry hexdump_menu_items[] =
 {
     ITEM_FACTORY_ENTRY("/Copy", NULL, NULL, 0, "<Branch>", NULL),
-    ITEM_FACTORY_ENTRY("/Copy/...All Information", NULL, copy_hex_cb,
+    ITEM_FACTORY_ENTRY("/Copy/All Information", NULL, copy_hex_cb,
                        0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Copy/...Text Only", NULL, copy_hex_cb,
+    ITEM_FACTORY_ENTRY("/Copy/Text Only", NULL, copy_hex_cb,
                        1, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Export Selected Packet Bytes...", NULL, savehex_cb,
                        0, NULL, NULL),
 };
 
+
 static int initialize = TRUE;
 static GtkItemFactory *main_menu_factory = NULL;
 static GtkItemFactory *packet_list_menu_factory = NULL;
@@ -818,7 +831,13 @@ menus_init(void) {
     /* main */
     main_menu_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", grp);
     gtk_item_factory_create_items_ac(main_menu_factory, nmenu_items, menu_items, NULL, 2);
-
+	
+#ifdef HAVE_LUA_5_1
+		if (! have_items_in_tools_menu) {
+			gtk_widget_hide(gtk_item_factory_get_item(main_menu_factory,"/Tools"));
+		}
+#endif
+		
     merge_all_tap_menus(tap_menu_tree_root);
 
     /* Initialize enabled/disabled state of menu items */
@@ -909,7 +928,7 @@ static GList * tap_menu_item_add(
 void
 register_stat_menu_item(
     const char *name,
-    REGISTER_STAT_GROUP_E group,
+    register_stat_group_t group,
     GtkItemFactoryCallback callback,
     gboolean (*selected_packet_enabled)(frame_data *, epan_dissect_t *),
     gboolean (*selected_tree_row_enabled)(field_info *),
@@ -937,8 +956,14 @@ register_stat_menu_item(
     case(REGISTER_STAT_GROUP_TELEPHONY): toolspath = "/Statistics/"; break;
     case(REGISTER_STAT_GROUP_NONE): toolspath = "/Statistics/"; break;
     case(REGISTER_ANALYZE_GROUP_NONE): toolspath = "/Analyze/"; break;
+#ifdef HAVE_LUA_5_1
+    case(REGISTER_TOOLS_GROUP_NONE):
+		toolspath = "/Tools/";
+		have_items_in_tools_menu = TRUE;
+		break;
+#endif
     default:
-        g_assert(0);
+        g_assert(!"no such menu group");
         toolspath = NULL;
     }
 
@@ -1062,6 +1087,10 @@ static guint merge_tap_menus_layered(GList *node, gint group) {
                 break;
             case(REGISTER_ANALYZE_GROUP_NONE):
                 break;
+#ifdef HAVE_LUA_5_1
+            case(REGISTER_TOOLS_GROUP_NONE):
+                break;
+#endif
             default:
                 g_assert_not_reached();
             }
@@ -1134,6 +1163,12 @@ void merge_all_tap_menus(GList *node) {
 		entry->path = "/Analyze/";
         /*gtk_item_factory_create_item(main_menu_factory, entry, NULL, 2);*/
     }
+#ifdef HAVE_LUA_5_1
+    if (merge_tap_menus_layered(node, REGISTER_TOOLS_GROUP_NONE)) {
+        /*gtk_item_factory_create_item(main_menu_factory, entry, NULL, 2);*/
+    }
+#endif
+
 }
 
 
