@@ -1,7 +1,7 @@
 /* prefs_dlg.c
  * Routines for handling preferences
  *
- * $Id: prefs_dlg.c 18903 2006-08-14 22:34:46Z gerald $
+ * $Id: prefs_dlg.c 19674 2006-10-24 21:50:59Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -50,6 +50,7 @@
 #include "simple_dialog.h"
 #include "compat_macros.h"
 #include "help_dlg.h"
+#include "keys.h"
 
 #include <epan/prefs-int.h>
 
@@ -59,6 +60,12 @@
 #endif /* _WIN32 */
 #endif /* HAVE_LIBPCAP */
 
+#ifdef HAVE_AIRPCAP
+#include "airpcap.h"
+#include "airpcap_loader.h"
+#include "airpcap_gui_utils.h"
+#endif
+
 static void     prefs_main_ok_cb(GtkWidget *, gpointer);
 static void     prefs_main_apply_cb(GtkWidget *, gpointer);
 static void     prefs_main_save_cb(GtkWidget *, gpointer);
@@ -75,8 +82,10 @@ static void	prefs_tree_select_cb(GtkTreeSelection *, gpointer);
 #define E_PREFSW_SCROLLW_KEY    "prefsw_scrollw"
 #define E_PREFSW_TREE_KEY       "prefsw_tree"
 #define E_PREFSW_NOTEBOOK_KEY   "prefsw_notebook"
+#define E_PREFSW_SAVE_BT_KEY    "prefsw_save_bt"
 #define E_PAGE_ITER_KEY         "page_iter"
 #define E_PAGE_MODULE_KEY       "page_module"
+#define E_PAGESW_FRAME_KEY      "pagesw_frame"
 
 #define E_GUI_PAGE_KEY	        "gui_options_page"
 #define E_GUI_LAYOUT_PAGE_KEY	"gui_layout_page"
@@ -315,9 +324,10 @@ module_prefs_show(module_t *module, gpointer user_data)
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(main_sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 
     /* Frame */
-    frame = gtk_frame_new(module->title);
+    frame = gtk_frame_new(module->description);
     gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(main_sw), frame);
+    OBJECT_SET_DATA(main_sw, E_PAGESW_FRAME_KEY, frame);
 
     /* Main vertical box */
     main_vb = gtk_vbox_new(FALSE, 5);
@@ -617,7 +627,7 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   cts.is_protocol = FALSE;
   prefs_module_list_foreach(NULL, module_prefs_show, &cts);
 
-  /* Button row: OK and cancel buttons */
+  /* Button row: OK and alike buttons */
 
   if(topic_available(HELP_PREFERENCES_DIALOG)) {
     bbox = dlg_button_row_new(GTK_STOCK_HELP, GTK_STOCK_OK, GTK_STOCK_APPLY, GTK_STOCK_SAVE, GTK_STOCK_CANCEL, NULL);
@@ -635,6 +645,7 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
 
   save_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_SAVE);
   SIGNAL_CONNECT(save_bt, "clicked", prefs_main_save_cb, prefs_w);
+  OBJECT_SET_DATA(prefs_w, E_PREFSW_SAVE_BT_KEY, save_bt);
 
   cancel_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
   SIGNAL_CONNECT(cancel_bt, "clicked", prefs_main_cancel_cb, prefs_w);
@@ -651,6 +662,12 @@ prefs_cb(GtkWidget *w _U_, gpointer dummy _U_)
   SIGNAL_CONNECT(prefs_w, "destroy", prefs_main_destroy_cb, prefs_w);
 
   gtk_widget_show(prefs_w);
+
+  /* hide the Save button if the user uses implicit save */
+  if(!prefs.gui_use_pref_save) {
+    gtk_widget_hide(save_bt);
+  }
+
   window_present(prefs_w);
 
 #if GTK_MAJOR_VERSION >= 2
@@ -1060,6 +1077,67 @@ module_prefs_fetch(module_t *module, gpointer user_data)
   return 0;	/* keep fetching module preferences */
 }
 
+#ifdef HAVE_AIRPCAP
+/*
+ * This function is used to apply changes and update the Wireless Toolbar
+ * whenever we apply some changes to the WEP preferences
+ */
+static void
+prefs_airpcap_update()
+{
+GtkWidget *decryption_cm;
+GtkWidget *decryption_en;
+gboolean wireshark_decryption_was_enabled;
+gboolean airpcap_decryption_was_enabled;
+gboolean wireshark_decryption_is_now_enabled;
+
+decryption_cm = GTK_WIDGET(OBJECT_GET_DATA(airpcap_tb,AIRPCAP_TOOLBAR_DECRYPTION_KEY));
+decryption_en = GTK_WIDGET(GTK_ENTRY(GTK_COMBO(decryption_cm)->entry));
+
+if( g_strcasecmp(gtk_entry_get_text(GTK_ENTRY(decryption_en)),AIRPCAP_DECRYPTION_TYPE_STRING_WIRESHARK) == 0 )
+{
+wireshark_decryption_was_enabled = TRUE;
+airpcap_decryption_was_enabled = FALSE;
+}
+else if( g_strcasecmp(gtk_entry_get_text(GTK_ENTRY(decryption_en)),AIRPCAP_DECRYPTION_TYPE_STRING_AIRPCAP) == 0 )
+{
+wireshark_decryption_was_enabled = FALSE;
+airpcap_decryption_was_enabled = TRUE;
+}
+else if( g_strcasecmp(gtk_entry_get_text(GTK_ENTRY(decryption_en)),AIRPCAP_DECRYPTION_TYPE_STRING_NONE) == 0 )
+{
+wireshark_decryption_was_enabled = FALSE;
+airpcap_decryption_was_enabled = FALSE;
+}
+
+wireshark_decryption_is_now_enabled = wireshark_decryption_on();
+
+if(wireshark_decryption_is_now_enabled && airpcap_decryption_was_enabled)
+	{
+	set_airpcap_decryption(FALSE);
+	gtk_entry_set_text(GTK_ENTRY(decryption_en),AIRPCAP_DECRYPTION_TYPE_STRING_WIRESHARK);
+	}
+if(wireshark_decryption_is_now_enabled && !airpcap_decryption_was_enabled)
+	{
+	set_airpcap_decryption(FALSE);
+	gtk_entry_set_text(GTK_ENTRY(decryption_en),AIRPCAP_DECRYPTION_TYPE_STRING_WIRESHARK);
+	}
+else if(!wireshark_decryption_is_now_enabled && wireshark_decryption_was_enabled)
+	{
+	if(airpcap_decryption_was_enabled)
+		{
+		set_airpcap_decryption(TRUE);
+		gtk_entry_set_text(GTK_ENTRY(decryption_en),AIRPCAP_DECRYPTION_TYPE_STRING_AIRPCAP);
+		}
+	else
+		{
+		set_airpcap_decryption(FALSE);
+		gtk_entry_set_text(GTK_ENTRY(decryption_en),AIRPCAP_DECRYPTION_TYPE_STRING_NONE);
+		}
+	}
+}
+#endif
+
 static guint
 pref_clean(pref_t *pref, gpointer user_data _U_)
 {
@@ -1165,6 +1243,8 @@ prefs_main_fetch_all(GtkWidget *dlg, gboolean *must_redissect)
 static void
 prefs_main_apply_all(GtkWidget *dlg)
 {
+  GtkWidget *save_bt;
+
   /*
    * Apply the protocol preferences first - "gui_prefs_apply()" could
    * cause redissection, and we have to make sure the protocol
@@ -1189,6 +1269,14 @@ prefs_main_apply_all(GtkWidget *dlg)
 #endif /* HAVE_LIBPCAP */
   printer_prefs_apply(OBJECT_GET_DATA(dlg, E_PRINT_PAGE_KEY));
   nameres_prefs_apply(OBJECT_GET_DATA(dlg, E_NAMERES_PAGE_KEY));
+
+  /* show/hide the Save button - depending on setting */
+  save_bt = OBJECT_GET_DATA(prefs_w, E_PREFSW_SAVE_BT_KEY);
+  if(prefs.gui_use_pref_save) {
+    gtk_widget_show(save_bt);
+  } else {
+    gtk_widget_hide(save_bt);
+  }
 }
 
 
@@ -1234,51 +1322,12 @@ prefs_main_destroy_all(GtkWidget *dlg)
 
 
 static void
-prefs_main_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w)
+prefs_main_write(void)
 {
-  gboolean must_redissect = FALSE;
-
-  if (!prefs_main_fetch_all(parent_w, &must_redissect))
-    return; /* Errors in some preference setting */
-
-  prefs_main_apply_all(parent_w);
-
-  /* Now destroy the "Preferences" dialog. */
-  window_destroy(GTK_WIDGET(parent_w));
-
-  if (must_redissect) {
-    /* Redissect all the packets, and re-evaluate the display filter. */
-    cf_redissect_packets(&cfile);
-  }
-}
-
-static void
-prefs_main_apply_cb(GtkWidget *apply_bt _U_, gpointer parent_w)
-{
-  gboolean must_redissect = FALSE;
-
-  if (!prefs_main_fetch_all(parent_w, &must_redissect))
-    return; /* Errors in some preference setting */
-
-  prefs_main_apply_all(parent_w);
-
-  if (must_redissect) {
-    /* Redissect all the packets, and re-evaluate the display filter. */
-    cf_redissect_packets(&cfile);
-  }
-}
-
-static void
-prefs_main_save_cb(GtkWidget *save_bt _U_, gpointer parent_w)
-{
-  gboolean must_redissect = FALSE;
   int err;
   char *pf_dir_path;
   char *pf_path;
 
-  if (!prefs_main_fetch_all(parent_w, &must_redissect))
-    return; /* Errors in some preference setting */
-
   /* Create the directory that holds personal configuration files, if
      necessary.  */
   if (create_persconffile_dir(&pf_dir_path) == -1) {
@@ -1297,6 +1346,80 @@ prefs_main_save_cb(GtkWidget *save_bt _U_, gpointer parent_w)
     }
   }
 
+#ifdef HAVE_AIRPCAP
+/* 
+ * Load the Wireshark decryption keys (just set) and save 
+ * the changes to the adapters' registry 
+ */ 
+airpcap_load_decryption_keys(airpcap_if_list);
+#endif
+}
+
+
+static void
+prefs_main_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w)
+{
+  gboolean must_redissect = FALSE;
+
+  if (!prefs_main_fetch_all(parent_w, &must_redissect))
+    return; /* Errors in some preference setting - already reported */
+
+  /* if we don't have a Save button, just save the settings now */
+  if (!prefs.gui_use_pref_save) {
+      prefs_main_write();
+  }
+
+  prefs_main_apply_all(parent_w);
+
+	#ifdef HAVE_AIRPCAP
+	prefs_airpcap_update();
+	#endif
+
+  /* Now destroy the "Preferences" dialog. */
+  window_destroy(GTK_WIDGET(parent_w));
+
+  if (must_redissect) {
+    /* Redissect all the packets, and re-evaluate the display filter. */
+    cf_redissect_packets(&cfile);
+  }
+
+}
+
+static void
+prefs_main_apply_cb(GtkWidget *apply_bt _U_, gpointer parent_w)
+{
+  gboolean must_redissect = FALSE;
+
+  if (!prefs_main_fetch_all(parent_w, &must_redissect))
+    return; /* Errors in some preference setting - already reported */
+
+  /* if we don't have a Save button, just save the settings now */
+  if (!prefs.gui_use_pref_save) {
+      prefs_main_write();
+  }
+
+  prefs_main_apply_all(parent_w);
+
+  	#ifdef HAVE_AIRPCAP
+	prefs_airpcap_update();
+	#endif
+
+  if (must_redissect) {
+    /* Redissect all the packets, and re-evaluate the display filter. */
+    cf_redissect_packets(&cfile);
+  }
+}
+
+static void
+prefs_main_save_cb(GtkWidget *save_bt _U_, gpointer parent_w)
+{
+  gboolean must_redissect = FALSE;
+
+  if (!prefs_main_fetch_all(parent_w, &must_redissect))
+    return; /* Errors in some preference setting - already reported */
+
+  prefs_main_write();
+
   /* Now apply those preferences.
      XXX - should we do this?  The user didn't click "OK" or "Apply".
      However:
@@ -1521,7 +1644,7 @@ tree_select_node(GtkWidget *tree, prefs_tree_iter *iter)
 }
 
 
-
+/* search the corresponding protocol page of the currently selected field */
 void
 properties_cb(GtkWidget *w, gpointer dummy)
 {
@@ -1529,6 +1652,7 @@ properties_cb(GtkWidget *w, gpointer dummy)
   const gchar *title;
   struct properties_data p;
   int page_num;
+  GtkWidget *sw;
   GtkWidget *frame;
   module_t *page_module;
 
@@ -1567,8 +1691,10 @@ properties_cb(GtkWidget *w, gpointer dummy)
   /* Search all the pages in that window for the one with the specified
      module. */
   for (page_num = 0;
-       (frame = gtk_notebook_get_nth_page(OBJECT_GET_DATA(prefs_w, E_PREFSW_NOTEBOOK_KEY), page_num)) != NULL;
+       (sw = gtk_notebook_get_nth_page(OBJECT_GET_DATA(prefs_w, E_PREFSW_NOTEBOOK_KEY), page_num)) != NULL;
        page_num++) {
+    /* Get the frame from the scrollable window */
+    frame = OBJECT_GET_DATA(sw, E_PAGESW_FRAME_KEY);
     /* Get the module for this page. */
     page_module = OBJECT_GET_DATA(frame, E_PAGE_MODULE_KEY);
     if (page_module == NULL)
