@@ -1,7 +1,7 @@
 /* proto_draw.c
  * Routines for GTK+ packet display
  *
- * $Id: proto_draw.c 18543 2006-06-22 11:23:26Z sahlberg $
+ * $Id: proto_draw.c 19384 2006-09-30 14:57:50Z ulfl $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -73,6 +73,7 @@
 
 #include "../ui_util.h"
 #include "file_util.h"
+#include "webbrowser.h"
 
 #if GTK_MAJOR_VERSION >= 2 && _WIN32
 #include <gdk/gdkwin32.h>
@@ -1721,7 +1722,8 @@ static void tree_cell_renderer(GtkTreeViewColumn *tree_column _U_,
         g_object_set (cell, "weight-set", TRUE, NULL);*/
 	}
 
-    if(fi->hfinfo->type == FT_FRAMENUM) {
+    if((fi->hfinfo->type == FT_FRAMENUM) ||
+       (FI_GET_FLAG(fi, FI_URL) && IS_FT_STRING(fi->hfinfo->type))) {
         g_object_set (cell, "foreground", "blue", NULL);
         g_object_set (cell, "foreground-set", TRUE, NULL);
 
@@ -1939,9 +1941,16 @@ main_proto_tree_draw(proto_tree *protocol_tree)
 static void
 tree_view_follow_link(field_info   *fi)
 {
+    gchar *url;
+
     if(fi->hfinfo->type == FT_FRAMENUM) {
         cf_goto_frame(&cfile, fi->value.value.integer);
     }
+    if(FI_GET_FLAG(fi, FI_URL) && IS_FT_STRING(fi->hfinfo->type)) {
+      url = g_strndup(tvb_get_ptr(fi->ds_tvb, fi->start, fi->length), fi->length);
+      browser_open_url(url);
+      g_free(url);
+    }
 }
 
 
