@@ -2,7 +2,7 @@
  * Recent "preference" handling routines
  * Copyright 2004, Ulf Lamping <ulf.lamping@web.de>
  *
- * $Id: recent.c 18205 2006-05-22 07:29:40Z guy $
+ * $Id: recent.c 19749 2006-10-31 01:51:39Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -50,6 +50,8 @@
 
 #define RECENT_KEY_MAIN_TOOLBAR_SHOW        "gui.toolbar_main_show"
 #define RECENT_KEY_FILTER_TOOLBAR_SHOW      "gui.filter_toolbar_show"
+#define RECENT_KEY_AIRPCAP_TOOLBAR_SHOW      "gui.airpcap_toolbar_show"
+#define RECENT_KEY_DRIVER_CHECK_SHOW		"gui.airpcap_driver_check_show"
 #define RECENT_KEY_PACKET_LIST_SHOW         "gui.packet_list_show"
 #define RECENT_KEY_TREE_VIEW_SHOW           "gui.tree_view_show"
 #define RECENT_KEY_BYTE_VIEW_SHOW           "gui.byte_view_show"
@@ -165,6 +167,20 @@ write_recent(void)
   fprintf(rf, RECENT_KEY_FILTER_TOOLBAR_SHOW ": %s\n",
 		  recent.filter_toolbar_show == TRUE ? "TRUE" : "FALSE");
 
+#ifdef HAVE_AIRPCAP
+  fprintf(rf, "\n# Wireless Settings Toolbar show (hide).\n");
+  fprintf(rf, "# TRUE or FALSE (case-insensitive).\n");
+  fprintf(rf, RECENT_KEY_AIRPCAP_TOOLBAR_SHOW ": %s\n",
+		  recent.airpcap_toolbar_show == TRUE ? "TRUE" : "FALSE");
+#endif
+
+#ifdef HAVE_AIRPCAP
+  fprintf(rf, "\n# Show (hide) old AirPcap driver warning dialog box.\n");
+  fprintf(rf, "# TRUE or FALSE (case-insensitive).\n");
+  fprintf(rf, RECENT_KEY_DRIVER_CHECK_SHOW ": %s\n",
+		  recent.airpcap_driver_check_show == TRUE ? "TRUE" : "FALSE");
+#endif
+
   fprintf(rf, "\n# Packet list show (hide).\n");
   fprintf(rf, "# TRUE or FALSE (case-insensitive).\n");
   fprintf(rf, RECENT_KEY_PACKET_LIST_SHOW ": %s\n",
@@ -213,7 +229,7 @@ write_recent(void)
   		  recent.gui_geometry_main_width);
   fprintf(rf, RECENT_GUI_GEOMETRY_MAIN_HEIGHT ": %d\n",
   		  recent.gui_geometry_main_height);
-  
+
   fprintf(rf, "\n# Main window maximized (GTK2 only!).\n");
   fprintf(rf, "# TRUE or FALSE (case-insensitive).\n");
   fprintf(rf, RECENT_GUI_GEOMETRY_MAIN_MAXIMIZED ": %s\n",
@@ -259,7 +275,7 @@ write_recent(void)
 
 
 /* write the geometry values of a window to recent file */
-void 
+void
 write_recent_geom(gpointer key _U_, gpointer value, gpointer rf)
 {
     window_geometry_t *geom = value;
@@ -301,7 +317,21 @@ read_set_recent_pair_static(gchar *key, gchar *value)
     else {
         recent.filter_toolbar_show = FALSE;
     }
-  } else if (strcmp(key, RECENT_KEY_PACKET_LIST_SHOW) == 0) {
+  } else if (strcmp(key, RECENT_KEY_AIRPCAP_TOOLBAR_SHOW) == 0) {
+    if (strcasecmp(value, "true") == 0) {
+        recent.airpcap_toolbar_show = TRUE;
+    }
+    else {
+        recent.airpcap_toolbar_show = FALSE;
+    }
+  } else if (strcmp(key, RECENT_KEY_DRIVER_CHECK_SHOW) == 0) {
+    if (strcasecmp(value, "true") == 0) {
+        recent.airpcap_driver_check_show = TRUE;
+    }
+    else {
+        recent.airpcap_driver_check_show = FALSE;
+    }
+  }else if (strcmp(key, RECENT_KEY_PACKET_LIST_SHOW) == 0) {
     if (strcasecmp(value, "true") == 0) {
         recent.packet_list_show = TRUE;
     }
@@ -489,6 +519,8 @@ recent_read_static(char **rf_path_return, int *rf_errno_return)
   /* set defaults */
   recent.main_toolbar_show      = TRUE;
   recent.filter_toolbar_show    = TRUE;
+  recent.airpcap_toolbar_show   = FALSE;
+  recent.airpcap_driver_check_show   = TRUE;
   recent.packet_list_show       = TRUE;
   recent.tree_view_show         = TRUE;
   recent.byte_view_show         = TRUE;
