@@ -1,7 +1,7 @@
 /* voip_calls.c
  * VoIP calls summary addition for Wireshark
  *
- * $Id: voip_calls.c 18791 2006-07-25 09:21:41Z jake $
+ * $Id: voip_calls.c 19676 2006-10-24 22:21:04Z gerald $
  *
  * Copyright 2004, Ericsson, Spain
  * By Francisco Alcoba <francisco.alcoba@ericsson.com>
@@ -65,6 +65,12 @@
 #include "alert_box.h"
 #include "simple_dialog.h"
 
+#ifdef HAVE_LIBPORTAUDIO
+#if GTK_MAJOR_VERSION >= 2
+#include "rtp_player.h"
+#endif /* GTK_MAJOR_VERSION >= 2 */
+#endif /* HAVE_LIBPORTAUDIO */
+
 const char *voip_call_state_name[7]={
 	"CALL SETUP",
 	"RINGING",
@@ -79,11 +85,11 @@ const char *voip_call_state_name[7]={
 const char *voip_protocol_name[7]={
 	"SIP",
 	"ISUP",
-	"H323",
+	"H.323",
 	"MGCP",
 	"AC_ISDN",
 	"AC_CAS",
-	"T38"
+	"T.38"
 	};
 
 typedef struct {
@@ -124,6 +130,13 @@ void voip_calls_reset(voip_calls_tapinfo_t *tapinfo)
 	GList* list;
 	GList* list2;
 
+#ifdef HAVE_LIBPORTAUDIO
+#if GTK_MAJOR_VERSION >= 2
+	/* reset the RTP player */
+	reset_rtp_player();
+#endif
+#endif
+
 	/* free the data items first */
 	list = g_list_first(tapinfo->strinfo_list);
 	while (list)
@@ -161,8 +174,8 @@ void voip_calls_reset(voip_calls_tapinfo_t *tapinfo)
 	tapinfo->ncalls = 0;
 	tapinfo->npackets = 0;
 	tapinfo->start_packets = 0;
-    tapinfo->completed_calls = 0;
-    tapinfo->rejected_calls = 0;
+	tapinfo->completed_calls = 0;
+	tapinfo->rejected_calls = 0;
 
 	/* free the graph data items first */
 	list = g_list_first(tapinfo->graph_analysis->list);
@@ -190,7 +203,6 @@ void graph_analysis_data_init(void){
 	the_tapinfo_struct.graph_analysis = g_malloc(sizeof(graph_analysis_info_t));
 	the_tapinfo_struct.graph_analysis->nconv = 0;
 	the_tapinfo_struct.graph_analysis->list = NULL;
-
 }
 
 /****************************************************************************/
@@ -466,7 +478,7 @@ static void voip_rtp_reset(void *ptr _U_)
 /****************************************************************************/
 /* whenever a RTP packet is seen by the tap listener */
 static int 
-RTP_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, const void *RTPinfo)
+RTP_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, void const *RTPinfo)
 {
 	voip_rtp_tapinfo_t *tapinfo = &the_tapinfo_rtp_struct;
 	voip_rtp_stream_info_t *tmp_listinfo;
@@ -481,7 +493,14 @@ RTP_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, const vo
 		return 0;
 	}
 
-	/* check whether we already have a RTP stream with this setup frame and ssrc in the list */
+	/* add this RTP for future listening using the RTP Player*/
+#if GTK_MAJOR_VERSION >= 2
+#ifdef HAVE_LIBPORTAUDIO
+	add_rtp_packet(pi, pinfo);
+#endif
+#endif
+	
+	/* check wether we already have a RTP stream with this setup frame and ssrc in the list */
 	list = g_list_first(tapinfo->list);
 	while (list)
 	{
@@ -592,8 +611,14 @@ static void RTP_packet_draw(void *prs _U_)
 						g_free(gai->comment);
 						gai->comment = g_strdup_printf("RTP Num packets:%u  Duration:%u.%03us ssrc:%u", rtp_listinfo->npackets, duration/1000000,(duration%1000000)/1000, rtp_listinfo->ssrc);
 						break;
+					}
+
+					/* we increment the list here to be able to check if it is the last item in this calls, which means the RTP is after so we have to draw it */
+					voip_calls_graph_list = g_list_next(voip_calls_graph_list);
+					if (!voip_calls_graph_list) item++;
+
 					/* add the RTP item to the graph if was not there*/
-					} else if (rtp_listinfo->first_frame_num<gai->frame_num){
+					if (rtp_listinfo->first_frame_num<gai->frame_num || !voip_calls_graph_list){
 						new_gai = g_malloc(sizeof(graph_analysis_item_t));
 						new_gai->frame_num = rtp_listinfo->first_frame_num;
 						new_gai->time = (double)rtp_listinfo->start_rel_sec + (double)rtp_listinfo->start_rel_usec/1000000;
@@ -602,7 +627,7 @@ static void RTP_packet_draw(void *prs _U_)
 						new_gai->port_src = rtp_listinfo->src_port;
 						new_gai->port_dst = rtp_listinfo->dest_port;
 						duration = (rtp_listinfo->stop_rel_sec*1000000 + rtp_listinfo->stop_rel_usec) - (rtp_listinfo->start_rel_sec*1000000 + rtp_listinfo->start_rel_usec);
-						new_gai->frame_label = g_strdup_printf("RTP (%s) %s", rtp_listinfo->pt_str, (rtp_listinfo->rtp_event == -1)?"":val_to_str(rtp_listinfo->rtp_event, rtp_event_type_values, "Uknown RTP Event")); 
+						new_gai->frame_label = g_strdup_printf("RTP (%s) %s", rtp_listinfo->pt_str, (rtp_listinfo->rtp_event == -1)?"":val_to_str(rtp_listinfo->rtp_event, rtp_event_type_values, "Unknown RTP Event")); 
 						g_free(rtp_listinfo->pt_str);
 						new_gai->comment = g_strdup_printf("RTP Num packets:%u  Duration:%u.%03us ssrc:%u", rtp_listinfo->npackets, duration/1000000,(duration%1000000)/1000, rtp_listinfo->ssrc);
 						new_gai->conv_num = conv_num;
@@ -611,9 +636,7 @@ static void RTP_packet_draw(void *prs _U_)
 						the_tapinfo_struct.graph_analysis->list = g_list_insert(the_tapinfo_struct.graph_analysis->list, new_gai, item);
 						break;
 					}
-					
-					voip_calls_graph_list = g_list_next(voip_calls_graph_list);
-					item++;
+					if (voip_calls_graph_list) item++;
 				}
 				break;
 			}
@@ -659,6 +682,10 @@ remove_tap_listener_rtp(void)
 	have_RTP_tap_listener=FALSE;
 }
 
+/* XXX just copied from gtk/rpc_stat.c */
+void protect_thread_critical_region(void);
+void unprotect_thread_critical_region(void);
+
 /****************************************************************************/
 /******************************TAP for T38 **********************************/
 /****************************************************************************/
@@ -699,7 +726,7 @@ T38_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, const vo
 	}
 
 	/* if setup_frame_number in the t38 packet is 0, it means it was not set using an SDP or H245 sesion, which means we don't 
-	 * have the associated Voip calls. It probably means the the packet was decoded using the deafult t38 port, or using "Decode as.."
+	 * have the associated Voip calls. It probably means the the packet was decoded using the default t38 port, or using "Decode as.."
 	 * in this case we create a "voip" call that only have t38 media (no signaling)
 	 * OR if we have not found the Setup message in the graph. 
 	 */
