@@ -289,20 +289,27 @@ typedef union {
 ** functions that take a state number and lookahead value and return an
 ** action integer.  
 **
-** The action integer is a number N between
-** 0 and YYNSTATE-1 mean shift the look-ahead and go to state N.
-** Numbers between YYNSTATE and YYNSTATE+YYNRULE-1 mean reduce by
-** rule N-YYNSTATE.  Number YYNSTATE+YYNRULE means that a syntax
-** error has occurred.  Number YYNSTATE+YYNRULE+1 means the parser
-** accepts its input.
+** Suppose the action integer is N.  Then the action is determined as
+** follows
 **
-** The action table is constructed as a single large hash table with
-** a perfect hash.  Given state S and lookahead X, the action is computed
-** as
+**   0 <= N < YYNSTATE                  Shift N.  That is, push the lookahead
+**                                      token onto the stack and goto state N.
+**
+**   YYNSTATE <= N < YYNSTATE+YYNRULE   Reduce by rule N-YYNSTATE.
+**
+**   N == YYNSTATE+YYNRULE              A syntax error has occurred.
+**
+**   N == YYNSTATE+YYNRULE+1            The parser accepts its input.
+**
+**   N == YYNSTATE+YYNRULE+2            No such action.  Denotes unused
+**                                      slots in the yy_action[] table.
+**
+** The action table is constructed as a single large table named yy_action[].
+** Given state S and lookahead X, the action is computed as
 **
 **      yy_action[ yy_shift_ofst[S] + X ]
 **
-** If the index yy_shift_ofst[S]+X is out of range or if the value
+** If the index value yy_shift_ofst[S]+X is out of range or if the value
 ** yy_lookahead[yy_shift_ofst[S]+X] is not equal to X or if yy_shift_ofst[S]
 ** is equal to YY_SHIFT_USE_DFLT, it means that the action is not in the table
 ** and that yy_default[S] should be used instead.  
@@ -323,7 +330,7 @@ typedef union {
 **  yy_reduce_ofst[]   For each state, the offset into yy_action for
 **                     shifting non-terminals after a reduce.
 **  yy_default[]       Default action for each state.
- */
+*/
 static const YYACTIONTYPE yy_action[] = {
  /*     0 */   102,  183,  103,  228,  229,   19,  231,    5,  245,    4,
  /*    10 */   145,  157,  172,  119,  196,  197,  198,  199,  239,  240,
@@ -500,7 +507,6 @@ typedef struct yyStackEntry yyStackEntry;
 struct yyParser {
   int yyidx;                    /* Index of top element in stack */
   int yyerrcnt;                 /* Shifts left before out of the error */
-  yyStackEntry *yytop;          /* Pointer to the top stack element */
   MateParserARG_SDECL                /* A place to hold %extra_argument */
   yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
 };
@@ -541,7 +547,7 @@ void MateParserTrace(FILE *TraceFILE, char *zTracePrompt){
 #ifndef NDEBUG
 /* For tracing shifts, the names of all terminals and nonterminals
 ** are required.  The following table supplies these names */
-static const char *yyTokenName[] = {
+static const char *const yyTokenName[] = {
   "$",             "DONE_KW",       "SEMICOLON",     "DEBUG_KW",    
   "OPEN_BRACE",    "CLOSE_BRACE",   "FILENAME_KW",   "QUOTED",      
   "NAME",          "LEVEL_KW",      "INTEGER",       "PDU_KW",      
@@ -583,7 +589,7 @@ static const char *yyTokenName[] = {
 #ifndef NDEBUG
 /* For tracing reduce actions, the names of all rules are required.
 */
-static const char *yyRuleName[] = {
+static const char *const yyRuleName[] = {
  /*   0 */ "mate_config ::= decls",
  /*   1 */ "decls ::= decls decl",
  /*   2 */ "decls ::=",
@@ -750,7 +756,6 @@ const char *MateParserTokenName(int tokenType){
 #endif
 }
 
-
 /*
 ** This function allocates a new parser.
 ** The only argument is a pointer to a function which works like
@@ -852,7 +857,7 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
     case 61:
 #line 180 "./mate_grammar.lemon"
 { if ((yypminor->yy0)) g_free((yypminor->yy0)); }
-#line 857 "mate_grammar.c"
+#line 862 "mate_grammar.c"
       break;
     default:  break;   /* If no destructor action specified: do nothing */
   }
@@ -868,19 +873,19 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
 */
 static int yy_pop_parser_stack(yyParser *pParser){
   YYCODETYPE yymajor;
+  yyStackEntry *yytos = &pParser->yystack[pParser->yyidx];
 
   if( pParser->yyidx<0 ) return 0;
 #ifndef NDEBUG
   if( yyTraceFILE && pParser->yyidx>=0 ){
     fprintf(yyTraceFILE,"%sPopping %s\n",
       yyTracePrompt,
-     yyTokenName[pParser->yytop->major]);
+     yyTokenName[yytos->major]);
   }
 #endif
-  yymajor = pParser->yytop->major;
-  yy_destructor( yymajor, &pParser->yytop->minor);
+  yymajor = yytos->major;
+  yy_destructor( yymajor, &yytos->minor);
   pParser->yyidx--;
-  pParser->yytop--;
   return yymajor;
 }
 
@@ -898,7 +903,7 @@ static int yy_pop_parser_stack(yyParser *pParser){
 */
 void MateParserFree(
   void *p,                 /* The parser to be deleted */
-  void (*freeProc)(void *) /* Function used to reclaim memory */
+  void (*freeProc)(void*)  /* Function used to reclaim memory */
 ){
   yyParser *pParser = (yyParser*)p;
   if( pParser==0 ) return;
@@ -922,7 +927,7 @@ static int yy_find_shift_action(
   int stateno = pParser->yystack[pParser->yyidx].stateno;
 
   if( stateno>YY_SHIFT_MAX || (i = yy_shift_ofst[stateno])==YY_SHIFT_USE_DFLT ){
-    return yy_default[pParser->yytop->stateno];
+    return yy_default[stateno];
   }
   if( iLookAhead==YYNOCODE ){
     return YY_NO_ACTION;
@@ -942,7 +947,7 @@ static int yy_find_shift_action(
       return yy_find_shift_action(pParser, iFallback);
     }
 #endif
-    return yy_default[pParser->yytop->stateno];
+    return yy_default[stateno];
   }else{
     return yy_action[i];
   }
@@ -978,7 +983,6 @@ static int yy_find_reduce_action(
   }
 }
 
-
 /*
 ** Perform a shift action.
 */
@@ -988,12 +992,11 @@ static void yy_shift(
   int yyMajor,                  /* The major token to shift in */
   YYMINORTYPE *yypMinor         /* Pointer ot the minor token to shift in */
 ){
+  yyStackEntry *yytos;
   yypParser->yyidx++;
-  yypParser->yytop++;
   if( yypParser->yyidx>=YYSTACKDEPTH ){
      MateParserARG_FETCH;
      yypParser->yyidx--;
-     yypParser->yytop--;
 #ifndef NDEBUG
      if( yyTraceFILE ){
        fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
@@ -1005,9 +1008,10 @@ static void yy_shift(
 	 MateParserARG_STORE; /* Suppress warning about unused %extra_argument var */
      return;
   }
-  yypParser->yytop->stateno = yyNewState;
-  yypParser->yytop->major = yyMajor;
-  yypParser->yytop->minor = *yypMinor;
+  yytos = &yypParser->yystack[yypParser->yyidx];
+  yytos->stateno = yyNewState;
+  yytos->major = yyMajor;
+  yytos->minor = *yypMinor;
 #ifndef NDEBUG
   if( yyTraceFILE && yypParser->yyidx>0 ){
     int i;
@@ -1194,7 +1198,7 @@ static void yy_reduce(
   yyStackEntry *yymsp;            /* The top of the parser's stack */
   int yysize;                     /* Amount to pop the stack */
   MateParserARG_FETCH;
-  yymsp = yypParser->yytop;
+  yymsp = &yypParser->yystack[yypParser->yyidx];
 #ifndef NDEBUG
   if( yyTraceFILE && yyruleno>=0 
         && yyruleno<sizeof(yyRuleName)/sizeof(yyRuleName[0]) ){
@@ -1229,21 +1233,21 @@ static void yy_reduce(
 { mc->dbg_facility = fopen(yymsp[-1].minor.yy0,"w"); if (mc->dbg_facility == NULL) report_open_failure(yymsp[-1].minor.yy0,errno,TRUE);   yy_destructor(6,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1234 "mate_grammar.c"
+#line 1238 "mate_grammar.c"
         break;
       case 12:
 #line 267 "./mate_grammar.lemon"
 { mc->dbg_facility = fopen(yymsp[-1].minor.yy0,"w"); if (mc->dbg_facility == NULL) report_open_failure(yymsp[-1].minor.yy0,errno,TRUE);    yy_destructor(6,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1241 "mate_grammar.c"
+#line 1245 "mate_grammar.c"
         break;
       case 14:
 #line 270 "./mate_grammar.lemon"
 { mc->dbg_lvl = (int) strtol(yymsp[-1].minor.yy0,NULL,10);   yy_destructor(9,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1248 "mate_grammar.c"
+#line 1252 "mate_grammar.c"
         break;
       case 16:
 #line 273 "./mate_grammar.lemon"
@@ -1251,7 +1255,7 @@ static void yy_reduce(
   yy_destructor(9,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1256 "mate_grammar.c"
+#line 1260 "mate_grammar.c"
         break;
       case 18:
 #line 276 "./mate_grammar.lemon"
@@ -1259,7 +1263,7 @@ static void yy_reduce(
   yy_destructor(9,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1264 "mate_grammar.c"
+#line 1268 "mate_grammar.c"
         break;
       case 20:
 #line 279 "./mate_grammar.lemon"
@@ -1267,28 +1271,28 @@ static void yy_reduce(
   yy_destructor(9,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1272 "mate_grammar.c"
+#line 1276 "mate_grammar.c"
         break;
       case 25:
 #line 291 "./mate_grammar.lemon"
 { mc->defaults.pdu.last_extracted = yymsp[-1].minor.yy182;   yy_destructor(15,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1279 "mate_grammar.c"
+#line 1283 "mate_grammar.c"
         break;
       case 27:
 #line 294 "./mate_grammar.lemon"
 { mc->defaults.pdu.drop_unassigned = yymsp[-1].minor.yy182;   yy_destructor(16,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1286 "mate_grammar.c"
+#line 1290 "mate_grammar.c"
         break;
       case 29:
 #line 297 "./mate_grammar.lemon"
 { mc->defaults.pdu.discard = yymsp[-1].minor.yy182;   yy_destructor(17,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1293 "mate_grammar.c"
+#line 1297 "mate_grammar.c"
         break;
       case 33:
       case 47:
@@ -1296,49 +1300,49 @@ static void yy_reduce(
 { mc->defaults.gop.expiration = yymsp[-1].minor.yy255;   yy_destructor(18,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1301 "mate_grammar.c"
+#line 1305 "mate_grammar.c"
         break;
       case 35:
 #line 306 "./mate_grammar.lemon"
 { mc->defaults.gop.idle_timeout = yymsp[-1].minor.yy255;   yy_destructor(19,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1308 "mate_grammar.c"
+#line 1312 "mate_grammar.c"
         break;
       case 37:
 #line 309 "./mate_grammar.lemon"
 { mc->defaults.gop.lifetime = yymsp[-1].minor.yy255;   yy_destructor(20,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1315 "mate_grammar.c"
+#line 1319 "mate_grammar.c"
         break;
       case 39:
 #line 312 "./mate_grammar.lemon"
 { mc->defaults.gop.drop_unassigned = yymsp[-1].minor.yy182;   yy_destructor(16,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1322 "mate_grammar.c"
+#line 1326 "mate_grammar.c"
         break;
       case 41:
 #line 315 "./mate_grammar.lemon"
 { mc->defaults.gop.pdu_tree_mode = yymsp[-1].minor.yy256;   yy_destructor(21,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1329 "mate_grammar.c"
+#line 1333 "mate_grammar.c"
         break;
       case 43:
 #line 318 "./mate_grammar.lemon"
 { mc->defaults.gop.show_times = yymsp[-1].minor.yy182;   yy_destructor(22,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1336 "mate_grammar.c"
+#line 1340 "mate_grammar.c"
         break;
       case 49:
 #line 327 "./mate_grammar.lemon"
 { mc->defaults.gog.gop_tree_mode = yymsp[-1].minor.yy256;   yy_destructor(23,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1343 "mate_grammar.c"
+#line 1347 "mate_grammar.c"
         break;
       case 51:
 #line 334 "./mate_grammar.lemon"
@@ -1358,14 +1362,14 @@ static void yy_reduce(
   yy_destructor(24,&yymsp[-3].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1363 "mate_grammar.c"
+#line 1367 "mate_grammar.c"
         break;
       case 52:
 #line 349 "./mate_grammar.lemon"
 { yygotominor.yy11 = yymsp[-1].minor.yy11;   yy_destructor(4,&yymsp[-2].minor);
   yy_destructor(5,&yymsp[0].minor);
 }
-#line 1370 "mate_grammar.c"
+#line 1374 "mate_grammar.c"
         break;
       case 53:
 #line 351 "./mate_grammar.lemon"
@@ -1376,12 +1380,12 @@ static void yy_reduce(
 	c->next = yymsp[0].minor.yy11;
 	yygotominor.yy11 = yymsp[-1].minor.yy11;
 }
-#line 1381 "mate_grammar.c"
+#line 1385 "mate_grammar.c"
         break;
       case 54:
 #line 359 "./mate_grammar.lemon"
 { yygotominor.yy11 = yymsp[0].minor.yy11; }
-#line 1386 "mate_grammar.c"
+#line 1390 "mate_grammar.c"
         break;
       case 55:
 #line 361 "./mate_grammar.lemon"
@@ -1389,7 +1393,7 @@ static void yy_reduce(
 	yygotominor.yy11 = new_transform_elem(yymsp[-2].minor.yy146->avpl,yymsp[-1].minor.yy85->avpl,yymsp[-2].minor.yy146->match_mode,yymsp[-1].minor.yy85->replace_mode);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1394 "mate_grammar.c"
+#line 1398 "mate_grammar.c"
         break;
       case 56:
 #line 365 "./mate_grammar.lemon"
@@ -1399,7 +1403,7 @@ static void yy_reduce(
     yygotominor.yy146->avpl = yymsp[0].minor.yy70;
   yy_destructor(25,&yymsp[-2].minor);
 }
-#line 1404 "mate_grammar.c"
+#line 1408 "mate_grammar.c"
         break;
       case 57:
 #line 371 "./mate_grammar.lemon"
@@ -1409,7 +1413,7 @@ static void yy_reduce(
     yygotominor.yy146->avpl = new_avpl("");
 
 }
-#line 1414 "mate_grammar.c"
+#line 1418 "mate_grammar.c"
         break;
       case 58:
 #line 378 "./mate_grammar.lemon"
@@ -1418,7 +1422,7 @@ static void yy_reduce(
     yygotominor.yy85->replace_mode = AVPL_INSERT;
     yygotominor.yy85->avpl = new_avpl("");
 }
-#line 1423 "mate_grammar.c"
+#line 1427 "mate_grammar.c"
         break;
       case 59:
 #line 383 "./mate_grammar.lemon"
@@ -1427,47 +1431,47 @@ static void yy_reduce(
     yygotominor.yy85->replace_mode = yymsp[-1].minor.yy143;
     yygotominor.yy85->avpl = yymsp[0].minor.yy70;
 }
-#line 1432 "mate_grammar.c"
+#line 1436 "mate_grammar.c"
         break;
       case 60:
 #line 389 "./mate_grammar.lemon"
 { yygotominor.yy274 = AVPL_STRICT; }
-#line 1437 "mate_grammar.c"
+#line 1441 "mate_grammar.c"
         break;
       case 61:
 #line 390 "./mate_grammar.lemon"
 { yygotominor.yy274 = AVPL_STRICT;   yy_destructor(26,&yymsp[0].minor);
 }
-#line 1443 "mate_grammar.c"
+#line 1447 "mate_grammar.c"
         break;
       case 62:
 #line 391 "./mate_grammar.lemon"
 { yygotominor.yy274 = AVPL_EVERY;   yy_destructor(27,&yymsp[0].minor);
 }
-#line 1449 "mate_grammar.c"
+#line 1453 "mate_grammar.c"
         break;
       case 63:
 #line 392 "./mate_grammar.lemon"
 { yygotominor.yy274 = AVPL_LOOSE;   yy_destructor(28,&yymsp[0].minor);
 }
-#line 1455 "mate_grammar.c"
+#line 1459 "mate_grammar.c"
         break;
       case 64:
 #line 394 "./mate_grammar.lemon"
 { yygotominor.yy143 = AVPL_REPLACE;   yy_destructor(29,&yymsp[0].minor);
 }
-#line 1461 "mate_grammar.c"
+#line 1465 "mate_grammar.c"
         break;
       case 65:
 #line 395 "./mate_grammar.lemon"
 { yygotominor.yy143 = AVPL_INSERT;   yy_destructor(30,&yymsp[0].minor);
 }
-#line 1467 "mate_grammar.c"
+#line 1471 "mate_grammar.c"
         break;
       case 66:
 #line 396 "./mate_grammar.lemon"
 { yygotominor.yy143 = AVPL_INSERT; }
-#line 1472 "mate_grammar.c"
+#line 1476 "mate_grammar.c"
         break;
       case 67:
 #line 412 "./mate_grammar.lemon"
@@ -1522,24 +1526,24 @@ static void yy_reduce(
   yy_destructor(5,&yymsp[-1].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1527 "mate_grammar.c"
+#line 1531 "mate_grammar.c"
         break;
       case 68:
 #line 458 "./mate_grammar.lemon"
 { yygotominor.yy147 = NULL; }
-#line 1532 "mate_grammar.c"
+#line 1536 "mate_grammar.c"
         break;
       case 69:
 #line 459 "./mate_grammar.lemon"
 { yygotominor.yy147 = yymsp[-1].minor.yy147;   yy_destructor(33,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1539 "mate_grammar.c"
+#line 1543 "mate_grammar.c"
         break;
       case 70:
 #line 461 "./mate_grammar.lemon"
 { yygotominor.yy231 = NULL; }
-#line 1544 "mate_grammar.c"
+#line 1548 "mate_grammar.c"
         break;
       case 71:
 #line 462 "./mate_grammar.lemon"
@@ -1551,34 +1555,34 @@ static void yy_reduce(
   yy_destructor(34,&yymsp[-4].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1556 "mate_grammar.c"
+#line 1560 "mate_grammar.c"
         break;
       case 72:
 #line 469 "./mate_grammar.lemon"
 { yygotominor.yy148 = ACCEPT_MODE; }
-#line 1561 "mate_grammar.c"
+#line 1565 "mate_grammar.c"
         break;
       case 73:
 #line 470 "./mate_grammar.lemon"
 { yygotominor.yy148 = ACCEPT_MODE;   yy_destructor(35,&yymsp[0].minor);
 }
-#line 1567 "mate_grammar.c"
+#line 1571 "mate_grammar.c"
         break;
       case 74:
 #line 471 "./mate_grammar.lemon"
 { yygotominor.yy148 = REJECT_MODE;   yy_destructor(36,&yymsp[0].minor);
 }
-#line 1573 "mate_grammar.c"
+#line 1577 "mate_grammar.c"
         break;
       case 75:
 #line 473 "./mate_grammar.lemon"
 { yygotominor.yy179 = yymsp[-1].minor.yy179; yygotominor.yy179->last = yygotominor.yy179->last->next = yymsp[0].minor.yy179; }
-#line 1578 "mate_grammar.c"
+#line 1582 "mate_grammar.c"
         break;
       case 76:
 #line 474 "./mate_grammar.lemon"
 { yygotominor.yy179 = yymsp[0].minor.yy179; yygotominor.yy179->last = yygotominor.yy179; }
-#line 1583 "mate_grammar.c"
+#line 1587 "mate_grammar.c"
         break;
       case 77:
 #line 476 "./mate_grammar.lemon"
@@ -1591,7 +1595,7 @@ static void yy_reduce(
   yy_destructor(38,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1596 "mate_grammar.c"
+#line 1600 "mate_grammar.c"
         break;
       case 78:
       case 88:
@@ -1599,36 +1603,36 @@ static void yy_reduce(
 { yygotominor.yy182 = yymsp[-1].minor.yy182;   yy_destructor(16,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1604 "mate_grammar.c"
+#line 1608 "mate_grammar.c"
         break;
       case 79:
 #line 485 "./mate_grammar.lemon"
 { yygotominor.yy182 =  mc->defaults.pdu.drop_unassigned; }
-#line 1609 "mate_grammar.c"
+#line 1613 "mate_grammar.c"
         break;
       case 80:
 #line 487 "./mate_grammar.lemon"
 { yygotominor.yy182 = yymsp[-1].minor.yy182;   yy_destructor(17,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1616 "mate_grammar.c"
+#line 1620 "mate_grammar.c"
         break;
       case 81:
 #line 488 "./mate_grammar.lemon"
 { yygotominor.yy182 =  mc->defaults.pdu.discard; }
-#line 1621 "mate_grammar.c"
+#line 1625 "mate_grammar.c"
         break;
       case 82:
 #line 490 "./mate_grammar.lemon"
 { yygotominor.yy182 = yymsp[-1].minor.yy182;   yy_destructor(39,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1628 "mate_grammar.c"
+#line 1632 "mate_grammar.c"
         break;
       case 83:
 #line 491 "./mate_grammar.lemon"
 { yygotominor.yy182 = mc->defaults.pdu.last_extracted; }
-#line 1633 "mate_grammar.c"
+#line 1637 "mate_grammar.c"
         break;
       case 84:
 #line 493 "./mate_grammar.lemon"
@@ -1642,7 +1646,7 @@ static void yy_reduce(
 	yygotominor.yy147 = yymsp[-2].minor.yy147;
   yy_destructor(40,&yymsp[-1].minor);
 }
-#line 1647 "mate_grammar.c"
+#line 1651 "mate_grammar.c"
         break;
       case 85:
 #line 503 "./mate_grammar.lemon"
@@ -1655,14 +1659,14 @@ static void yy_reduce(
 	yygotominor.yy147 = g_ptr_array_new();
 	g_ptr_array_add(yygotominor.yy147,hfidp);
 }
-#line 1660 "mate_grammar.c"
+#line 1664 "mate_grammar.c"
         break;
       case 86:
 #line 513 "./mate_grammar.lemon"
 {
 	yygotominor.yy210 = proto_registrar_get_byname(yymsp[0].minor.yy0);
 }
-#line 1667 "mate_grammar.c"
+#line 1671 "mate_grammar.c"
         break;
       case 87:
 #line 531 "./mate_grammar.lemon"
@@ -1697,56 +1701,56 @@ static void yy_reduce(
   yy_destructor(5,&yymsp[-1].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1702 "mate_grammar.c"
+#line 1706 "mate_grammar.c"
         break;
       case 89:
 #line 558 "./mate_grammar.lemon"
 { yygotominor.yy182 =  mc->defaults.gop.drop_unassigned; }
-#line 1707 "mate_grammar.c"
+#line 1711 "mate_grammar.c"
         break;
       case 90:
 #line 560 "./mate_grammar.lemon"
 { yygotominor.yy70 = yymsp[-1].minor.yy70;   yy_destructor(42,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1714 "mate_grammar.c"
+#line 1718 "mate_grammar.c"
         break;
       case 91:
       case 93:
 #line 561 "./mate_grammar.lemon"
 { yygotominor.yy70 = NULL; }
-#line 1720 "mate_grammar.c"
+#line 1724 "mate_grammar.c"
         break;
       case 92:
 #line 563 "./mate_grammar.lemon"
 { yygotominor.yy70 = yymsp[-1].minor.yy70;   yy_destructor(43,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1727 "mate_grammar.c"
+#line 1731 "mate_grammar.c"
         break;
       case 94:
 #line 566 "./mate_grammar.lemon"
 { yygotominor.yy256 = yymsp[-1].minor.yy256;   yy_destructor(21,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1734 "mate_grammar.c"
+#line 1738 "mate_grammar.c"
         break;
       case 95:
 #line 567 "./mate_grammar.lemon"
 { yygotominor.yy256 = mc->defaults.gop.pdu_tree_mode; }
-#line 1739 "mate_grammar.c"
+#line 1743 "mate_grammar.c"
         break;
       case 96:
 #line 569 "./mate_grammar.lemon"
 { yygotominor.yy182 = yymsp[-1].minor.yy182;   yy_destructor(22,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1746 "mate_grammar.c"
+#line 1750 "mate_grammar.c"
         break;
       case 97:
 #line 570 "./mate_grammar.lemon"
 { yygotominor.yy182 = mc->defaults.gop.show_times; }
-#line 1751 "mate_grammar.c"
+#line 1755 "mate_grammar.c"
         break;
       case 98:
       case 116:
@@ -1754,64 +1758,64 @@ static void yy_reduce(
 { yygotominor.yy255 = yymsp[-1].minor.yy255;   yy_destructor(18,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1759 "mate_grammar.c"
+#line 1763 "mate_grammar.c"
         break;
       case 99:
       case 101:
       case 103:
 #line 573 "./mate_grammar.lemon"
 { yygotominor.yy255 = mc->defaults.gop.lifetime; }
-#line 1766 "mate_grammar.c"
+#line 1770 "mate_grammar.c"
         break;
       case 100:
 #line 575 "./mate_grammar.lemon"
 { yygotominor.yy255 = yymsp[-1].minor.yy255;   yy_destructor(19,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1773 "mate_grammar.c"
+#line 1777 "mate_grammar.c"
         break;
       case 102:
 #line 578 "./mate_grammar.lemon"
 { yygotominor.yy255 = yymsp[-1].minor.yy255;   yy_destructor(20,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1780 "mate_grammar.c"
+#line 1784 "mate_grammar.c"
         break;
       case 104:
 #line 581 "./mate_grammar.lemon"
 { yygotominor.yy256 = GOP_NO_TREE;   yy_destructor(44,&yymsp[0].minor);
 }
-#line 1786 "mate_grammar.c"
+#line 1790 "mate_grammar.c"
         break;
       case 105:
 #line 582 "./mate_grammar.lemon"
 { yygotominor.yy256 = GOP_PDU_TREE;   yy_destructor(45,&yymsp[0].minor);
 }
-#line 1792 "mate_grammar.c"
+#line 1796 "mate_grammar.c"
         break;
       case 106:
 #line 583 "./mate_grammar.lemon"
 { yygotominor.yy256 = GOP_FRAME_TREE;   yy_destructor(46,&yymsp[0].minor);
 }
-#line 1798 "mate_grammar.c"
+#line 1802 "mate_grammar.c"
         break;
       case 107:
 #line 584 "./mate_grammar.lemon"
 { yygotominor.yy256 = GOP_BASIC_PDU_TREE;   yy_destructor(47,&yymsp[0].minor);
 }
-#line 1804 "mate_grammar.c"
+#line 1808 "mate_grammar.c"
         break;
       case 108:
 #line 586 "./mate_grammar.lemon"
 { yygotominor.yy182 = TRUE;   yy_destructor(48,&yymsp[0].minor);
 }
-#line 1810 "mate_grammar.c"
+#line 1814 "mate_grammar.c"
         break;
       case 109:
 #line 587 "./mate_grammar.lemon"
 { yygotominor.yy182 = FALSE;   yy_destructor(49,&yymsp[0].minor);
 }
-#line 1816 "mate_grammar.c"
+#line 1820 "mate_grammar.c"
         break;
       case 110:
 #line 589 "./mate_grammar.lemon"
@@ -1823,7 +1827,7 @@ static void yy_reduce(
 		configuration_error(mc,"No such Pdu: '%s'",yymsp[0].minor.yy0);
 	}
 }
-#line 1828 "mate_grammar.c"
+#line 1832 "mate_grammar.c"
         break;
       case 111:
       case 112:
@@ -1831,7 +1835,7 @@ static void yy_reduce(
 {
 	yygotominor.yy255 = (float) strtod(yymsp[0].minor.yy0,NULL);
 }
-#line 1836 "mate_grammar.c"
+#line 1840 "mate_grammar.c"
         break;
       case 113:
 #line 616 "./mate_grammar.lemon"
@@ -1856,42 +1860,42 @@ static void yy_reduce(
   yy_destructor(5,&yymsp[-1].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1861 "mate_grammar.c"
+#line 1865 "mate_grammar.c"
         break;
       case 114:
 #line 634 "./mate_grammar.lemon"
 { yygotominor.yy256 = yymsp[-1].minor.yy256;   yy_destructor(23,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1868 "mate_grammar.c"
+#line 1872 "mate_grammar.c"
         break;
       case 115:
 #line 635 "./mate_grammar.lemon"
 { yygotominor.yy256 = mc->defaults.gog.gop_tree_mode; }
-#line 1873 "mate_grammar.c"
+#line 1877 "mate_grammar.c"
         break;
       case 117:
 #line 638 "./mate_grammar.lemon"
 { yygotominor.yy255 = mc->defaults.gog.expiration; }
-#line 1878 "mate_grammar.c"
+#line 1882 "mate_grammar.c"
         break;
       case 118:
 #line 640 "./mate_grammar.lemon"
 { yygotominor.yy256 = GOP_NULL_TREE;   yy_destructor(51,&yymsp[0].minor);
 }
-#line 1884 "mate_grammar.c"
+#line 1888 "mate_grammar.c"
         break;
       case 119:
 #line 641 "./mate_grammar.lemon"
 { yygotominor.yy256 = GOP_FULL_TREE;   yy_destructor(52,&yymsp[0].minor);
 }
-#line 1890 "mate_grammar.c"
+#line 1894 "mate_grammar.c"
         break;
       case 120:
 #line 642 "./mate_grammar.lemon"
 { yygotominor.yy256 = GOP_BASIC_TREE;   yy_destructor(47,&yymsp[0].minor);
 }
-#line 1896 "mate_grammar.c"
+#line 1900 "mate_grammar.c"
         break;
       case 121:
 #line 644 "./mate_grammar.lemon"
@@ -1899,7 +1903,7 @@ static void yy_reduce(
 	loal_append(yymsp[-1].minor.yy77,yymsp[0].minor.yy70);
 	yygotominor.yy77 = yymsp[-1].minor.yy77;
 }
-#line 1904 "mate_grammar.c"
+#line 1908 "mate_grammar.c"
         break;
       case 122:
 #line 649 "./mate_grammar.lemon"
@@ -1907,7 +1911,7 @@ static void yy_reduce(
 	yygotominor.yy77 = new_loal("");
 	loal_append(yygotominor.yy77,yymsp[0].minor.yy70);
 }
-#line 1912 "mate_grammar.c"
+#line 1916 "mate_grammar.c"
         break;
       case 123:
 #line 655 "./mate_grammar.lemon"
@@ -1917,7 +1921,7 @@ static void yy_reduce(
   yy_destructor(53,&yymsp[-3].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1922 "mate_grammar.c"
+#line 1926 "mate_grammar.c"
         break;
       case 124:
 #line 660 "./mate_grammar.lemon"
@@ -1929,31 +1933,31 @@ static void yy_reduce(
 		configuration_error(mc,"No Gop called '%s' has been already declared",yymsp[0].minor.yy0);
 	}
 }
-#line 1934 "mate_grammar.c"
+#line 1938 "mate_grammar.c"
         break;
       case 125:
 #line 672 "./mate_grammar.lemon"
 { yygotominor.yy70 = yymsp[-1].minor.yy70;   yy_destructor(54,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1941 "mate_grammar.c"
+#line 1945 "mate_grammar.c"
         break;
       case 126:
 #line 673 "./mate_grammar.lemon"
 { yygotominor.yy70 = new_avpl(""); }
-#line 1946 "mate_grammar.c"
+#line 1950 "mate_grammar.c"
         break;
       case 127:
 #line 675 "./mate_grammar.lemon"
 { yygotominor.yy147 = yymsp[-1].minor.yy147;   yy_destructor(24,&yymsp[-2].minor);
   yy_destructor(2,&yymsp[0].minor);
 }
-#line 1953 "mate_grammar.c"
+#line 1957 "mate_grammar.c"
         break;
       case 128:
 #line 676 "./mate_grammar.lemon"
 { yygotominor.yy147 = g_ptr_array_new(); }
-#line 1958 "mate_grammar.c"
+#line 1962 "mate_grammar.c"
         break;
       case 129:
 #line 678 "./mate_grammar.lemon"
@@ -1962,7 +1966,7 @@ static void yy_reduce(
 	g_ptr_array_add(yymsp[-2].minor.yy147,yymsp[0].minor.yy11);
   yy_destructor(55,&yymsp[-1].minor);
 }
-#line 1967 "mate_grammar.c"
+#line 1971 "mate_grammar.c"
         break;
       case 130:
 #line 683 "./mate_grammar.lemon"
@@ -1970,7 +1974,7 @@ static void yy_reduce(
 	yygotominor.yy147 = g_ptr_array_new();
 	g_ptr_array_add(yygotominor.yy147,yymsp[0].minor.yy11);
 }
-#line 1975 "mate_grammar.c"
+#line 1979 "mate_grammar.c"
         break;
       case 131:
 #line 688 "./mate_grammar.lemon"
@@ -1983,60 +1987,60 @@ static void yy_reduce(
 		configuration_error(mc,"There's no such Transformation: %s",yymsp[0].minor.yy0);
 	}	
 }
-#line 1988 "mate_grammar.c"
+#line 1992 "mate_grammar.c"
         break;
       case 132:
 #line 698 "./mate_grammar.lemon"
 { yygotominor.yy70 = yymsp[-1].minor.yy70;   yy_destructor(56,&yymsp[-2].minor);
   yy_destructor(57,&yymsp[0].minor);
 }
-#line 1995 "mate_grammar.c"
+#line 1999 "mate_grammar.c"
         break;
       case 133:
 #line 699 "./mate_grammar.lemon"
 { yygotominor.yy70 = new_avpl("");   yy_destructor(56,&yymsp[-1].minor);
   yy_destructor(57,&yymsp[0].minor);
 }
-#line 2002 "mate_grammar.c"
+#line 2006 "mate_grammar.c"
         break;
       case 134:
 #line 701 "./mate_grammar.lemon"
 { yygotominor.yy70 = yymsp[-2].minor.yy70; if ( ! insert_avp(yymsp[-2].minor.yy70,yymsp[0].minor.yy226) ) delete_avp(yymsp[0].minor.yy226);   yy_destructor(55,&yymsp[-1].minor);
 }
-#line 2008 "mate_grammar.c"
+#line 2012 "mate_grammar.c"
         break;
       case 135:
 #line 702 "./mate_grammar.lemon"
 { yygotominor.yy70 = new_avpl(""); if ( ! insert_avp(yygotominor.yy70,yymsp[0].minor.yy226) ) delete_avp(yymsp[0].minor.yy226); }
-#line 2013 "mate_grammar.c"
+#line 2017 "mate_grammar.c"
         break;
       case 136:
 #line 704 "./mate_grammar.lemon"
 { yygotominor.yy226 = new_avp(yymsp[-2].minor.yy0,yymsp[0].minor.yy212,*yymsp[-1].minor.yy0); }
-#line 2018 "mate_grammar.c"
+#line 2022 "mate_grammar.c"
         break;
       case 137:
 #line 705 "./mate_grammar.lemon"
 { yygotominor.yy226 = new_avp(yymsp[0].minor.yy0,"",'?'); }
-#line 2023 "mate_grammar.c"
+#line 2027 "mate_grammar.c"
         break;
       case 138:
 #line 706 "./mate_grammar.lemon"
 { yygotominor.yy226 = new_avp(yymsp[-3].minor.yy0,yymsp[-1].minor.yy212,'|');   yy_destructor(4,&yymsp[-2].minor);
   yy_destructor(5,&yymsp[0].minor);
 }
-#line 2030 "mate_grammar.c"
+#line 2034 "mate_grammar.c"
         break;
       case 139:
 #line 708 "./mate_grammar.lemon"
 { yygotominor.yy212 = g_strdup_printf("%s|%s",yymsp[-2].minor.yy212,yymsp[0].minor.yy212);   yy_destructor(59,&yymsp[-1].minor);
 }
-#line 2036 "mate_grammar.c"
+#line 2040 "mate_grammar.c"
         break;
       case 140:
 #line 709 "./mate_grammar.lemon"
 { yygotominor.yy212 = g_strdup(yymsp[0].minor.yy212); }
-#line 2041 "mate_grammar.c"
+#line 2045 "mate_grammar.c"
         break;
       case 141:
       case 142:
@@ -2045,18 +2049,17 @@ static void yy_reduce(
       case 145:
 #line 711 "./mate_grammar.lemon"
 { yygotominor.yy212 = g_strdup(yymsp[0].minor.yy0); }
-#line 2050 "mate_grammar.c"
+#line 2054 "mate_grammar.c"
         break;
       case 146:
 #line 716 "./mate_grammar.lemon"
 { yygotominor.yy212 = recolonize(mc,yymsp[0].minor.yy0); }
-#line 2055 "mate_grammar.c"
+#line 2059 "mate_grammar.c"
         break;
   };
   yygoto = yyRuleInfo[yyruleno].lhs;
   yysize = yyRuleInfo[yyruleno].nrhs;
   yypParser->yyidx -= yysize;
-  yypParser->yytop -= yysize;
   yyact = yy_find_reduce_action(yymsp[-yysize].stateno,yygoto);
   if( yyact < YYNSTATE ){
 #ifdef NDEBUG
@@ -2098,7 +2101,7 @@ static void yy_parse_failed(
 #line 188 "./mate_grammar.lemon"
 
 	configuration_error(mc,"Parse Error");
-#line 2104 "mate_grammar.c"
+#line 2107 "mate_grammar.c"
   MateParserARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
@@ -2115,7 +2118,7 @@ static void yy_syntax_error(
 #line 184 "./mate_grammar.lemon"
 
 	configuration_error(mc,"Syntax Error before %s",yyminor);
-#line 2122 "mate_grammar.c"
+#line 2125 "mate_grammar.c"
   MateParserARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
@@ -2174,9 +2177,8 @@ void MateParser(
     /* if( yymajor==0 ) return; // not sure why this was here... */
     yypParser->yyidx = 0;
     yypParser->yyerrcnt = -1;
-    yypParser->yytop = &yypParser->yystack[0];
-    yypParser->yytop->stateno = 0;
-    yypParser->yytop->major = 0;
+    yypParser->yystack[0].stateno = 0;
+    yypParser->yystack[0].major = 0;
   }
   yyminorunion.yy0 = yyminor;
   yyendofinput = (yymajor==0);
@@ -2201,6 +2203,7 @@ void MateParser(
     }else if( yyact < YYNSTATE + YYNRULE ){
       yy_reduce(yypParser,yyact-YYNSTATE);
     }else if( yyact == YY_ERROR_ACTION ){
+      int yymx;
 #ifndef NDEBUG
       if( yyTraceFILE ){
         fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
@@ -2229,7 +2232,8 @@ void MateParser(
       if( yypParser->yyerrcnt<0 ){
         yy_syntax_error(yypParser,yymajor,yyminorunion);
       }
-     if( yypParser->yytop->major==YYERRORSYMBOL || yyerrorhit ){
+      yymx = yypParser->yystack[yypParser->yyidx].major;
+      if( yymx==YYERRORSYMBOL || yyerrorhit ){
 #ifndef NDEBUG
         if( yyTraceFILE ){
           fprintf(yyTraceFILE,"%sDiscard input token %s\n",
@@ -2241,7 +2245,7 @@ void MateParser(
       }else{
          while(
           yypParser->yyidx >= 0 &&
-          yypParser->yytop->major != YYERRORSYMBOL &&
+          yymx != YYERRORSYMBOL &&
           (yyact = yy_find_shift_action(yypParser,YYERRORSYMBOL)) >= YYNSTATE
         ){
           yy_pop_parser_stack(yypParser);
@@ -2250,7 +2254,7 @@ void MateParser(
           yy_destructor(yymajor,&yyminorunion);
           yy_parse_failed(yypParser);
           yymajor = YYNOCODE;
-        }else if( yypParser->yytop->major!=YYERRORSYMBOL ){
+        }else if( yymx!=YYERRORSYMBOL ){
           YYMINORTYPE u2;
           u2.YYERRSYMDT = 0;
           yy_shift(yypParser,yyact,YYERRORSYMBOL,&u2);
@@ -2285,3 +2289,4 @@ void MateParser(
   }while( yymajor!=YYNOCODE && yypParser->yyidx>=0 );
   return;
 }
+
