@@ -1,7 +1,7 @@
 /* version_info.c
  * Routines to report version information for stuff used by Wireshark
  *
- * $Id: version_info.c 18235 2006-05-28 20:28:20Z etxrab $
+ * $Id: version_info.c 19643 2006-10-20 20:21:52Z ulfl $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -52,12 +52,25 @@
 
 #endif /* HAVE_SOME_SNMP */
 
+#if (defined(HAVE_LIBGCRYPT) || defined(HAVE_LIBGNUTLS)) && defined(_WIN32)
+#include <winposixtype.h>
+#endif
+
+#ifdef HAVE_LIBGCRYPT
+#include <gcrypt.h>
+#endif /* HAVE_LIBGCRYPT */
+
+#ifdef HAVE_LIBGNUTLS
+#include <gnutls/gnutls.h>
+#endif /* HAVE_LIBGNUTLS */
+
 #ifdef HAVE_SYS_UTSNAME_H
 #include <sys/utsname.h>
 #endif
 
 #include "version_info.h"
 #include "capture-pcap-util.h"
+#include "epan/unicode-utils.h"
 
 #include "svnversion.h"
 
@@ -75,65 +88,64 @@
 	const char *svnversion = "";
 #endif
 
-/*
- * See whether the last line in the string goes past column 80; if so,
- * replace the blank at the specified point with a newline.
- */
-static void
-do_word_wrap(GString *str, gint point)
-{
-	char *line_begin;
-
-	line_begin = strrchr(str->str, '\n');
-	if (line_begin == NULL)
-		line_begin = str->str;
-	else
-		line_begin++;
-	if (strlen(line_begin) > 80) {
-		g_assert(str->str[point] == ' ');
-		str->str[point] = '\n';
-	}
-}	
-
 /*
  * If the string doesn't end with a newline, append one.
+ * Then word-wrap it to 80 columns.
  */
 static void
 end_string(GString *str)
 {
 	size_t point;
+	char *p, *q;
 
 	point = strlen(str->str);
 	if (point == 0 || str->str[point - 1] != '\n')
 		g_string_append(str, "\n");
-}	
+	p = str->str;
+	while (*p != '\0') {
+		q = strchr(p, '\n');
+		if (q - p > 80) {
+			/*
+			 * Break at or before this point.
+			 */
+			q = p + 80;
+			while (q > p && *q != ' ')
+				q--;
+			if (q != p)
+				*q = '\n';
+		}
+		p = q + 1;
+	}
+}
 
 /*
  * Get various library compile-time versions and append them to
  * the specified GString.
+ *
+ * "additional_info" is called at the end to append any additional
+ * information; this is required in order to, for example, put the
+ * Portaudio information at the end of the string, as we currently
+ * don't use Portaudio in TShark.
  */
 void
-get_compiled_version_info(GString *str)
+get_compiled_version_info(GString *str, void (*additional_info)(GString *))
 {
-	gint break_point;
-
+        /* GLIB */
 	g_string_append(str, "with ");
 	g_string_sprintfa(str,
 #ifdef GLIB_MAJOR_VERSION
-	    "GLib %d.%d.%d,", GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION,
+	    "GLib %d.%d.%d", GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION,
 	    GLIB_MICRO_VERSION);
 #else
-	    "GLib (version unknown),");
+	    "GLib (version unknown)");
 #endif
 
-	g_string_append(str, " ");
-	break_point = str->len - 1;
+	/* Libpcap */
+	g_string_append(str, ", ");
 	get_compiled_pcap_version(str);
-	g_string_append(str, ",");
-	do_word_wrap(str, break_point);
 
-	g_string_append(str, " ");
-	break_point = str->len - 1;
+        /* LIBZ */
+	g_string_append(str, ", ");
 #ifdef HAVE_LIBZ
 	g_string_append(str, "with libz ");
 #ifdef ZLIB_VERSION
@@ -144,11 +156,30 @@ get_compiled_version_info(GString *str)
 #else /* HAVE_LIBZ */
 	g_string_append(str, "without libz");
 #endif /* HAVE_LIBZ */
-	g_string_append(str, ",");
-	do_word_wrap(str, break_point);
 
-	g_string_append(str, " ");
-	break_point = str->len - 1;
+	/* Additional application-dependent information */
+	if (additional_info)
+		(*additional_info)(str);
+	g_string_append(str, ".");
+
+#ifndef HAVE_LIBPCRE
+	g_string_append(str,
+	"\nNOTE: this build doesn't support the \"matches\" operator for Wireshark filter syntax");
+	g_string_append(str, ".");
+#endif	/* HAVE_LIBPCRE */
+
+	end_string(str);
+}
+
+/*
+ * Get compile-time information used only by applications that use
+ * libethereal.
+ */
+void
+get_epan_compiled_version_info(GString *str)
+{
+        /* PCRE */
+	g_string_append(str, ", ");
 #ifdef HAVE_LIBPCRE
 	g_string_append(str, "with libpcre ");
 #ifdef PCRE_MAJOR
@@ -164,13 +195,10 @@ get_compiled_version_info(GString *str)
 	g_string_append(str, "without libpcre");
 #endif	/* HAVE_LIBPCRE */
 
-	g_string_append(str, ",");
-	do_word_wrap(str, break_point);
-
+        /* SNMP */
+	g_string_append(str, ", ");
 /* Oh, this is pretty. */
 /* Oh, ha.  you think that was pretty.  Try this:! --Wes */
-	g_string_append(str, " ");
-	break_point = str->len - 1;
 #ifdef HAVE_SOME_SNMP
 
 #ifdef HAVE_UCD_SNMP
@@ -186,20 +214,17 @@ get_compiled_version_info(GString *str)
 #else /* no SNMP library */
 	g_string_append(str, "without UCD-SNMP or Net-SNMP");
 #endif /* HAVE_SOME_SNMP */
-	g_string_append(str, ",");
-	do_word_wrap(str, break_point);
 
-	g_string_append(str, " ");
-	break_point = str->len - 1;
+        /* ADNS */
+	g_string_append(str, ", ");
 #ifdef HAVE_GNU_ADNS
 	g_string_append(str, "with ADNS");
 #else
 	g_string_append(str, "without ADNS");
 #endif /* HAVE_GNU_ADNS */
-	g_string_append(str, ",");
 
-	g_string_append(str, " ");
-	break_point = str->len - 1;
+        /* LUA */
+	g_string_append(str, ", ");
 #ifdef HAVE_LUA
 	g_string_append(str, "with ");
 	g_string_append(str, LUA_VERSION);
@@ -207,18 +232,35 @@ get_compiled_version_info(GString *str)
 	g_string_append(str, "without Lua");
 #endif /* HAVE_LUA */
 
-	g_string_append(str, ".");
-	do_word_wrap(str, break_point);
+        /* GnuTLS */
+	g_string_append(str, ", ");
+#ifdef HAVE_LIBGNUTLS
+	g_string_append(str, "with GnuTLS " LIBGNUTLS_VERSION);
+#else
+	g_string_append(str, "without GnuTLS");
+#endif /* HAVE_LIBGNUTLS */
 
-#ifndef HAVE_LIBPCRE
-	break_point = str->len - 1;
-	g_string_append(str,
-			"\nNOTE: this build doesn't support the \"matches\" operator for Wireshark filter"
-			"\nsyntax.");
-	do_word_wrap(str, break_point);
-#endif	/* HAVE_LIBPCRE */
+        /* Gcrypt */
+	g_string_append(str, ", ");
+#ifdef HAVE_LIBGCRYPT
+	g_string_append(str, "with Gcrypt " GCRYPT_VERSION);
+#else
+	g_string_append(str, "without Gcrypt");
+#endif /* HAVE_LIBGCRYPT */
 
-	end_string(str);
+        /* Kerberos */
+        /* XXX - I don't see how to get the version number, at least for KfW */
+	g_string_append(str, ", ");
+#ifdef HAVE_KERBEROS
+#ifdef HAVE_MIT_KERBEROS
+	g_string_append(str, "with MIT Kerberos");
+#else
+        /* HAVE_HEIMDAL_KERBEROS */
+	g_string_append(str, "with Heimdal Kerberos");
+#endif
+#else
+	g_string_append(str, "without Kerberos");
+#endif /* HAVE_KERBEROS */
 }
 
 /*
@@ -226,7 +268,7 @@ get_compiled_version_info(GString *str)
  * them to the specified GString.
  */
 void
-get_runtime_version_info(GString *str)
+get_runtime_version_info(GString *str, void (*additional_info)(GString *))
 {
 #if defined(_WIN32)
 	OSVERSIONINFO info;
@@ -234,9 +276,8 @@ get_runtime_version_info(GString *str)
 	struct utsname name;
 #endif
 
-	get_runtime_pcap_version(str);
-
 	g_string_append(str, "on ");
+
 #if defined(_WIN32)
 	/*
 	 * See
@@ -354,7 +395,7 @@ get_runtime_version_info(GString *str)
 		break;
 	}
 	if (info.szCSDVersion[0] != '\0')
-		g_string_sprintfa(str, " %s", info.szCSDVersion);
+		g_string_sprintfa(str, " %s", utf_16to8(info.szCSDVersion));
 	g_string_sprintfa(str, ", build %lu", info.dwBuildNumber);
 #elif defined(HAVE_SYS_UTSNAME_H)
 	/*
@@ -390,8 +431,82 @@ get_runtime_version_info(GString *str)
 #else
 	g_string_append(str, "an unknown OS");
 #endif
+
+	/* Libpcap */
+	g_string_append(str, ", ");
+	get_runtime_pcap_version(str);
+
+	/* Additional application-dependent information */
+	if (additional_info)
+		(*additional_info)(str);
+
 	g_string_append(str, ".");
 
+	/* Compiler info */
+
+	/*
+	 * See http://predef.sourceforge.net/precomp.html for
+	 * information on various defined strings.
+	 *
+	 * GCC's __VERSION__ is a nice text string for humans to
+	 * read.  The page at predef.sourceforge.net largely
+	 * describes numeric #defines that encode the version;
+	 * if the compiler doesn't also offer a nice printable
+	 * string, we should probably prettify the number somehow.
+	 */
+#if defined(__GNUC__) && defined(__VERSION__)
+	g_string_sprintfa(str, "\n\nBuilt using gcc %s.\n", __VERSION__);
+#elif defined(__HP_aCC)
+	g_string_sprintfa(str, "\n\nBuilt using HP aCC %d.\n", __HP_aCC);
+#elif defined(__xlC__)
+	g_string_sprintfa(str, "\n\nBuilt using IBM XL C %d.%d\n",
+	    (__xlC__ >> 8) & 0xFF, __xlC__ & 0xFF);
+#ifdef __IBMC__
+	if ((__IBMC__ % 10) != 0)
+		g_string_sprintfa(str, " patch %d", __IBMC__ % 10);
+#endif /* __IBMC__ */
+	g_string_sprintfa(str, "\n");
+#elif defined(__INTEL_COMPILER)
+	g_string_sprintfa(str, "\n\nBuilt using Intel C %d.%d",
+	    __INTEL_COMPILER / 100, (__INTEL_COMPILER / 10) % 10);
+	if ((__INTEL_COMPILER % 10) != 0)
+		g_string_sprintfa(str, " patch %d", __INTEL_COMPILER % 10);
+#ifdef __INTEL_COMPILER_BUILD_DATE
+	g_string_sprinta(str, ", compiler built %04d-%02d-%02d",
+	    __INTEL_COMPILER_BUILD_DATE / 10000,
+	    (__INTEL_COMPILER_BUILD_DATE / 100) % 100,
+	    __INTEL_COMPILER_BUILD_DATE % 100);
+#endif /* __INTEL_COMPILER_BUILD_DATE */
+	g_string_sprintfa(str, "\n");
+#elif defined(_MSC_FULL_VER)
+	if (_MSC_FULL_VER > 99999999) {
+		g_string_sprintfa(str, "\n\nBuilt using Microsoft Visual C++ %d.%d",
+		    (_MSC_FULL_VER / 10000000) - 6,
+		    (_MSC_FULL_VER / 100000) % 100);
+		if ((_MSC_FULL_VER % 100000) != 0)
+			g_string_sprintfa(str, " build %d",
+			    _MSC_FULL_VER % 100000);
+	} else {
+		g_string_sprintfa(str, "\n\nBuilt using Microsoft Visual C++ %d.%d",
+		    (_MSC_FULL_VER / 1000000) - 6,
+		    (_MSC_FULL_VER / 10000) % 100);
+		if ((_MSC_FULL_VER % 10000) != 0)
+			g_string_sprintfa(str, " build %d",
+			    _MSC_FULL_VER % 10000);
+	}
+	g_string_sprintfa(str, "\n");
+#elif defined(_MSC_VER)
+	/* _MSC_FULL_VER not defined, but _MSC_VER defined */
+	g_string_sprintfa(str, "\n\nBuilt using Microsoft Visual C++ %d.%d\n",
+	    (_MSC_VER / 100) - 6, _MSC_VER % 100);
+#elif defined(__SUNPRO_C)
+	g_string_sprintfa(str, "\n\nBuilt using Sun C %d.%d",
+	    (__SUNPRO_C >> 8) & 0xF, (__SUNPRO_C >> 4) & 0xF);
+	if ((__SUNPRO_C & 0xF) != 0)
+		g_string_sprintfa(str, " patch %d", __SUNPRO_C & 0xF);
+	g_string_sprintfa(str, "\n");
+#endif
+
 	end_string(str);
 }
 
