@@ -1,6 +1,6 @@
 /* catapult_dct2000.c
  *
- * $Id: catapult_dct2000.c 18458 2006-06-14 05:39:52Z etxrab $
+ * $Id: catapult_dct2000.c 19405 2006-10-03 14:21:07Z martinm $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -43,6 +43,7 @@
 #define MAX_PROTOCOL_NAME          64
 #define MAX_PORT_DIGITS            2
 #define MAX_VARIANT_DIGITS         32
+#define MAX_OUTHDR_NAME            64
 #define AAL_HEADER_CHARS           12
 
 /* TODO:
@@ -105,14 +106,9 @@ static guint8 context_port;
 /* The DCT2000 protocol name of the packet, plus variant number */
 static gchar protocol_name[MAX_PROTOCOL_NAME+1];
 static gchar variant_name[MAX_VARIANT_DIGITS+1];
+static gchar outhdr_name[MAX_OUTHDR_NAME+1];
 
 
-/*************************************************/
-/* Preference state (shared with stub protocol). */
-/* Set to FALSE to get better use out of other   */
-/* wiretap applications (mergecap, editcap)      */
-gboolean catapult_dct2000_board_ports_only = FALSE;
-
 /************************************************************/
 /* Functions called from wiretap                            */
 static gboolean catapult_dct2000_read(wtap *wth, int *err, gchar **err_info,
@@ -137,8 +133,7 @@ static gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
                            long *data_offset,
                            gint *data_chars,
                            packet_direction_t *direction,
-                           int *encap,
-                           gboolean seek_read);
+                           int *encap);
 static int write_stub_header(guchar *frame_buffer, char *timestamp_string,
                       packet_direction_t direction, int encap);
 static guchar hex_from_char(gchar c);
@@ -148,9 +143,8 @@ static void set_pseudo_header_info(wtap *wth,
                                    int pkt_encap,
                                    long file_offset,
                                    union wtap_pseudo_header *pseudo_header,
-                                   gint length,
                                    packet_direction_t direction);
-static void set_aal_info(union wtap_pseudo_header *pseudo_header, gint length,
+static void set_aal_info(union wtap_pseudo_header *pseudo_header,
                          packet_direction_t direction);
 static void set_isdn_info(union wtap_pseudo_header *pseudo_header,
                           packet_direction_t direction);
@@ -319,7 +313,7 @@ gboolean catapult_dct2000_read(wtap *wth, int *err, gchar **err_info _U_,
         if (parse_line(line_length, &seconds, &useconds,
                        &before_time_offset, &after_time_offset,
                        &dollar_offset,
-                       &data_chars, &direction, &encap, FALSE))
+                       &data_chars, &direction, &encap))
         {
             guchar *frame_buffer;
             int n;
@@ -350,11 +344,14 @@ gboolean catapult_dct2000_read(wtap *wth, int *err, gchar **err_info _U_,
 
             /* Get buffer pointer ready */
             buffer_assure_space(wth->frame_buffer,
-                                strlen(context_name)+1 +  /* Context name */
-                                1 +                       /* port */
-                                strlen(protocol_name)+1 + /* Protocol name */
-                                1 +                       /* direction */
-                                1 +                       /* encap */
+                                strlen(context_name)+1 +     /* Context name */
+                                1 +                          /* port */
+                                strlen(timestamp_string)+1 + /* timestamp */
+                                strlen(variant_name)+1 +     /* variant */
+                                strlen(outhdr_name)+1 +      /* outhdr */
+                                strlen(protocol_name)+1 +    /* Protocol name */
+                                1 +                          /* direction */
+                                1 +                          /* encap */
                                 (data_chars/2));
             frame_buffer = buffer_start_ptr(wth->frame_buffer);
 
@@ -397,7 +394,7 @@ gboolean catapult_dct2000_read(wtap *wth, int *err, gchar **err_info _U_,
 
             /* Set pseudo-header if necessary */
             set_pseudo_header_info(wth, encap, this_offset, &wth->pseudo_header,
-                                   data_chars/2, direction);
+                                   direction);
 
             /* OK, we have packet details to return */
             *err = errno;
@@ -444,7 +441,7 @@ catapult_dct2000_seek_read(wtap *wth, long seek_off,
     if (parse_line(length, &seconds, &useconds,
                    &before_time_offset, &after_time_offset,
                    &dollar_offset,
-                   &data_chars, &direction, &encap, TRUE))
+                   &data_chars, &direction, &encap))
     {
         int n;
         int stub_offset = 0;
@@ -457,7 +454,7 @@ catapult_dct2000_seek_read(wtap *wth, long seek_off,
 
         /*********************/
         /* Write stub header */
-        stub_offset = write_stub_header((char*)pd, timestamp_string,
+        stub_offset = write_stub_header((guchar*)pd, timestamp_string,
                                         direction, encap);
 
 
@@ -470,7 +467,7 @@ catapult_dct2000_seek_read(wtap *wth, long seek_off,
         }
 
         /* Set packet pseudo-header if necessary */
-        set_pseudo_header_info(wth, encap, seek_off, pseudo_header, data_chars/2, direction);
+        set_pseudo_header_info(wth, encap, seek_off, pseudo_header, direction);
 
         *err = errno = 0;
         return TRUE;
@@ -529,8 +526,8 @@ void catapult_dct2000_close(wtap *wth)
 gboolean catapult_dct2000_dump_open(wtap_dumper *wdh, gboolean cant_seek _U_, int *err _U_)
 {
     /* Fill in other dump callbacks */
-	wdh->subtype_write = catapult_dct2000_dump;
-	wdh->subtype_close = catapult_dct2000_dump_close;
+    wdh->subtype_write = catapult_dct2000_dump;
+    wdh->subtype_close = catapult_dct2000_dump_close;
 
     return TRUE;
 }
@@ -652,6 +649,10 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     for (; pd[n] != '\0'; n++);
     n++;
 
+    /* Outhdr (as string) */
+    for (; pd[n] != '\0'; n++);
+    n++;
+
     /* Direction & encap */
     n += 2;
 
@@ -737,14 +738,15 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
                     long *before_time_offset, long *after_time_offset,
                     long *data_offset, gint *data_chars,
                     packet_direction_t *direction,
-                    int *encap,
-                    gboolean seek_read)
+                    int *encap)
 {
     int  n = 0;
     int  port_digits = 0;
     char port_number_string[MAX_PORT_DIGITS+1];
     int  variant_digits = 0;
+    int  variant = 1;
     int  protocol_chars = 0;
+    int  outhdr_chars = 0;
 
     char seconds_buff[MAX_SECONDS_CHARS+1];
     int  seconds_chars;
@@ -831,6 +833,61 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
     n++;
 
 
+    /* Following the / is the variant number.  No digits indicate 1 */
+    for (variant_digits = 0;
+         (isdigit(linebuff[n])) && (variant_digits <= MAX_VARIANT_DIGITS) && (n+1 < line_length);
+         n++, variant_digits++)
+    {
+        if (!isdigit(linebuff[n]))
+        {
+            return FALSE;
+        }
+        variant_name[variant_digits] = linebuff[n];
+    }
+    if (variant_digits > MAX_VARIANT_DIGITS || (n+1 >= line_length))
+    {
+        return FALSE;
+    }
+    if (variant_digits > 0)
+    {
+        variant_name[variant_digits] = '\0';
+        variant = atoi(variant_name);
+    }
+    else
+    {
+        strcpy(variant_name, "1");
+    }
+
+
+    /* Outheader values may follow */
+    outhdr_name[0] = '\0';
+    if (linebuff[n] == ',')
+    {
+        /* Skip , */
+        n++;
+
+        for (outhdr_chars = 0;
+             (isdigit(linebuff[n]) || linebuff[n] == ',') &&
+             (outhdr_chars <= MAX_OUTHDR_NAME) && (n+1 < line_length);
+             n++, outhdr_chars++)
+        {
+            if (!isdigit(linebuff[n]) && (linebuff[n] != ','))
+            {
+                return FALSE;
+            }
+            outhdr_name[outhdr_chars] = linebuff[n];
+        }
+        if (outhdr_chars > MAX_OUTHDR_NAME || (n+1 >= line_length))
+        {
+            return FALSE;
+        }
+        /* Terminate (possibly empty) string */
+        outhdr_name[outhdr_chars] = '\0';
+    }
+
+
+
+
     /******************************************************************/
     /* Now check whether we know how to use a packet of this protocol */
 
@@ -840,14 +897,23 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
     }
     else
 
-    /* For ATM protocols, we need to read the separate atm headerparse */
+    /* FP may be carried over ATM, which has separate atm header to parse */
     if ((strcmp(protocol_name, "fp") == 0) ||
         (strcmp(protocol_name, "fp_r4") == 0) ||
         (strcmp(protocol_name, "fp_r5") == 0) ||
         (strcmp(protocol_name, "fp_r6") == 0))
     {
-        *encap = WTAP_ENCAP_ATM_PDUS_UNTRUNCATED;
-        atm_header_present = TRUE;
+        if ((variant > 256) && (variant % 256 == 3))
+        {
+            /* FP over udp is contained in IPPrim... */
+            *encap = 0;
+        }
+        else
+        {
+            /* FP over AAL0 or AAL2 */
+            *encap = WTAP_ENCAP_ATM_PDUS_UNTRUNCATED;
+            atm_header_present = TRUE;
+        }
     }
 
     else
@@ -884,45 +950,8 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
     }
     else
     {
-        /* Only reject protocol if reading for the first time and preference
-           setting says board ports only.  This should not fail to read a
-           non board-port protocol on re-reading because the preference setting
-           has since changed...
-        */
-        if (catapult_dct2000_board_ports_only && !seek_read)
-        {
-            return FALSE;
-        }
-        else
-        {
-            /* Not a supported protocol/encap, but should show as raw data anyway */
-            *encap = DCT2000_ENCAP_UNHANDLED;
-        }
-    }
-
-
-    /* Following the / is the variant number.  No digits indicate 1 */
-    for (variant_digits = 0;
-         (isdigit(linebuff[n])) && (variant_digits <= MAX_VARIANT_DIGITS) && (n+1 < line_length);
-         n++, variant_digits++)
-    {
-        if (!isdigit(linebuff[n]))
-        {
-            return FALSE;
-        }
-        variant_name[variant_digits] = linebuff[n];
-    }
-    if (variant_digits > MAX_VARIANT_DIGITS || (n+1 >= line_length))
-    {
-        return FALSE;
-    }
-    if (variant_digits > 0)
-    {
-        variant_name[variant_digits] = '\0';
-    }
-    else
-    {
-        strcpy(variant_name, "1");
+        /* Not a supported board port protocol/encap, but can show as raw data anyway */
+        *encap = DCT2000_ENCAP_UNHANDLED;
     }
 
 
@@ -950,6 +979,13 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
             aal_header_chars[header_chars_seen] = linebuff[n];
         }
 
+        /* Sometimes see strange encoding of cid in last (non-digit) character */
+        if (header_chars_seen == (AAL_HEADER_CHARS-1))
+        {
+            aal_header_chars[AAL_HEADER_CHARS-1] = linebuff[n];
+            header_chars_seen++;
+        }
+
         if (header_chars_seen != AAL_HEADER_CHARS || n >= line_length)
         {
             return FALSE;
@@ -1013,7 +1049,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
         /* Didn't fit in buffer.  Fail rather than use truncated */
         return FALSE;
     }
-    
+
     /* Convert found value into number */
     seconds_buff[seconds_chars] = '\0';
     *seconds = atoi(seconds_buff);
@@ -1106,10 +1142,14 @@ int write_stub_header(guchar *frame_buffer, char *timestamp_string,
     strcpy((char*)&frame_buffer[stub_offset], protocol_name);
     stub_offset += (strlen(protocol_name) + 1);
 
-    /* Protocol variant number */
+    /* Protocol variant number (as string) */
     strcpy((void*)&frame_buffer[stub_offset], variant_name);
     stub_offset += (strlen(variant_name) + 1);
 
+    /* Outhdr */
+    strcpy((char*)&frame_buffer[stub_offset], outhdr_name);
+    stub_offset += (strlen(outhdr_name) + 1);
+
     /* Direction */
     frame_buffer[stub_offset] = direction;
     stub_offset++;
@@ -1117,7 +1157,7 @@ int write_stub_header(guchar *frame_buffer, char *timestamp_string,
     /* Encap */
     frame_buffer[stub_offset] = (guint8)encap;
     stub_offset++;
-    
+
     return stub_offset;
 }
 
@@ -1129,7 +1169,6 @@ void set_pseudo_header_info(wtap *wth,
                             int pkt_encap,
                             long file_offset,
                             union wtap_pseudo_header *pseudo_header,
-                            gint length,
                             packet_direction_t direction)
 {
     pseudo_header->dct2000.seek_off = file_offset;
@@ -1138,7 +1177,7 @@ void set_pseudo_header_info(wtap *wth,
     switch (pkt_encap)
     {
         case WTAP_ENCAP_ATM_PDUS_UNTRUNCATED:
-            set_aal_info(pseudo_header, length, direction);
+            set_aal_info(pseudo_header, direction);
             break;
         case WTAP_ENCAP_ISDN:
             set_isdn_info(pseudo_header, direction);
@@ -1157,8 +1196,7 @@ void set_pseudo_header_info(wtap *wth,
 /*********************************************/
 /* Fill in atm pseudo-header with known info */
 /*********************************************/
-void set_aal_info(union wtap_pseudo_header *pseudo_header, gint length,
-                  packet_direction_t direction)
+void set_aal_info(union wtap_pseudo_header *pseudo_header, packet_direction_t direction)
 {
     /* 'aal_head_chars' has this format (for AAL2 at least):
        Global Flow Control (4 bits) | VPI (8 bits) | VCI (16 bits) |
@@ -1178,7 +1216,7 @@ void set_aal_info(union wtap_pseudo_header *pseudo_header, gint length,
     /* Assume always AAL2 for FP */
     pseudo_header->dct2000.inner_pseudo_header.atm.aal = AAL_2;
 
-    pseudo_header->dct2000.inner_pseudo_header.atm.type = TRAF_UNKNOWN;
+    pseudo_header->dct2000.inner_pseudo_header.atm.type = TRAF_UMTS_FP;
     pseudo_header->dct2000.inner_pseudo_header.atm.subtype = TRAF_ST_UNKNOWN;
 
     /* vpi is 8 bits (2nd & 3rd nibble) */
@@ -1196,9 +1234,20 @@ void set_aal_info(union wtap_pseudo_header *pseudo_header, gint length,
     /* 0 means we don't know how many cells the frame comprises. */
     pseudo_header->dct2000.inner_pseudo_header.atm.cells = 0;
 
-    pseudo_header->dct2000.inner_pseudo_header.atm.aal5t_u2u = 0;
-    pseudo_header->dct2000.inner_pseudo_header.atm.aal5t_len = length;
-    pseudo_header->dct2000.inner_pseudo_header.atm.aal5t_chksum = 0;
+    /* cid is usually last byte.  Unless last char is not hex digit, in which
+       case cid is derived from last char in ascii */
+    if (isalnum(aal_header_chars[11]))
+    {
+        pseudo_header->dct2000.inner_pseudo_header.atm.aal2_cid =
+            ((hex_from_char(aal_header_chars[10]) << 4) |
+              hex_from_char(aal_header_chars[11]));
+    }
+    else
+    {
+        pseudo_header->dct2000.inner_pseudo_header.atm.aal2_cid =
+            (int)aal_header_chars[11] - 48;
+    }
+
 }
 
 
