@@ -1,6 +1,6 @@
 /* libpcap.c
  *
- * $Id: libpcap.c 18940 2006-08-17 20:24:05Z etxrab $
+ * $Id: libpcap.c 19513 2006-10-13 15:18:10Z jmayer $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -410,6 +410,10 @@ static const struct {
         { 181,          WTAP_ENCAP_JUNIPER_CHDLC },
         /* VOIP Frames prepended with meta-information */
         { 183,          WTAP_ENCAP_JUNIPER_VP },
+	/* raw USB packets */
+	{ 186, 		WTAP_ENCAP_USB },
+	/* bluetooth hci frams, like hcidump */
+	{ 187, 		WTAP_ENCAP_BLUETOOTH_H4 },
 
 
 	/*
