@@ -1,6 +1,6 @@
 /* wtap.c
  *
- * $Id: wtap.c 18940 2006-08-17 20:24:05Z etxrab $
+ * $Id: wtap.c 19480 2006-10-10 21:00:35Z sahlberg $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -363,6 +363,9 @@ static const struct encap_type_info {
 
 	/* WTAP_ENCAP_JUNIPER_VP */
 	{ "Juniper Voice PIC", "juniper-vp" },
+
+	/* WTAP_ENCAP_USB */
+	{ "Raw USB packets", "usb" },
 };
 
 /* Name that should be somewhat descriptive. */
