@@ -1,6 +1,6 @@
 /* wtap.h
  *
- * $Id: wtap.h 18940 2006-08-17 20:24:05Z etxrab $
+ * $Id: wtap.h 19480 2006-10-10 21:00:35Z sahlberg $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -182,9 +182,10 @@
 #define WTAP_ENCAP_CATAPULT_DCT2000             89
 #define WTAP_ENCAP_BER                          90
 #define WTAP_ENCAP_JUNIPER_VP                   91
+#define WTAP_ENCAP_USB				92
 
 /* last WTAP_ENCAP_ value + 1 */
-#define WTAP_NUM_ENCAP_TYPES			92
+#define WTAP_NUM_ENCAP_TYPES			93
 
 /* File types that can be read by wiretap.
    We support writing some many of these file types, too, so we
@@ -318,6 +319,7 @@ struct isdn_phdr {
 #define TRAF_FR		5	/* Frame Relay */
 #define TRAF_SPANS	6	/* FORE SPANS */
 #define TRAF_IPSILON	7	/* Ipsilon */
+#define TRAF_UMTS_FP	8	/* UMTS Frame Protocol */
 
 /*
  * Traffic subtypes.
@@ -363,6 +365,7 @@ struct atm_phdr {
 	guint8	subtype;	/* traffic subtype */
 	guint16	vpi;		/* virtual path identifier */
 	guint16	vci;		/* virtual circuit identifier */
+	guint8	aal2_cid;	/* channel id */
 	guint16	channel;	/* link: 0 for DTE->DCE, 1 for DCE->DTE */
 	guint16	cells;		/* number of cells */
 	guint16	aal5t_u2u;	/* user-to-user indicator */
